"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Catalog = exports.HTTPLAN = exports.HTTPDDL = void 0;
const vscode_1 = require("vscode");
const CatalogEntry_1 = require("./CatalogEntry");
const PlanningDomains_1 = require("./PlanningDomains");
const request = require("request");
const path_1 = require("path");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const COMMAND_SHOW_DOMAIN_PROBLEM = 'pddl.planning.domains.show';
exports.HTTPDDL = 'httpddl';
exports.HTTPLAN = 'httplan';
class Catalog {
    constructor(context) {
        const catalogDataProvider = new CatalogDataProvider(context);
        this.treeView = vscode_1.window.createTreeView(Catalog.VIEW, { treeDataProvider: catalogDataProvider, showCollapseAll: true });
        const catalogContentProvider = new CatalogDomainProblemProvider();
        context.subscriptions.push(vscode_1.workspace.registerTextDocumentContentProvider(exports.HTTPDDL, catalogContentProvider));
        context.subscriptions.push(vscode_1.workspace.registerTextDocumentContentProvider(exports.HTTPLAN, new CatalogPlanProvider()));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(COMMAND_SHOW_DOMAIN_PROBLEM, (domainUrl, problemUrl, planUrl) => this.showProblem(domainUrl, problemUrl, planUrl)));
    }
    showProblem(domainUrl, problemUrl, planUrl) {
        try {
            vscode_1.commands.executeCommand('vscode.setEditorLayout', { orientation: 0, groups: [{ size: 0.4 }, { size: 0.3 }, { size: 0.3 }] });
            vscode_1.workspace.openTextDocument(vscode_1.Uri.parse(domainUrl)).then(document => {
                vscode_1.window.showTextDocument(document, vscode_1.ViewColumn.One);
            });
            vscode_1.workspace.openTextDocument(vscode_1.Uri.parse(problemUrl)).then(document => {
                vscode_1.window.showTextDocument(document, vscode_1.ViewColumn.Two);
            });
            vscode_1.workspace.openTextDocument(vscode_1.Uri.parse(planUrl)).then(document => {
                vscode_1.window.showTextDocument(document, { viewColumn: vscode_1.ViewColumn.Three, preview: true });
            });
        }
        catch (ex) {
            console.log(ex);
            vscode_1.window.showWarningMessage("Failed to display domain/problem/plan from planning.domains: " + ex);
        }
    }
}
exports.Catalog = Catalog;
Catalog.VIEW = "pddl.planning.domains";
class CatalogDomainProblemProvider {
    async provideTextDocumentContent(uri, token) {
        if (token.isCancellationRequested) {
            return "Operation canceled.";
        }
        uri = uri.with({ scheme: "http" });
        return new Promise((resolve) => {
            request.get(uri.toString(), (error, _httpResponse, httpBody) => {
                if (error) {
                    resolve(error);
                }
                else {
                    resolve(httpBody);
                }
            });
        });
    }
}
class CatalogPlanProvider {
    async provideTextDocumentContent(uri, token) {
        if (token.isCancellationRequested) {
            return "Operation canceled.";
        }
        uri = decodePlanUri(uri);
        return new Promise((resolve) => {
            request.get(uri.toString(), { json: true }, (error, _httpResponse, httpBody) => {
                if (error) {
                    resolve(error);
                }
                else {
                    resolve(httpBody["result"]["plan"]);
                }
            });
        });
    }
}
class CatalogDataProvider {
    constructor(context) {
        this.context = context;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this._onDidChangeTreeData = new vscode_1.EventEmitter();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.onDidChangeTreeData = this._onDidChangeTreeData.event;
        this.planningDomains = new PlanningDomains_1.PlanningDomains();
    }
    getTreeItem(element) {
        const isCollapsible = element.kind !== CatalogEntry_1.CatalogEntryKind.Problem;
        return {
            label: element.label,
            collapsibleState: isCollapsible ? vscode_1.TreeItemCollapsibleState.Collapsed : void 0,
            command: this.createCommand(element),
            iconPath: this.getIcon(element.kind),
            tooltip: element.tooltip
        };
    }
    getIcon(kind) {
        if (!kind) {
            return undefined;
        }
        else if (kind === CatalogEntry_1.CatalogEntryKind.Domain) {
            return this.context.asAbsolutePath(path_1.join('views', 'overview', 'file_type_pddl.svg'));
        }
        else if (kind === CatalogEntry_1.CatalogEntryKind.Problem) {
            return this.context.asAbsolutePath(path_1.join('views', 'overview', 'file_type_pddl_plan.svg'));
        }
        else {
            return undefined;
        }
    }
    createCommand(element) {
        if (element.kind === CatalogEntry_1.CatalogEntryKind.Problem) {
            const problem = element;
            const domainUrl = vscode_1.Uri.parse(problem.domain_url).with({ scheme: exports.HTTPDDL });
            const problemUrl = vscode_1.Uri.parse(problem.problem_url).with({ scheme: exports.HTTPDDL });
            const planUrl = encodePlanUri(problem);
            return {
                command: COMMAND_SHOW_DOMAIN_PROBLEM,
                arguments: [domainUrl, problemUrl, planUrl],
                title: 'PDDL: Show PDDL domain, problem and plan'
            };
        }
        else {
            return void 0;
        }
    }
    async getChildren(element) {
        if (!element) {
            return this.withProgress("Fetching collections", () => this.planningDomains.getCollections());
        }
        else if (element.kind === CatalogEntry_1.CatalogEntryKind.Collection) {
            const collection = element;
            return this.withProgress("Fetching domains", () => this.planningDomains.getDomains(collection));
        }
        else if (element.kind === CatalogEntry_1.CatalogEntryKind.Domain) {
            const domain = element;
            return this.withProgress("Fetching problems", () => this.planningDomains.getProblems(domain));
        }
        else {
            return [];
        }
    }
    withProgress(message, workload) {
        return vscode_1.window.withProgress({ location: { viewId: Catalog.VIEW }, title: message }, workload);
    }
}
function encodePlanUri(problem) {
    const extension = '.plan';
    let fileName = problem.label.replace('.pddl', extension);
    if (!fileName.endsWith(extension)) {
        fileName += extension;
    }
    return vscode_1.Uri.parse(PlanningDomains_1.PlanningDomains.URL + `plan/${problem.id}/${fileName}`).with({ scheme: exports.HTTPLAN });
}
function decodePlanUri(encodedUri) {
    const lastSlash = encodedUri.path.lastIndexOf('/');
    return encodedUri.with({ path: encodedUri.path.substring(0, lastSlash), scheme: "http" });
}
//# sourceMappingURL=Catalog.js.map