/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanComparer = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const NormalizedPlanDocumentContentProvider_1 = require("./NormalizedPlanDocumentContentProvider");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
/**
 * Delegate for handling requests to run the planner and visualize the plans.
 */
class PlanComparer {
    constructor(pddlWorkspace, pddlConfiguration) {
        this.pddlWorkspace = pddlWorkspace;
        this.normalizedPlanScheme = 'normalized-pddl-plan';
        this.disposables = [];
        this.provider = new NormalizedPlanDocumentContentProvider_1.NormalizedPlanDocumentContentProvider(pddlWorkspace, pddlConfiguration, true);
        this.disposables.push(vscode_1.workspace.registerTextDocumentContentProvider(this.normalizedPlanScheme, this.provider));
        this.disposables.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.plan.compareNormalized", async (rightPlanUri, selectedFiles) => {
            if (selectedFiles.length !== 2) {
                vscode_1.window.showErrorMessage("Hold down the Ctrl/Command key and select two plan files.");
                return;
            }
            const leftFileUri = selectedFiles.find(otherFileUri => otherFileUri.toString() !== rightPlanUri.toString());
            const leftTextDocument = await vscode_1.workspace.openTextDocument(leftFileUri);
            if (workspaceUtils_1.toLanguage(leftTextDocument) !== pddl_workspace_1.PddlLanguage.PLAN) {
                vscode_1.window.showErrorMessage("Select 2 .plan files.");
                return;
            }
            const rightPlan = await vscode_1.workspace.openTextDocument(rightPlanUri);
            const leftPlan = leftTextDocument;
            this.compare(leftPlan, rightPlan);
        }));
        this.disposables.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.plan.normalize", async (uri) => {
            if (!uri && vscode_1.window.activeTextEditor) {
                uri = vscode_1.window.activeTextEditor.document.uri;
            }
            const planDoc = await vscode_1.workspace.openTextDocument(uri);
            if (!workspaceUtils_1.isPlan(planDoc)) {
                vscode_1.window.showErrorMessage("Active document is not a plan.");
            }
            else {
                const normalizedUri = this.subscribeToNormalizedUri(uri);
                vscode_1.window.showTextDocument(normalizedUri, { viewColumn: vscode_1.ViewColumn.Beside });
            }
        }));
    }
    dispose() {
        this.disposables.forEach(d => d.dispose());
    }
    compare(leftPlan, rightPlan) {
        const title = `${vscode_1.workspace.asRelativePath(leftPlan.uri)} ↔ ${vscode_1.workspace.asRelativePath(rightPlan.uri)}`;
        const leftPlanNormalized = this.subscribeToNormalizedUri(leftPlan.uri);
        const rightPlanNormalized = this.subscribeToNormalizedUri(rightPlan.uri);
        vscode_1.commands.executeCommand('vscode.diff', leftPlanNormalized, rightPlanNormalized, title, { viewColumn: vscode_1.ViewColumn.Active });
    }
    subscribeToNormalizedUri(uri) {
        const normalizedPlanUri = uri.with({ scheme: this.normalizedPlanScheme });
        this.disposables.push(vscode_1.workspace.onDidChangeTextDocument(e => {
            if (e.document.uri.toString() === uri.toString()) {
                this.provider.planChanged(normalizedPlanUri);
            }
        }));
        return normalizedPlanUri;
    }
}
exports.PlanComparer = PlanComparer;
//# sourceMappingURL=PlanComparer.js.map