/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoCompletion = void 0;
const vscode_1 = require("vscode");
const ProblemInitDelegate_1 = require("./ProblemInitDelegate");
const OperatorDelegate_1 = require("./OperatorDelegate");
const VariableDelegate_1 = require("./VariableDelegate");
const TypeDelegate_1 = require("./TypeDelegate");
class AutoCompletion {
    constructor(codePddlWorkspace) {
        this.codePddlWorkspace = codePddlWorkspace;
        this.operatorDelegate = new OperatorDelegate_1.OperatorDelegate();
        this.variableDelegate = new VariableDelegate_1.VariableDelegate(codePddlWorkspace);
        this.typeDelegate = new TypeDelegate_1.TypeDelegate(codePddlWorkspace.pddlWorkspace);
    }
    async provideCompletionItems(document, position, token, context) {
        if (token.isCancellationRequested) {
            return [];
        }
        const completionCollector = await new CompletionCollector(this.codePddlWorkspace, document, position, context, this.operatorDelegate, this.variableDelegate, this.typeDelegate).initialize();
        return completionCollector.getCompletions();
    }
}
exports.AutoCompletion = AutoCompletion;
class CompletionCollector {
    constructor(codePddlWorkspace, document, position, context, operatorDelegate, variableDelegate, typeDelegate) {
        this.codePddlWorkspace = codePddlWorkspace;
        this.document = document;
        this.position = position;
        this.context = context;
        this.operatorDelegate = operatorDelegate;
        this.variableDelegate = variableDelegate;
        this.typeDelegate = typeDelegate;
        this.completions = [];
        this.lineText = document.lineAt(position.line).text;
        this.leadingText = this.lineText.substring(0, position.character);
    }
    async initialize() {
        if (this.leadingText.includes(';')) {
            return this;
        } // do not auto-complete in comment text
        const activeFileInfo = await this.codePddlWorkspace.upsertFile(this.document);
        if (!activeFileInfo) {
            return this;
        }
        if (activeFileInfo.isProblem()) {
            const problemFileInfo = activeFileInfo;
            this.createProblemCompletionItems(problemFileInfo);
        }
        if (this.leadingText.length > 0 && this.leadingText.endsWith('(')) {
            this.pushAll(this.operatorDelegate.getOperatorItems());
            this.pushAll(this.variableDelegate.getVariableItems(activeFileInfo));
        }
        else if (this.leadingText.endsWith(' -')) {
            this.pushAll(this.typeDelegate.getTypeItems(activeFileInfo));
        }
        return this;
    }
    getCompletions() {
        return this.completions;
    }
    createProblemCompletionItems(problemFileInfo) {
        var _a;
        const folder = this.codePddlWorkspace.pddlWorkspace.getFolderOf(problemFileInfo);
        // find domain files in the same folder that match the problem's domain name
        const domainFiles = (_a = folder === null || folder === void 0 ? void 0 : folder.getDomainFilesFor(problemFileInfo)) !== null && _a !== void 0 ? _a : [];
        if (this.isInInit()) {
            new ProblemInitDelegate_1.ProblemInitDelegate(this.completions, this.context).createProblemInitCompletionItems(problemFileInfo, domainFiles);
        }
    }
    pushAll(items) {
        this.completions.push(...items);
    }
    isTriggeredByBracket() {
        return this.leadingText.length > 0 && this.leadingText.endsWith('(');
    }
    isInInit() {
        const startPosition = new vscode_1.Position(0, 0);
        const endPosition = new vscode_1.Position(this.document.lineCount, 10000);
        return this.document.getText(new vscode_1.Range(startPosition, this.position)).includes('(:init')
            && this.document.getText(new vscode_1.Range(this.position, endPosition)).includes('(:goal');
    }
}
//# sourceMappingURL=AutoCompletion.js.map