/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContinuousEffectCompletionItemProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const AbstractCompletionItemProvider_1 = require("./AbstractCompletionItemProvider");
const Delegate_1 = require("./Delegate");
const DomainCompletionItemProvider_1 = require("./DomainCompletionItemProvider");
class ContinuousEffectCompletionItemProvider extends AbstractCompletionItemProvider_1.AbstractCompletionItemProvider {
    constructor() {
        super();
        const continuousEffectHint = 'Use this in `:durative-action`\'s `:effect` block. Do not use it inside `(at start ...)` or `(at end ...)` effect. Example usage:';
        const continuousEffectExample = `(:durative-action
...
:effect (and 
    (at start ...)
    (increase (function1) (* #t 2.0))
    (decrease (function2) (* #t 3.0))
    (at end ...)
)`;
        const requiresContinuousEffects = DomainCompletionItemProvider_1.requires([':continuous-effects']);
        this.addSuggestionDocumentation('increase', 'Continuous numeric increase effect', new vscode_1.MarkdownString('For example to increment a function value by *twice* the amount of time that elapsed since action started, use:')
            .appendCodeblock('(increase (function1) (* $t 2.0))', pddl_workspace_1.PDDL)
            .appendMarkdown(continuousEffectHint)
            .appendCodeblock(continuousEffectExample)
            .appendMarkdown(requiresContinuousEffects), vscode_1.CompletionItemKind.Method);
        this.addSuggestionDocumentation('decrease', 'Continuous numeric decrease effect', new vscode_1.MarkdownString('For example to decrement a function value by *twice* the amount of time that elapsed since action started, use:')
            .appendCodeblock('(decrease (function1) (* $t 2.0))', pddl_workspace_1.PDDL)
            .appendMarkdown(continuousEffectHint)
            .appendCodeblock(continuousEffectExample)
            .appendMarkdown(requiresContinuousEffects), vscode_1.CompletionItemKind.Method);
        this.addSuggestionDocumentation('forall', 'For all duration-dependent effect', new vscode_1.MarkdownString('Effect that shall be applied to all objects of specified type. For example:')
            .appendCodeblock('(forall (?p - product) (increase (stock ?p) (* #t 2.0))))', pddl_workspace_1.PDDL), vscode_1.CompletionItemKind.TypeParameter);
    }
    static inside(currentNode) {
        return pddl_workspace_2.ModelHierarchy.isInsideEffect(currentNode)
            && (pddl_workspace_2.ModelHierarchy.isInsideProcess(currentNode)
                || pddl_workspace_2.ModelHierarchy.isInsideDurativeActionUnqualifiedEffect(currentNode));
    }
    provide(domainInfo, context, range) {
        if (context.triggerKind !== vscode_1.CompletionTriggerKind.Invoke && context.triggerCharacter !== '(') {
            return [];
        }
        const functions = domainInfo.getFunctions();
        const functionNamesCsv = Delegate_1.Delegate.toTypeLessNamesCsv(functions);
        return [
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("increase", context.triggerCharacter, '('), "(increase (" + DomainCompletionItemProvider_1.toSelection(1, functionNamesCsv, "new_function") + ") (* #t ${2:1.0}))$0", range, context, 2),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("decrease", context.triggerCharacter, '('), "(decrease (" + DomainCompletionItemProvider_1.toSelection(1, functionNamesCsv, "new_function") + ") (* #t ${2:1.0}))$0", range, context, 3),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("forall", context.triggerCharacter, '('), "(forall ($1) $2)$0", range, context, 4),
        ];
    }
}
exports.ContinuousEffectCompletionItemProvider = ContinuousEffectCompletionItemProvider;
//# sourceMappingURL=ContinuousEffectCompletionItemProvider.js.map