/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemCompletionItemProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const AbstractCompletionItemProvider_1 = require("./AbstractCompletionItemProvider");
const utils_1 = require("../utils");
class ProblemCompletionItemProvider extends AbstractCompletionItemProvider_1.AbstractCompletionItemProvider {
    constructor() {
        super();
        this.addSuggestionDocumentation(':domain', 'Domain name', 'Name of the domain this problem is designed for');
        this.addRequirementsDocumentation();
        this.addSuggestionDocumentation(':objects', 'Objects', new vscode_1.MarkdownString('List of object names, for example:').appendCodeblock('car1 car2 car3 - vehicle'));
        this.addSuggestionDocumentation(':init', 'Initial state', 'Facts that are true in the initial state, initial values of numeric functions.');
        this.addSuggestionDocumentation(':goal', 'Goal state condition', '');
        this.addConstraintsDocumentation();
        this.addSuggestionDocumentation(':metric', 'Metric', 'Function that the planner will either minimize or maximize.');
    }
    async provide(document, problemInfo, position, context) {
        let currentNode = problemInfo.syntaxTree.getNodeAt(document.offsetAt(position));
        if (currentNode.isType(pddl_workspace_1.parser.PddlTokenType.Document) ||
            (currentNode.isType(pddl_workspace_1.parser.PddlTokenType.Whitespace) || currentNode.isType(pddl_workspace_1.parser.PddlTokenType.Comment))
                && currentNode.getParent() && currentNode.getParent().isType(pddl_workspace_1.parser.PddlTokenType.Document)) {
            const items = [];
            {
                const item = new vscode_1.CompletionItem(";;!pre-parsing:command", vscode_1.CompletionItemKind.Snippet);
                item.insertText = new vscode_1.SnippetString(";;!pre-parsing:{type: \"command\", command: \"${1:program}\", args: [${2:\"data.json\", \"1234\"}]}\n$0");
                item.detail = "Pre-parsing problem file transformation via a shell command.";
                items.push(item);
            }
            {
                const item = new vscode_1.CompletionItem(";;!pre-parsing:python", vscode_1.CompletionItemKind.Snippet);
                item.insertText = new vscode_1.SnippetString(";;!pre-parsing:{type: \"python\", command: \"${1:your_script.py}\", args: [${2:\"data.json\", \"1234\"}]}\n$0");
                item.detail = "Pre-parsing problem file transformation via a python script.";
                items.push(item);
            }
            {
                const item = new vscode_1.CompletionItem(";;!pre-parsing:", vscode_1.CompletionItemKind.Snippet);
                item.insertText = new vscode_1.SnippetString(";;!pre-parsing:{type: \"${1|nunjucks,jinja2|}\", data: \"${2:case1.json}\"}\n$0");
                item.detail = "Pre-parsing problem file transformation via Nunjucks or Jinja2.";
                items.push(item);
            }
            return items;
        }
        else if (this.insideDefine(problemInfo, currentNode, context)) {
            // inside the 'define' bracket
            if (currentNode.isType(pddl_workspace_1.parser.PddlTokenType.Comment)) {
                return [];
            }
            if (context.triggerCharacter) {
                currentNode = currentNode.expand();
            }
            const supportedSectionsHere = pddl_workspace_1.parser.PddlStructure.getSupportedSectionsHere(currentNode, currentNode, pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, pddl_workspace_1.parser.PddlStructure.PDDL_PROBLEM_SECTIONS, []);
            const range = context.triggerCharacter && ['(', ':'].includes(context.triggerCharacter)
                ? utils_1.nodeToRange(document, currentNode) : null;
            const suggestions = supportedSectionsHere
                .map(s => AbstractCompletionItemProvider_1.Suggestion.from(s, context.triggerCharacter, '('))
                .filter(s => !!s).map(s => s);
            return suggestions
                .map((suggestion, index) => this.createDefineCompletionItem(currentNode, suggestion, range, context, index))
                .filter(item => !!item).map(item => item); // filter out nulls
        }
        else if (this.insideRequirements(problemInfo, currentNode, context)) {
            return this.createRequirementsCompletionItems(document, currentNode, context);
        }
        return [];
    }
    createDefineCompletionItem(_currentNode, suggestion, range, context, index) {
        switch (suggestion.sectionName) {
            case pddl_workspace_1.parser.PddlStructure.PROBLEM:
                return this.createSnippetCompletionItem(suggestion, "(problem ${1:problem_name})", range, context, index);
            case pddl_workspace_1.parser.PddlStructure.PROBLEM_DOMAIN:
                return this.createSnippetCompletionItem(suggestion, "(:domain ${1:domain_name})", range, context, index);
            case pddl_workspace_1.parser.PddlStructure.REQUIREMENTS:
                return this.createSnippetCompletionItem(suggestion, "(:requirements $0)", range, context, index);
            case pddl_workspace_1.parser.PddlStructure.OBJECTS:
            case pddl_workspace_1.parser.PddlStructure.INIT:
            case pddl_workspace_1.parser.PddlStructure.CONSTRAINTS:
                return this.createSnippetCompletionItem(suggestion, "(" + suggestion.sectionName + " (and\n\t$0\n))", range, context, index);
            case pddl_workspace_1.parser.PddlStructure.GOAL:
                return this.createSnippetCompletionItem(suggestion, "(" + suggestion.sectionName + " (and\n\t$0\n))", range, context, index);
            case pddl_workspace_1.parser.PddlStructure.METRIC:
                return this.createSnippetCompletionItem(suggestion, "(:metric ${3|minimize,maximize|} (???))", range, context, index);
            default:
                return null;
        }
    }
}
exports.ProblemCompletionItemProvider = ProblemCompletionItemProvider;
//# sourceMappingURL=ProblemCompletionItemProvider.js.map