/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2020. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlannersConfiguration = exports.STATUS_BAR_PRIORITY = exports.PlannerConfigurationScope = exports.EXECUTION_TARGET = exports.PLANNER_OUTPUT_TARGET_SEARCH_DEBUGGER = exports.DEF_PLANNER_OUTPUT_TARGET = exports.PDDL_CONFIGURE_PLANNER_OUTPUT_TARGET = exports.CONFIGURE_PLANNER_OUTPUT_TARGET = exports.PDDL_SELECT_PLANNER = exports.CONF_SELECTED_PLANNER = exports.CONF_PLANNERS = void 0;
const os = require("os");
const path = require("path");
const jsonc_parser_1 = require("jsonc-parser");
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const plannerConfigurations_1 = require("./plannerConfigurations");
const configuration_1 = require("./configuration");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const utils_1 = require("../utils");
exports.CONF_PLANNERS = "planners";
exports.CONF_SELECTED_PLANNER = "selectedPlanner";
exports.PDDL_SELECT_PLANNER = configuration_1.CONF_PDDL + '.' + 'selectPlanner';
exports.CONFIGURE_PLANNER_OUTPUT_TARGET = 'configureTarget';
exports.PDDL_CONFIGURE_PLANNER_OUTPUT_TARGET = configuration_1.CONF_PDDL + '.' + exports.CONFIGURE_PLANNER_OUTPUT_TARGET;
exports.DEF_PLANNER_OUTPUT_TARGET = "Output window";
exports.PLANNER_OUTPUT_TARGET_SEARCH_DEBUGGER = "Search debugger";
const PDDL_GET_SELECTED_PLANNER = configuration_1.CONF_PDDL + '.' + exports.CONF_SELECTED_PLANNER;
const PDDL_CONFIGURE_PLANNER = configuration_1.CONF_PDDL + '.' + 'configurePlanner';
const PDDL_DELETE_PLANNER = configuration_1.CONF_PDDL + '.' + 'deletePlanner';
const PDDL_JSON_SETTINGS = configuration_1.CONF_PDDL + '.' + 'plannersJsonSettings';
const PDDL_ADD_PLANNER = configuration_1.CONF_PDDL + '.' + 'addPlanner';
exports.EXECUTION_TARGET = "executionTarget";
var PlannerConfigurationScope;
(function (PlannerConfigurationScope) {
    PlannerConfigurationScope[PlannerConfigurationScope["Default"] = 0] = "Default";
    PlannerConfigurationScope[PlannerConfigurationScope["Extension"] = 1] = "Extension";
    PlannerConfigurationScope[PlannerConfigurationScope["User"] = 2] = "User";
    PlannerConfigurationScope[PlannerConfigurationScope["Workspace"] = 3] = "Workspace";
    PlannerConfigurationScope[PlannerConfigurationScope["WorkspaceFolder"] = 4] = "WorkspaceFolder";
})(PlannerConfigurationScope = exports.PlannerConfigurationScope || (exports.PlannerConfigurationScope = {}));
/** Base priority for all the status bar items */
exports.STATUS_BAR_PRIORITY = 10;
class PlannersConfiguration {
    constructor(context, pddlWorkspace) {
        var _a;
        this.context = context;
        this.pddlWorkspace = pddlWorkspace;
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_ADD_PLANNER, () => this.createPlannerConfiguration().catch(utils_1.showError)));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_GET_SELECTED_PLANNER, () => { var _a; return (_a = this.getSelectedPlanner()) === null || _a === void 0 ? void 0 : _a.configuration; }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.PDDL_SELECT_PLANNER, async () => { var _a; return (_a = (await this.selectPlanner())) === null || _a === void 0 ? void 0 : _a.configuration; }));
        if (vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get('showPlannerInStatusBar', true)) {
            this.plannerSelector = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Left, exports.STATUS_BAR_PRIORITY);
            this.plannerSelector.command = exports.PDDL_SELECT_PLANNER;
            context.subscriptions.push(this.plannerSelector);
            this.plannerOutputSelector = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Left, exports.STATUS_BAR_PRIORITY);
            this.plannerOutputSelector.command = exports.PDDL_CONFIGURE_PLANNER_OUTPUT_TARGET;
            context.subscriptions.push(this.plannerOutputSelector);
            setTimeout(() => this.refreshStatusBar(), 3000); // delayed initialization
        }
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_CONFIGURE_PLANNER, (plannerConfiguration) => {
            if (!plannerConfiguration) {
                plannerConfiguration = this.getSelectedPlanner();
            }
            if (!plannerConfiguration) {
                utils_1.showError(new Error('No planner selected.'));
                return;
            }
            this.configureAndSavePlanner(plannerConfiguration).catch(utils_1.showError);
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(configuration_1.CONF_PDDL + '.showPlannerConfiguration', (plannerConfiguration) => {
            if (plannerConfiguration) {
                this.openPlannerSettingsInJson(plannerConfiguration);
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_DELETE_PLANNER, async (plannerConfiguration) => {
            if (!plannerConfiguration) {
                plannerConfiguration = this.getSelectedPlanner();
            }
            if (!plannerConfiguration) {
                utils_1.showError(new Error('No planner selected.'));
                return;
            }
            const yes = 'Yes';
            const answer = await vscode_1.window.showWarningMessage(`Delete configuration '${plannerConfiguration.configuration.title}'?`, { modal: true }, yes, 'No');
            if (answer === yes) {
                this.deletePlanner(plannerConfiguration).catch(utils_1.showError);
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_JSON_SETTINGS, () => {
            this.openSettingsInJson();
        }));
        // migrate legacy configuration in all workspaces
        ((_a = vscode_1.workspace.workspaceFolders) !== null && _a !== void 0 ? _a : [undefined]).forEach((wf) => this.migrateLegacyConfiguration(wf).catch(utils_1.showError));
    }
    async configureAndSavePlanner(plannerConfiguration) {
        if (!plannerConfiguration.configuration.canConfigure) {
            throw new Error(`Planner configuration ${plannerConfiguration.configuration.title} is not configurable.`);
        }
        const plannerProvider = this.pddlWorkspace.getPlannerRegistrar()
            .getPlannerProvider(new pddl_workspace_1.planner.PlannerKind(plannerConfiguration.configuration.kind));
        if (!plannerProvider) {
            new Error(`Planner provider for '${plannerConfiguration.configuration.kind}' is not currently available. Are you missing an extension?`);
        }
        const newPlannerConfiguration = await (plannerProvider === null || plannerProvider === void 0 ? void 0 : plannerProvider.configurePlanner(plannerConfiguration.configuration));
        if (!newPlannerConfiguration) {
            return undefined;
        }
        const workspaceFolder = this.toWorkspaceFolder(plannerConfiguration);
        return await this.savePlannerConfiguration(plannerConfiguration.index, plannerConfiguration.scope, newPlannerConfiguration, workspaceFolder);
    }
    async migrateLegacyConfiguration(workspaceFolder) {
        const config = vscode_1.workspace.getConfiguration(configuration_1.PDDL_PLANNER, workspaceFolder);
        const legacyPlannerInspect = config.inspect(configuration_1.EXECUTABLE_OR_SERVICE);
        const legacySyntaxInspect = config.inspect(configuration_1.EXECUTABLE_OPTIONS);
        if (legacyPlannerInspect === null || legacyPlannerInspect === void 0 ? void 0 : legacyPlannerInspect.globalValue) {
            await this.migrateLegacyConfigurationInTarget(legacyPlannerInspect.globalValue, legacySyntaxInspect === null || legacySyntaxInspect === void 0 ? void 0 : legacySyntaxInspect.globalValue, PlannerConfigurationScope.User);
        }
        if (legacyPlannerInspect === null || legacyPlannerInspect === void 0 ? void 0 : legacyPlannerInspect.workspaceValue) {
            await this.migrateLegacyConfigurationInTarget(legacyPlannerInspect.workspaceValue, legacySyntaxInspect === null || legacySyntaxInspect === void 0 ? void 0 : legacySyntaxInspect.workspaceValue, PlannerConfigurationScope.Workspace);
        }
        if (legacyPlannerInspect === null || legacyPlannerInspect === void 0 ? void 0 : legacyPlannerInspect.workspaceFolderValue) {
            await this.migrateLegacyConfigurationInTarget(legacyPlannerInspect.workspaceFolderValue, legacySyntaxInspect === null || legacySyntaxInspect === void 0 ? void 0 : legacySyntaxInspect.workspaceFolderValue, PlannerConfigurationScope.WorkspaceFolder, workspaceFolder);
        }
    }
    async migrateLegacyConfigurationInTarget(legacyPlanner, legacySyntax, scope, workspaceFolder) {
        const config = vscode_1.workspace.getConfiguration(configuration_1.PDDL_PLANNER, workspaceFolder);
        const migratedPlanner = utils_1.isHttp(legacyPlanner)
            ? legacyPlanner.endsWith('/solve')
                ? new plannerConfigurations_1.SolveServicePlannerProvider([]).createPlannerConfiguration(legacyPlanner)
                : new plannerConfigurations_1.RequestServicePlannerProvider([]).createPlannerConfiguration(legacyPlanner)
            : new plannerConfigurations_1.CommandPlannerProvider().createPlannerConfiguration(legacyPlanner, legacySyntax);
        const target = this.toConfigurationTarget(scope);
        const newPlannerConfig = await this.addPlannerConfiguration(scope, migratedPlanner, workspaceFolder);
        await config.update(configuration_1.EXECUTABLE_OR_SERVICE, undefined, target);
        await config.update(configuration_1.EXECUTABLE_OPTIONS, undefined, target);
        console.log(`Migrated ${legacyPlanner} to ${newPlannerConfig.scope.toString()}:${newPlannerConfig.configuration.title}`);
    }
    refreshStatusBar() {
        var _a, _b, _c;
        if (!this.plannerSelector) {
            return;
        }
        else {
            const selectedPlanner = this.getSelectedPlanner();
            const activePlannerTitle = (_a = selectedPlanner === null || selectedPlanner === void 0 ? void 0 : selectedPlanner.configuration.title) !== null && _a !== void 0 ? _a : '$(warning)';
            this.plannerSelector.text = `$(circuit-board) ${activePlannerTitle}`;
            this.plannerSelector.tooltip = (_c = (_b = selectedPlanner === null || selectedPlanner === void 0 ? void 0 : selectedPlanner.configuration.path) !== null && _b !== void 0 ? _b : selectedPlanner === null || selectedPlanner === void 0 ? void 0 : selectedPlanner.configuration.url) !== null && _c !== void 0 ? _c : 'Click here to select a planning engine.';
            this.plannerSelector.color = !selectedPlanner ? "yellow" : undefined;
            this.plannerSelector.show();
        }
        if (!this.plannerOutputSelector) {
            return;
        }
        else {
            const target = vscode_1.workspace.getConfiguration(configuration_1.PDDL_PLANNER).get(exports.EXECUTION_TARGET, exports.DEF_PLANNER_OUTPUT_TARGET);
            const targetIcon = this.createTargetIcon(target);
            this.plannerOutputSelector.text = `$(arrow-small-right)$(${targetIcon})`;
            this.plannerOutputSelector.tooltip = 'Planner output is being re-directed to the ' + target;
            this.plannerOutputSelector.show();
        }
    }
    createTargetIcon(target) {
        switch (target) {
            case exports.DEF_PLANNER_OUTPUT_TARGET:
                return 'note';
            case "Terminal":
                return 'terminal';
            case exports.PLANNER_OUTPUT_TARGET_SEARCH_DEBUGGER:
                return 'type-hierarchy';
            default:
                return 'warning';
        }
    }
    registerBuiltInPlannerProviders() {
        [
            new plannerConfigurations_1.ExecutablePlannerProvider(),
            new plannerConfigurations_1.CommandPlannerProvider(),
            new plannerConfigurations_1.SolveServicePlannerProvider(this.context.subscriptions),
            new plannerConfigurations_1.RequestServicePlannerProvider(this.context.subscriptions),
            new plannerConfigurations_1.Popf(),
            new plannerConfigurations_1.Lpg(),
            new plannerConfigurations_1.JavaPlannerProvider(),
        ].forEach(provider => this.pddlWorkspace.getPlannerRegistrar().registerPlannerProvider(provider.kind, provider));
    }
    async selectPlanner() {
        const planners = this.getPlanners();
        const items = [
            NewPlannerQuickPickItem.CREATE_NEW,
            ConfigurePlannerQuickPickItem.CONFIGURE,
            ...planners.map(plannerConfig => new ExistingPlannerQuickPickItem(plannerConfig))
        ];
        const selectedItem = await vscode_1.window.showQuickPick(items, { placeHolder: 'Select planner ...' });
        if (!selectedItem) {
            return undefined;
        }
        if (selectedItem === NewPlannerQuickPickItem.CREATE_NEW) {
            vscode_1.commands.executeCommand(PDDL_ADD_PLANNER);
            return undefined;
        }
        else if (selectedItem === ConfigurePlannerQuickPickItem.CONFIGURE) {
            vscode_1.commands.executeCommand('pddl.showOverview');
            return undefined;
        }
        else if (selectedItem instanceof ExistingPlannerQuickPickItem) {
            await this.setSelectedPlanner(selectedItem.planner);
            return selectedItem.planner;
        }
        else {
            throw new Error(`Unexpected item: ${selectedItem}`);
        }
    }
    /**
     * Sets selected planner (and clears all lower-level selections).
     * @param selectedPlanner selected planner
     * @param workspaceFolder workspace folder context of the possible previous workspaceFolder-level selection (which shall now be removed)
     */
    async setSelectedPlanner(selectedPlanner, previousWorkspaceFolder) {
        var _a;
        const workspaceFolder = previousWorkspaceFolder !== null && previousWorkspaceFolder !== void 0 ? previousWorkspaceFolder : this.toWorkspaceFolder(selectedPlanner);
        const pddlConfig = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL, workspaceFolder);
        // first, clear the lower-level selections to make the selection on the `selectedPlanner.scope` effective
        switch (selectedPlanner.scope) {
            case PlannerConfigurationScope.Default:
            case PlannerConfigurationScope.User:
            case PlannerConfigurationScope.Extension:
                // clear workspace-level selection (if any)
                await pddlConfig.update(exports.CONF_SELECTED_PLANNER, undefined, this.toConfigurationTarget(PlannerConfigurationScope.Workspace));
            // falling through intentionally
            case PlannerConfigurationScope.Workspace:
                // clear workspaceFolder-level selection (if any)
                if (workspaceFolder) {
                    await pddlConfig.update(exports.CONF_SELECTED_PLANNER, undefined, this.toConfigurationTarget(PlannerConfigurationScope.WorkspaceFolder));
                }
        }
        await pddlConfig
            .update(exports.CONF_SELECTED_PLANNER, selectedPlanner.configuration.title, (_a = this.toConfigurationTarget(selectedPlanner.scope)) !== null && _a !== void 0 ? _a : vscode_1.ConfigurationTarget.Global);
    }
    toWorkspaceFolder(selectedPlanner) {
        return selectedPlanner.workspaceFolder ? vscode_1.workspace.getWorkspaceFolder(vscode_1.Uri.parse(selectedPlanner.workspaceFolder)) : undefined;
    }
    getPlannersPerScope(scope, workingFolder) {
        var _a, _b, _c, _d;
        const plannersConf = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL, workingFolder)
            .inspect(exports.CONF_PLANNERS);
        switch (scope) {
            case PlannerConfigurationScope.Default:
                return (_a = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.defaultValue) !== null && _a !== void 0 ? _a : [];
            case PlannerConfigurationScope.User:
                return (_b = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.globalValue) !== null && _b !== void 0 ? _b : [];
            case PlannerConfigurationScope.Workspace:
                return (_c = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.workspaceValue) !== null && _c !== void 0 ? _c : [];
            case PlannerConfigurationScope.WorkspaceFolder:
                return (_d = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.workspaceFolderValue) !== null && _d !== void 0 ? _d : [];
            default:
                console.warn(`Unexpected scope in getPlannersPerScope(): ${scope}`);
                return [];
        }
    }
    getPlanners(workingFolder) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        const plannersConf = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL, workingFolder)
            .inspect(exports.CONF_PLANNERS);
        const scopedConfigs = [
            (_b = (_a = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.workspaceFolderValue) === null || _a === void 0 ? void 0 : _a.map((conf, index) => this.toScopedConfiguration(conf, index, PlannerConfigurationScope.WorkspaceFolder, workingFolder))) !== null && _b !== void 0 ? _b : [],
            (_d = (_c = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.workspaceValue) === null || _c === void 0 ? void 0 : _c.map((conf, index) => this.toScopedConfiguration(conf, index, PlannerConfigurationScope.Workspace))) !== null && _d !== void 0 ? _d : [],
            (_f = (_e = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.globalValue) === null || _e === void 0 ? void 0 : _e.map((conf, index) => this.toScopedConfiguration(conf, index, PlannerConfigurationScope.User))) !== null && _f !== void 0 ? _f : [],
            (_h = (_g = plannersConf === null || plannersConf === void 0 ? void 0 : plannersConf.defaultValue) === null || _g === void 0 ? void 0 : _g.map((conf, index) => this.toScopedConfiguration(conf, index, PlannerConfigurationScope.Default))) !== null && _h !== void 0 ? _h : []
        ];
        // flatten the structure, but ignore the workspace-level configurations that repeat the workspaceFolder-level configs
        const allConfigs = scopedConfigs.reduce((cumPlanners, scopePlanners) => {
            const stringifiedCumPlanners = cumPlanners.map(spc => JSON.stringify(spc.configuration));
            const unique = scopePlanners.filter(spc => !stringifiedCumPlanners.includes(JSON.stringify(spc.configuration)));
            return cumPlanners.concat(unique);
        }, []);
        return [...new Set(allConfigs).values()];
    }
    toScopedConfiguration(config, index, scope, workspaceFolder) {
        if (scope === PlannerConfigurationScope.WorkspaceFolder) {
            if (!workspaceFolder) {
                throw new Error('WorkspaceFolder-scoped planner configurations must specify originating workspaceFolder');
            }
        }
        else {
            // omit the workspace folder, when it is not needed
            workspaceFolder = undefined;
        }
        return {
            configuration: config,
            scope: scope,
            index: index,
            workspaceFolder: workspaceFolder === null || workspaceFolder === void 0 ? void 0 : workspaceFolder.uri.toString()
        };
    }
    async deletePlanner(plannerConfiguration) {
        const workspaceFolder = this.toWorkspaceFolder(plannerConfiguration);
        const remainingPlannerConfigs = this.getPlannersPerScope(plannerConfiguration.scope, workspaceFolder);
        remainingPlannerConfigs.splice(plannerConfiguration.index, 1);
        await this.savePlanners(plannerConfiguration.scope, remainingPlannerConfigs, workspaceFolder);
    }
    async savePlannerConfiguration(index, scope, newPlannerConfiguration, workspaceFolder) {
        const scopePlanners = this.getPlannersPerScope(scope, workspaceFolder);
        scopePlanners.splice(index, 1, newPlannerConfiguration);
        await this.savePlanners(scope, scopePlanners, workspaceFolder);
        return this.toScopedConfiguration(newPlannerConfiguration, index, scope, workspaceFolder);
    }
    /**
     * Overwrite planners in given `scope`.
     * @param scope configuration scope
     * @param scopePlanners planners to set for this scope
     * @param workspaceFolder workspace folder - must be provider if `scope` is set to WorkspaceFolder
     */
    async savePlanners(scope, scopePlanners, workspaceFolder) {
        const target = this.toConfigurationTarget(scope);
        const scopedPlannersToSave = scopePlanners.length ?
            scopePlanners :
            // remove the setting key altogether to keep settings.json tidy
            undefined;
        const configuration = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL, workspaceFolder);
        await configuration
            .update(exports.CONF_PLANNERS, scopedPlannersToSave, target);
    }
    toConfigurationTarget(scope) {
        switch (scope) {
            case PlannerConfigurationScope.User:
                return vscode_1.ConfigurationTarget.Global;
            case PlannerConfigurationScope.Workspace:
                return vscode_1.ConfigurationTarget.Workspace;
            case PlannerConfigurationScope.WorkspaceFolder:
                return vscode_1.ConfigurationTarget.WorkspaceFolder;
            default:
                return undefined;
        }
    }
    getSelectedPlanner(workingFolder) {
        const selectedPlannerTitle = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL, workingFolder).get(exports.CONF_SELECTED_PLANNER);
        if (selectedPlannerTitle !== undefined) {
            for (const scope of PlannersConfiguration.SCOPES) {
                // todo: get the planners for all scopes, then filter
                const planners = this.getPlannersPerScope(scope, workingFolder);
                if (!Array.isArray(planners)) {
                    console.error(`Planners configuration for scope ${scope} is invalid: ${planners}`);
                    continue;
                }
                const indexFound = planners.findIndex(p => p.title === selectedPlannerTitle);
                if (indexFound > -1) {
                    return this.toScopedConfiguration(planners[indexFound], indexFound, scope, workingFolder);
                }
            }
            console.warn(`selected planner ${selectedPlannerTitle} was not found in any scope (configuration is out of sync)`);
            return undefined;
        }
        else {
            throw new Error(`Configuration is missing default value for: ${configuration_1.CONF_PDDL}.${exports.CONF_SELECTED_PLANNER}`);
        }
    }
    async getOrAskSelectedPlanner(workingFolder) {
        let selectedPlanner = this.getSelectedPlanner(workingFolder);
        if (!selectedPlanner) {
            selectedPlanner = await this.selectPlanner();
            if (selectedPlanner) {
                await this.setSelectedPlanner(selectedPlanner);
            }
        }
        return selectedPlanner;
    }
    async createPlannerConfiguration() {
        const allSpecs = this.pddlWorkspace.getPlannerRegistrar().getPlannerProviders()
            .map(provider => new PlannerSpecPickItem(provider));
        const selectedItem = await vscode_1.window.showQuickPick(allSpecs, { placeHolder: 'Select type of planner ...' });
        if (!selectedItem) {
            return undefined;
        }
        const newConfiguration = await selectedItem.provider.configurePlanner();
        if (!newConfiguration) { // canceled by user
            return undefined;
        }
        // we only support adding planner configurations to the Global/User scope
        return await this.addPlannerConfiguration(PlannerConfigurationScope.User, newConfiguration);
    }
    /**
     * Adds a new planner configuration to the `scope` and sets it as a default.
     * @param scope scope
     * @param newPlannerConfig new planner configuration
     * @param workspaceFolder workspace folder - must be provider if `scope` is set to WorkspaceFolder
     */
    async addPlannerConfiguration(scope, newPlannerConfig, workspaceFolder) {
        const allPlannerNames = this.getPlanners(workspaceFolder)
            .map(spc => this.getUnNumberedTitle(spc));
        if (allPlannerNames.includes(newPlannerConfig.title)) {
            const similarPlannerCount = allPlannerNames
                .filter(name => name === newPlannerConfig.title)
                .length;
            newPlannerConfig.title += ` #${similarPlannerCount + 1}`;
        }
        const plannersInScope = this.getPlannersPerScope(scope);
        await this.savePlanners(scope, plannersInScope.concat([newPlannerConfig]), workspaceFolder);
        const scopedConfiguration = this.toScopedConfiguration(newPlannerConfig, plannersInScope.length, scope, workspaceFolder);
        await this.setSelectedPlanner(scopedConfiguration);
        return scopedConfiguration;
    }
    getUnNumberedTitle(config) {
        const title = config.configuration.title;
        if (title.includes('#')) {
            const match = /(.+)(#\w+)/.exec(title);
            if (match) {
                return match[1].trim();
            }
        }
        return title;
    }
    async openSettingsInJson() {
        var _a, _b;
        const settings = [
            this.createSettingForScope(PlannerConfigurationScope.User),
            this.createSettingForScope(PlannerConfigurationScope.Workspace),
        ].concat((_b = (_a = vscode_1.workspace.workspaceFolders) === null || _a === void 0 ? void 0 : _a.map(wf => this.toFolderConfigurationUri(wf))) !== null && _b !== void 0 ? _b : [])
            .filter(s => !!s)
            .map(s => s);
        const documentsAndRanges = await Promise.all(settings.map(s => this.toDocumentAndRange(s)));
        const validDocumentsAndRanges = documentsAndRanges
            .filter(setting => !!setting)
            .map(setting => setting);
        validDocumentsAndRanges.forEach(async (docAndRange, index) => {
            vscode_1.window.showTextDocument(docAndRange.settingsDoc, { selection: docAndRange.range, viewColumn: index + 1 });
        });
    }
    async openPlannerSettingsInJson(plannerConfiguration) {
        const workspaceFolder = plannerConfiguration.workspaceFolder ? vscode_1.workspace.getWorkspaceFolder(vscode_1.Uri.parse(plannerConfiguration.workspaceFolder)) : undefined;
        const setting = this.createSettingForScope(plannerConfiguration.scope, workspaceFolder);
        const documentAndRange = setting && await this.toDocumentAndRange(setting, plannerConfiguration.index);
        documentAndRange && await vscode_1.window.showTextDocument(documentAndRange.settingsDoc, { selection: documentAndRange.range, viewColumn: vscode_1.ViewColumn.Beside });
    }
    createSettingForScope(scope, workspaceFolder) {
        switch (scope) {
            case PlannerConfigurationScope.User:
                return { fileUri: vscode_1.Uri.file(this.getUserSettings()), settingRootPath: [] };
            case PlannerConfigurationScope.Workspace:
                return vscode_1.workspace.workspaceFile && { fileUri: vscode_1.workspace.workspaceFile, settingRootPath: ["settings"] };
            case PlannerConfigurationScope.WorkspaceFolder:
                return workspaceFolder && this.toFolderConfigurationUri(workspaceFolder);
            default:
                return undefined;
        }
    }
    toFolderConfigurationUri(wf) {
        const workspaceFolderSettingsPath = path.join(wf.uri.fsPath, '.vscode', 'settings.json');
        return {
            fileUri: vscode_1.Uri.file(workspaceFolderSettingsPath),
            settingRootPath: []
        };
    }
    async toDocumentAndRange(setting, index) {
        if (!setting.fileUri) {
            return undefined;
        }
        const exists = await utils_1.fileOrFolderExists(setting.fileUri);
        if (!exists) {
            return undefined;
        }
        const settingsText = await vscode_1.workspace.fs.readFile(setting.fileUri);
        const settingsRoot = jsonc_parser_1.parseTree(settingsText.toString());
        if (!settingsRoot) {
            return undefined;
        }
        let path = setting.settingRootPath.concat([configuration_1.CONF_PDDL + '.' + exports.CONF_PLANNERS]);
        if (index !== undefined) {
            path = path.concat([index]);
        }
        const plannersNode = jsonc_parser_1.findNodeAtLocation(settingsRoot, path);
        if (!plannersNode) {
            return undefined;
        }
        const settingsDoc = await vscode_1.workspace.openTextDocument(setting.fileUri);
        const range = plannersNode && utils_1.jsonNodeToRange(settingsDoc, plannersNode);
        return { settingsDoc, range };
    }
    getUserSettings() {
        // https://supunkavinda.blog/vscode-editing-settings-json
        const windows = "%APPDATA%\\Code\\User\\settings.json";
        const macOS = "$HOME/Library/Application Support/Code/User/settings.json";
        const linux = "$HOME/.config/Code/User/settings.json";
        switch (os.platform()) {
            case 'darwin':
                return macOS;
            case 'win32':
                return windows.replace(/%([^%]+)%/g, (_, n) => { var _a; return (_a = process.env[n]) !== null && _a !== void 0 ? _a : n; });
            default:
                // 'freebsd'
                // 'linux'
                // 'openbsd'
                // 'sunos'
                // 'cygwin';
                return linux;
        }
    }
}
exports.PlannersConfiguration = PlannersConfiguration;
PlannersConfiguration.SCOPES = [
    PlannerConfigurationScope.WorkspaceFolder,
    PlannerConfigurationScope.Workspace,
    PlannerConfigurationScope.User,
    PlannerConfigurationScope.Default,
];
class PlannerQuickPickItem {
    constructor(label) {
        this.label = label;
    }
}
class ExistingPlannerQuickPickItem extends PlannerQuickPickItem {
    constructor(plannerConfig) {
        super(plannerConfig.configuration.title);
        this.planner = plannerConfig;
        this.alwaysShow = false;
    }
}
class NewPlannerQuickPickItem extends PlannerQuickPickItem {
    constructor() {
        super("$(add) Create new planner configuration");
    }
}
NewPlannerQuickPickItem.CREATE_NEW = new NewPlannerQuickPickItem();
class ConfigurePlannerQuickPickItem extends PlannerQuickPickItem {
    constructor() {
        super("$(gear) Configure planners");
    }
}
ConfigurePlannerQuickPickItem.CONFIGURE = new ConfigurePlannerQuickPickItem();
class PlannerSpecPickItem {
    constructor(plannerSpec) {
        this.provider = plannerSpec;
    }
    get label() {
        return this.provider.getNewPlannerLabel();
    }
}
//# sourceMappingURL=PlannersConfiguration.js.map