"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PddlConfiguration = exports.DEFAULT_EPSILON = exports.PDDL_CONFIGURE_COMMAND = exports.PLANNER_VALUE_SEQ_PATH = exports.PLANNER_VAL_STEP_PATH = exports.PLAN_REPORT_WIDTH = exports.PLAN_REPORT_EXPORT_WIDTH = exports.PLAN_REPORT_LINE_PLOT_GROUP_BY_LIFTED = exports.VALUE_SEQ_PATH = exports.VAL_VERBOSE = exports.VAL_STEP_PATH = exports.VALIDATION_PATH = exports.VALIDATOR_VERSION = exports.CONF_PDDL = exports.PLANNER_EXECUTABLE_OR_SERVICE = exports.PDDL_PLANNER = exports.EXECUTABLE_OPTIONS = exports.PARSER_EXECUTABLE_OR_SERVICE = exports.PDDL_PARSER = exports.EXECUTABLE_OR_SERVICE = void 0;
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
const vscode_1 = require("vscode");
const utils_1 = require("../utils");
const valCommand_1 = require("../validation/valCommand");
const ExtensionInfo_1 = require("./ExtensionInfo");
exports.EXECUTABLE_OR_SERVICE = 'executableOrService';
exports.PDDL_PARSER = 'pddlParser';
exports.PARSER_EXECUTABLE_OR_SERVICE = exports.PDDL_PARSER + '.' + exports.EXECUTABLE_OR_SERVICE;
exports.EXECUTABLE_OPTIONS = 'executableOptions';
const PARSER_EXECUTABLE_OPTIONS = exports.PDDL_PARSER + '.' + exports.EXECUTABLE_OPTIONS;
const PARSER_SERVICE_AUTHENTICATION_REFRESH_TOKEN = exports.PDDL_PARSER + '.serviceAuthenticationRefreshToken';
const PARSER_SERVICE_AUTHENTICATION_ACCESS_TOKEN = exports.PDDL_PARSER + '.serviceAuthenticationAccessToken';
const PARSER_SERVICE_AUTHENTICATION_S_TOKEN = exports.PDDL_PARSER + '.serviceAuthenticationSToken';
exports.PDDL_PLANNER = 'pddlPlanner';
exports.PLANNER_EXECUTABLE_OR_SERVICE = exports.PDDL_PLANNER + '.' + exports.EXECUTABLE_OR_SERVICE;
const PLANNER_SERVICE_AUTHENTICATION_REFRESH_TOKEN = exports.PDDL_PLANNER + '.serviceAuthenticationRefreshToken';
const PLANNER_SERVICE_AUTHENTICATION_ACCESS_TOKEN = exports.PDDL_PLANNER + '.serviceAuthenticationAccessToken';
const PLANNER_SERVICE_AUTHENTICATION_S_TOKEN = exports.PDDL_PLANNER + '.serviceAuthenticationSToken';
const PLANNER_EPSILON_TIMESTEP = exports.PDDL_PLANNER + '.epsilonTimeStep';
exports.CONF_PDDL = 'pddl';
exports.VALIDATOR_VERSION = 'validatorVersion';
exports.VALIDATION_PATH = 'validatorPath';
exports.VAL_STEP_PATH = 'valStepPath';
exports.VAL_VERBOSE = 'valVerbose';
exports.VALUE_SEQ_PATH = 'valueSeqPath';
exports.PLAN_REPORT_LINE_PLOT_GROUP_BY_LIFTED = 'planReport.linePlot.groupByLifted';
exports.PLAN_REPORT_EXPORT_WIDTH = 'planReport.exportWidth';
exports.PLAN_REPORT_WIDTH = 'planReport.width';
exports.PLANNER_VAL_STEP_PATH = exports.CONF_PDDL + "." + exports.VAL_STEP_PATH;
exports.PLANNER_VALUE_SEQ_PATH = exports.CONF_PDDL + "." + exports.VALUE_SEQ_PATH;
exports.PDDL_CONFIGURE_COMMAND = exports.CONF_PDDL + "." + "configure";
exports.DEFAULT_EPSILON = 1e-3;
const EDITOR_FORMAT_ON_TYPE = "editor.formatOnType";
class PddlConfiguration {
    constructor(context) {
        this.context = context;
        this.NEVER_SETUP_PARSER = 'neverSetupParser';
        this.setupParserLater = false;
    }
    async updateEffectiveConfiguration(section, key, value, workspaceFolder) {
        const configuration = vscode_1.workspace.getConfiguration(section, workspaceFolder);
        const inspect = configuration.inspect(key);
        if (!inspect) {
            throw new Error(`Unexpected configuration: ${section}.${key}`);
        }
        if (inspect.workspaceFolderValue !== undefined) {
            await configuration.update(key, value, vscode_1.ConfigurationTarget.WorkspaceFolder);
        }
        else if (inspect.workspaceValue !== undefined) {
            await configuration.update(key, value, vscode_1.ConfigurationTarget.Workspace);
        }
        else {
            await configuration.update(key, value, vscode_1.ConfigurationTarget.Global);
        }
    }
    getEpsilonTimeStep() {
        return vscode_1.workspace.getConfiguration().get(PLANNER_EPSILON_TIMESTEP, exports.DEFAULT_EPSILON);
    }
    getEditorFormatOnType() {
        return vscode_1.workspace.getConfiguration('', { languageId: exports.CONF_PDDL }).get(EDITOR_FORMAT_ON_TYPE, false);
    }
    setEditorFormatOnType(newValue, options) {
        if (options.forPddlOnly) {
            vscode_1.workspace.getConfiguration(undefined, { languageId: exports.CONF_PDDL }).update(EDITOR_FORMAT_ON_TYPE, newValue, true, true);
        }
        else {
            vscode_1.workspace.getConfiguration().update(EDITOR_FORMAT_ON_TYPE, newValue, true);
        }
    }
    getParserPath(workspaceFolder) {
        const configuredPath = vscode_1.workspace.getConfiguration(exports.PDDL_PARSER, workspaceFolder).get(exports.EXECUTABLE_OR_SERVICE);
        return utils_1.ensureAbsoluteGlobalStoragePath(configuredPath, this.context);
    }
    async suggestNewParserConfiguration(showNever) {
        if (this.setupParserLater || this.context.globalState.get(this.NEVER_SETUP_PARSER)) {
            return;
        }
        const setupParserNow = { title: "Setup manually..." };
        const downloadVal = { title: "Download VAL now..." };
        const setupParserNever = { title: "Never" };
        const setupParserLater = { title: "Later", isCloseAffordance: true };
        const options = [downloadVal, setupParserNow, setupParserLater];
        if (showNever) {
            options.splice(options.length, 0, setupParserNever);
        }
        const choice = await vscode_1.window.showInformationMessage('Setup a [PDDL parser](https://github.com/jan-dolejsi/vscode-pddl/wiki/Configuring-the-PDDL-parser "Read more about PDDL parsers") or download [VAL Tools](https://github.com/KCL-Planning/VAL) in order to enable detailed syntactic analysis.', ...options);
        switch (choice) {
            case setupParserNow:
                this.askNewParserPath();
                // if the above method call updates the configuration, the parser will be notified
                break;
            case downloadVal:
                vscode_1.commands.executeCommand(valCommand_1.VAL_DOWNLOAD_COMMAND);
                break;
            case setupParserLater:
                this.setupParserLater = true; // will retry in the next session
                break;
            case setupParserNever:
                this.context.globalState.update(this.NEVER_SETUP_PARSER, true);
                break;
            default:
                break;
        }
    }
    async askNewParserPath() {
        const existingValue = vscode_1.workspace.getConfiguration().get(exports.PARSER_EXECUTABLE_OR_SERVICE);
        let newParserPath = await vscode_1.window.showInputBox({
            prompt: "Enter PDDL parser/validator path local command or web service URL",
            placeHolder: `parser.exe OR java -jar c:\\planner.jar OR https://someserver/parse`,
            value: existingValue,
            ignoreFocusOut: true
        });
        if (newParserPath) {
            newParserPath = newParserPath.trim().replace(/\\/g, '/');
            // todo: validate that this parser actually works by sending a dummy request to it
            const newParserScope = await this.askConfigurationScope();
            if (!newParserScope) {
                return undefined;
            }
            const configurationToUpdate = this.getConfigurationForScope(newParserScope);
            if (!configurationToUpdate) {
                return undefined;
            }
            if (!utils_1.isHttp(newParserPath)) {
                this.askParserOptions(newParserScope);
            }
            // Update the value in the target
            configurationToUpdate.update(exports.PARSER_EXECUTABLE_OR_SERVICE, newParserPath, newParserScope.target);
        }
        return newParserPath;
    }
    async askParserOptions(scope) {
        const existingValue = vscode_1.workspace.getConfiguration().get(PARSER_EXECUTABLE_OPTIONS);
        const newParserOptions = await vscode_1.window.showInputBox({
            prompt: "In case you use command line switches and options, override the default syntax. For more info, see (the wiki)[https://github.com/jan-dolejsi/vscode-pddl/wiki/Configuring-the-PDDL-parser].",
            placeHolder: `$(domain) $(problem)`,
            value: existingValue,
            ignoreFocusOut: true
        });
        if (newParserOptions) {
            // todo: validate that this parser actually works by sending a dummy request to it
            const configurationToUpdate = this.getConfigurationForScope(scope);
            if (!configurationToUpdate) {
                return undefined;
            }
            // Update the value in the target
            configurationToUpdate.update(PARSER_EXECUTABLE_OPTIONS, newParserOptions, scope.target);
        }
        return newParserOptions;
    }
    isPddlParserServiceAuthenticationEnabled() {
        return vscode_1.workspace.getConfiguration().get(exports.PDDL_PARSER + '.serviceAuthenticationEnabled', false);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    getPddlParserServiceAuthenticationConfiguration() {
        const configuration = vscode_1.workspace.getConfiguration();
        return {
            url: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationUrl'),
            requestEncoded: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationRequestEncoded'),
            clientId: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationClientId'),
            callbackPort: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationCallbackPort'),
            timeoutInMs: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTimeoutInMs'),
            tokensvcUrl: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcUrl'),
            tokensvcApiKey: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcApiKey'),
            tokensvcAccessPath: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcAccessPath'),
            tokensvcValidatePath: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcValidatePath'),
            tokensvcCodePath: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcCodePath'),
            tokensvcRefreshPath: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcRefreshPath'),
            tokensvcSvctkPath: configuration.get(exports.PDDL_PARSER + '.serviceAuthenticationTokensvcSvctkPath'),
            refreshToken: configuration.get(PARSER_SERVICE_AUTHENTICATION_REFRESH_TOKEN),
            accessToken: configuration.get(PARSER_SERVICE_AUTHENTICATION_ACCESS_TOKEN),
            sToken: configuration.get(PARSER_SERVICE_AUTHENTICATION_S_TOKEN),
        };
    }
    async savePddlParserAuthenticationTokens(configuration, refreshtoken, accesstoken, stoken, target) {
        configuration.update(PARSER_SERVICE_AUTHENTICATION_REFRESH_TOKEN, refreshtoken, target);
        configuration.update(PARSER_SERVICE_AUTHENTICATION_ACCESS_TOKEN, accesstoken, target);
        configuration.update(PARSER_SERVICE_AUTHENTICATION_S_TOKEN, stoken, target);
    }
    isPddlPlannerServiceAuthenticationEnabled() {
        return vscode_1.workspace.getConfiguration().get(exports.PDDL_PLANNER + '.serviceAuthenticationEnabled', false);
    }
    getPddlPlannerServiceAuthenticationConfiguration() {
        const configuration = vscode_1.workspace.getConfiguration();
        return {
            url: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationUrl', ''),
            requestEncoded: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationRequestEncoded', ''),
            clientId: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationClientId', ''),
            callbackPort: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationCallbackPort', 0),
            timeoutInMs: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTimeoutInMs', 0),
            tokensvcUrl: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcUrl', ""),
            tokensvcApiKey: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcApiKey', ""),
            tokensvcAccessPath: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcAccessPath', ""),
            tokensvcValidatePath: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcValidatePath', ""),
            tokensvcCodePath: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcCodePath', ""),
            tokensvcRefreshPath: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcRefreshPath', ""),
            tokensvcSvctkPath: configuration.get(exports.PDDL_PLANNER + '.serviceAuthenticationTokensvcSvctkPath', ""),
            refreshToken: configuration.get(PLANNER_SERVICE_AUTHENTICATION_REFRESH_TOKEN, ""),
            accessToken: configuration.get(PLANNER_SERVICE_AUTHENTICATION_ACCESS_TOKEN, ""),
            sToken: configuration.get(PLANNER_SERVICE_AUTHENTICATION_S_TOKEN, ""),
        };
    }
    async savePddlPlannerAuthenticationTokens(configuration, refreshtoken, accesstoken, stoken, target) {
        configuration.update(PLANNER_SERVICE_AUTHENTICATION_REFRESH_TOKEN, refreshtoken, target);
        configuration.update(PLANNER_SERVICE_AUTHENTICATION_ACCESS_TOKEN, accesstoken, target);
        configuration.update(PLANNER_SERVICE_AUTHENTICATION_S_TOKEN, stoken, target);
    }
    getValueSeqPath() {
        const configuredPath = vscode_1.workspace.getConfiguration().get(exports.PLANNER_VALUE_SEQ_PATH);
        return utils_1.ensureAbsoluteGlobalStoragePath(configuredPath, this.context);
    }
    getValidatorPath(workspaceFolder) {
        const configuredPath = vscode_1.workspace.getConfiguration(exports.CONF_PDDL, workspaceFolder).get(exports.VALIDATION_PATH);
        return utils_1.ensureAbsoluteGlobalStoragePath(configuredPath, this.context);
    }
    async askNewValidatorPath() {
        const configuredPath = await this.askAndUpdatePath(exports.VALIDATION_PATH, "Validate tool");
        return utils_1.ensureAbsoluteGlobalStoragePath(configuredPath, this.context);
    }
    async getValStepPath() {
        const configuredPath = await this.getOrAskPath(exports.VAL_STEP_PATH, "ValStep executable");
        return utils_1.ensureAbsoluteGlobalStoragePath(configuredPath, this.context);
    }
    async getOrAskPath(configName, configFriendlyName) {
        const configurationSection = vscode_1.workspace.getConfiguration(exports.CONF_PDDL);
        let configValue = configurationSection.get(configName);
        if (!configValue) {
            configValue = await this.askAndUpdatePath(configName, configFriendlyName);
        }
        return configValue;
    }
    getValStepVerbose() {
        return vscode_1.workspace.getConfiguration(exports.CONF_PDDL).get(exports.VAL_VERBOSE, false);
    }
    async suggestUpdatingPath(configName, configFriendlyName) {
        const configureOption = { title: `Select ${configFriendlyName}...` };
        const notNowOption = { title: "Not now", isCloseAffordance: true };
        const choice = await vscode_1.window.showErrorMessage(`${configFriendlyName} is not configured.`, ...[configureOption, notNowOption]);
        let configValue;
        if (choice === configureOption) {
            configValue = await this.askAndUpdatePath(configName, configFriendlyName);
        }
        return configValue;
    }
    async askAndUpdatePath(configName, configFriendlyName) {
        const selectedUris = await vscode_1.window.showOpenDialog({
            canSelectFiles: true,
            canSelectFolders: false, canSelectMany: false,
            openLabel: `Select ${configFriendlyName}`
        });
        let configValue;
        if (selectedUris) {
            configValue = selectedUris[0].fsPath;
            const scopeToUpdate = await this.askConfigurationScope();
            if (!scopeToUpdate) {
                return undefined;
            }
            const configurationSection = vscode_1.workspace.getConfiguration(exports.CONF_PDDL);
            configurationSection.update(configName, configValue, scopeToUpdate.target);
        }
        return configValue;
    }
    async askConfigurationScope() {
        const availableScopes = [
            { label: 'This machine (default)', description: 'Selected tool will be used for all domain/problem files on this computer.', target: vscode_1.ConfigurationTarget.Global }
        ];
        if (vscode_1.workspace.workspaceFolders) {
            vscode_1.workspace.workspaceFolders.forEach(folder => {
                availableScopes.push({ label: 'This workspace', description: `Selected tool will be used just for this workspace: ${folder.name}`, target: vscode_1.ConfigurationTarget.Workspace, uri: folder.uri });
            });
        }
        // todo: need to support folders?
        //{ label: 'Just one workspace folder', description: 'Selected tool will be used just for one workspace folder...', target: ConfigurationTarget.WorkspaceFolder }
        const selectedScope = availableScopes.length === 1 ? availableScopes[0] : await vscode_1.window.showQuickPick(availableScopes, { placeHolder: 'Select the target scope for which this setting should be applied' });
        return selectedScope;
    }
    async moveConfiguration(configuration, legacyConfigName, configName) {
        const legacyConfig = configuration.inspect(legacyConfigName);
        if (!legacyConfig) {
            return;
        }
        let target;
        if (legacyConfig.workspaceFolderValue) {
            target = vscode_1.ConfigurationTarget.WorkspaceFolder;
        }
        else if (legacyConfig.workspaceValue) {
            target = vscode_1.ConfigurationTarget.Workspace;
        }
        else if (legacyConfig.globalValue) {
            target = vscode_1.ConfigurationTarget.Global;
        }
        else if (legacyConfig.defaultValue) {
            await configuration.update(configName, legacyConfig.defaultValue, vscode_1.ConfigurationTarget.Global);
        }
        if (target) {
            await configuration.update(configName, configuration.get(legacyConfigName), target);
            await configuration.update(legacyConfigName, null, target);
        }
    }
    getConfigurationForScope(scope) {
        if (scope.target === vscode_1.ConfigurationTarget.WorkspaceFolder) {
            // let workspaceFolder = await window.showWorkspaceFolderPick({ placeHolder: 'Pick Workspace Folder to which this setting should be applied' })
            // if (workspaceFolder) {
            // 	// *Get the configuration for the workspace folder
            // 	const configuration = workspace.getConfiguration('', workspaceFolder.uri);
            vscode_1.window.showErrorMessage("Workspace folder not supported");
            return undefined;
        }
        else {
            return vscode_1.workspace.getConfiguration();
        }
    }
    getParserSettings() {
        const configurationAny = vscode_1.workspace.getConfiguration(exports.PDDL_PARSER);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const configuration = configurationAny;
        return configuration;
    }
    async askConfiguration(configName) {
        const thisExtension = vscode_1.extensions.getExtension(ExtensionInfo_1.ExtensionInfo.EXTENSION_ID);
        if (thisExtension === undefined) {
            return;
        } // something odd!
        const configurationElement = thisExtension.packageJSON["contributes"]["configuration"]["properties"][configName];
        if (!configurationElement) {
            throw new Error("Configuration not found: " + configName);
        }
        if (["number", "integer"].includes(configurationElement["type"])) {
            return this.askNumberConfiguration(configName, configurationElement);
        }
        else if (configurationElement["enum"] !== undefined) {
            return this.askEnumConfiguration(configName, configurationElement);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async askEnumConfiguration(configName, configurationElement) {
        const items = [];
        const enumNames = configurationElement["enum"];
        const enumDescriptions = configurationElement["enumDescriptions"];
        const currentValue = vscode_1.workspace.getConfiguration().get(configName, configurationElement["default"]);
        for (let index = 0; index < enumNames.length; index++) {
            const itemLabel = enumNames[index];
            const description = enumDescriptions && index < enumDescriptions.length ?
                enumDescriptions[index] : undefined;
            items.push({
                label: itemLabel,
                description: description,
                picked: itemLabel === currentValue
            });
        }
        const itemSelected = await vscode_1.window.showQuickPick(items, {
            placeHolder: configurationElement["description"] + ` (current value: ${currentValue})`
        });
        if (itemSelected === undefined) {
            return;
        }
        const configurationTarget = this.getEffectiveConfigurationTarget(configName);
        await vscode_1.workspace.getConfiguration().update(configName, itemSelected.label, configurationTarget);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async askNumberConfiguration(configName, configurationElement) {
        const currentValue = vscode_1.workspace.getConfiguration().get(configName, configurationElement["default"]);
        let hint;
        let parser;
        const valueType = configurationElement["type"];
        switch (valueType) {
            case "integer":
                hint = "Value must be a decimal integer.";
                parser = function (enteredValueAsString) { return Number.parseInt(enteredValueAsString); };
                break;
            default:
                hint = "Value must be a number.";
                parser = function (enteredValueAsString) { return Number.parseFloat(enteredValueAsString); };
                break;
        }
        const minimum = configurationElement["minimum"];
        const maximum = configurationElement["maximum"];
        const enteredValueAsString = await vscode_1.window.showInputBox({
            prompt: configurationElement["description"],
            value: currentValue.toString(),
            validateInput: (enteredValueAsString) => {
                const enteredValue = parser.apply(this, [enteredValueAsString]);
                if (Number.isNaN(enteredValue)) {
                    return hint;
                }
                if (minimum !== undefined && minimum > enteredValue) {
                    return `Minimum: ${minimum}`;
                }
                if (maximum !== undefined && maximum < enteredValue) {
                    return `Maximum: ${maximum}`;
                }
                return null;
            },
            placeHolder: "Value"
        });
        if (enteredValueAsString === undefined) {
            return;
        }
        const configurationTarget = this.getEffectiveConfigurationTarget(configName);
        await vscode_1.workspace.getConfiguration().update(configName, parser(enteredValueAsString), configurationTarget);
    }
    /**
     * Determines the configuration scope, where the `configName` was configured in the active workspace.
     * @param configName configuration item e.g. scope.name
     */
    getEffectiveConfigurationTarget(configName) {
        const conf = vscode_1.workspace.getConfiguration().inspect(configName);
        if (conf === null || conf === void 0 ? void 0 : conf.workspaceFolderValue) {
            return vscode_1.ConfigurationTarget.WorkspaceFolder;
        }
        else if (conf === null || conf === void 0 ? void 0 : conf.workspaceValue) {
            return vscode_1.ConfigurationTarget.Workspace;
        }
        else {
            return vscode_1.ConfigurationTarget.Global;
        }
    }
}
exports.PddlConfiguration = PddlConfiguration;
//# sourceMappingURL=configuration.js.map