/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.HappeningsToPlanResumeCasesConvertor = void 0;
const vscode_1 = require("vscode");
const ai_planning_val_1 = require("ai-planning-val");
const pddl_workspace_1 = require("pddl-workspace");
const path_1 = require("path");
const process = require("child_process");
const ai_planning_val_2 = require("ai-planning-val");
const TestsManifest_1 = require("../ptest/TestsManifest");
const Test_1 = require("../ptest/Test");
const pddl_workspace_2 = require("pddl-workspace");
const PTestCommands_1 = require("../ptest/PTestCommands");
/**
 * Executes sequence of plan happenings and generates suite of test cases to attempt planning from any intermediate state.
 */
class HappeningsToPlanResumeCasesConvertor {
    /**
     * Constructs the executor.
     * @param context debugging session context - the happenings, problem and domain files
     * @param pddlConfiguration user/workspace configuration
     */
    constructor(context, pddlConfiguration) {
        this.context = context;
        this.pddlConfiguration = pddlConfiguration;
        this.happeningsConvertor = new ai_planning_val_2.HappeningsToValStep();
        this.problemFilePath = context.problem.fileUri.fsPath;
    }
    /**
     * Executes all the happenings and decorates the happenings file with action effects.
     */
    async generate() {
        if (this.context.happenings.getParsingProblems().length > 0) {
            vscode_1.commands.executeCommand('workbench.action.problems.show');
            vscode_1.window.showErrorMessage('The plan happenings file contains syntactic errors. Fix those first.');
            return false;
        }
        const valStepPath = await this.pddlConfiguration.getValStepPath();
        if (!valStepPath) {
            return false;
        }
        // copy editor content to temp files to avoid using out-of-date content on disk
        const domainFilePath = await ai_planning_val_1.Util.toPddlFile('domain', this.context.domain.getText());
        const problemFilePath = await ai_planning_val_1.Util.toPddlFile('problem', this.context.problem.getText());
        const args = [domainFilePath, problemFilePath];
        const outputFolderUris = await vscode_1.window.showOpenDialog({ defaultUri: vscode_1.Uri.file(path_1.dirname(this.context.problem.fileUri.fsPath)), canSelectFiles: false, canSelectFolders: true, canSelectMany: false, openLabel: 'Select folder for problem files' });
        if (!outputFolderUris || !outputFolderUris.length) {
            return false;
        }
        const outputFolderUri = outputFolderUris[0];
        const cwd = outputFolderUri.fsPath;
        const cmd = valStepPath + ' ' + args.join(' ');
        const groupedHappenings = pddl_workspace_1.utils.Util.groupBy(this.context.happenings.getHappenings(), h => h.getTime());
        let valStepInput = '';
        const defaultDomain = path_1.relative(cwd, this.context.domain.fileUri.fsPath);
        const defaultProblem = undefined;
        const options = "";
        const manifestUri = vscode_1.Uri.file(path_1.join(cwd, path_1.basename(this.context.problem.fileUri.fsPath) + '_re-planning.ptest.json'));
        const manifest = new TestsManifest_1.TestsManifest(defaultDomain, defaultProblem, options, manifestUri);
        // add the original problem as a test case
        manifest.addCase(new Test_1.Test(undefined, "Original problem file", undefined, path_1.relative(cwd, this.context.problem.fileUri.fsPath), "", undefined, []));
        const problemFileWithoutExt = path_1.basename(this.context.problem.fileUri.fsPath, ".pddl");
        for (const time of groupedHappenings.keys()) {
            const happeningGroup = groupedHappenings.get(time);
            if (!happeningGroup) {
                continue;
            }
            const valSteps = this.happeningsConvertor.convert(happeningGroup);
            valStepInput += valSteps;
            const lastHappening = happeningGroup[happeningGroup.length - 1];
            const problemFileName = `${problemFileWithoutExt}_${time}_${this.getHappeningFullName(lastHappening)}.pddl`.split(' ').join('_');
            const testCaseLabel = `${time}: after (${lastHappening.getFullActionName()}) ${this.getHappeningSnapName(lastHappening)}`;
            const testCaseDescription = `Test case for planning from: \nTime point: ${time} and after the application of \nAction: ${lastHappening.getFullActionName()} ${this.getHappeningSnapName(lastHappening)}`;
            manifest.addCase(new Test_1.Test(testCaseLabel, testCaseDescription, undefined, problemFileName, "", undefined, []));
            valStepInput += `w ${problemFileName}\n`;
        }
        valStepInput += 'q\n';
        try {
            const output = process.execSync(cmd, { cwd: cwd, input: valStepInput });
            console.log(output.toLocaleString()); // for inspection while debugging
            await vscode_1.commands.executeCommand('workbench.view.extension.test');
            await vscode_1.commands.executeCommand('pddl.tests.report.view');
            await vscode_1.commands.executeCommand(PTestCommands_1.PTEST_REVEAL, manifestUri);
            await vscode_1.commands.executeCommand(PTestCommands_1.PTEST_REFRESH);
            return true;
        }
        catch (err) {
            vscode_1.window.showErrorMessage("Error running valstep: " + err);
            return false;
        }
        finally {
            if (manifest) {
                await manifest.store();
            }
        }
    }
    getHappeningSnapName(happening) {
        let snap = '';
        switch (happening.getType()) {
            case pddl_workspace_2.HappeningType.START:
                snap = 'start';
                break;
            case pddl_workspace_2.HappeningType.END:
                snap = 'end';
                break;
        }
        return snap;
    }
    getHappeningFullName(happening) {
        let output = happening.getFullActionName().split(' ').join('_');
        const snap = this.getHappeningSnapName(happening);
        if (snap.length) {
            output = `${output}_${snap}`;
        }
        return output;
    }
}
exports.HappeningsToPlanResumeCasesConvertor = HappeningsToPlanResumeCasesConvertor;
//# sourceMappingURL=HappeningsToPlanResumeCasesConvertor.js.map