/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Debugging = void 0;
const vscode = require("vscode");
const vscode_1 = require("vscode");
const PlanDebugSession_1 = require("./PlanDebugSession");
const Net = require("net");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const HappeningsExecutor_1 = require("./HappeningsExecutor");
const HappeningsToPlanResumeCasesConvertor_1 = require("./HappeningsToPlanResumeCasesConvertor");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
/*
 * Set the following compile time flag to true if the
 * debug adapter should run inside the extension host.
 * Please note: the test suite does no longer work in this mode.
 */
const EMBED_DEBUG_ADAPTER = true;
class Debugging {
    constructor(context, pddlWorkspace, plannerConfiguration) {
        this.pddlWorkspace = pddlWorkspace;
        this.plannerConfiguration = plannerConfiguration;
        this.decorations = new Map();
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.selectAndActivateHappenings', async (_config) => {
            return await workspaceUtils_1.selectHappenings();
        }));
        // register a configuration provider for 'pddl-happenings' debug type
        const provider = new PddlPlanDebugConfigurationProvider();
        context.subscriptions.push(vscode.debug.registerDebugConfigurationProvider('pddl-happenings', provider));
        context.subscriptions.push(provider);
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.happenings.debug', () => {
            return this.startDebugging();
        }));
        context.subscriptions.push(vscode.commands.registerTextEditorCommand("pddl.happenings.execute", async (editor) => {
            var _a;
            try {
                this.clearDecorations(editor.document);
                const context = await this.getActiveContext();
                const decorations = await new HappeningsExecutor_1.HappeningsExecutor(editor, context, this.plannerConfiguration).execute();
                this.saveDecorations(editor.document, decorations);
            }
            catch (ex) {
                vscode.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
            }
        }));
        context.subscriptions.push(vscode.commands.registerTextEditorCommand("pddl.happenings.generatePlanResumeCases", async () => {
            var _a;
            try {
                const context = await this.getActiveContext();
                new HappeningsToPlanResumeCasesConvertor_1.HappeningsToPlanResumeCasesConvertor(context, this.plannerConfiguration).generate();
            }
            catch (ex) {
                vscode.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
            }
        }));
        // clear decorations, when document is updated/closed
        context.subscriptions.push(vscode.workspace.onDidChangeTextDocument(evt => this.clearDecorations(evt.document)));
        context.subscriptions.push(vscode.workspace.onDidCloseTextDocument(d => this.clearDecorations(d)));
    }
    clearDecorations(document) {
        const decorations = this.decorations.get(document);
        if (decorations) {
            decorations.forEach(d => d.dispose());
        }
        this.decorations.set(document, []);
    }
    saveDecorations(document, decorations) {
        this.clearDecorations(document);
        this.decorations.set(document, decorations);
    }
    async getActiveContext() {
        if (!vscode_1.window.activeTextEditor) {
            throw new Error('There is no file active in the editor.');
        }
        if (!workspaceUtils_1.isHappenings(vscode_1.window.activeTextEditor.document)) {
            throw new Error('Active document cannot be debugged.');
        }
        const activeFileInfo = await this.pddlWorkspace.upsertAndParseFile(vscode_1.window.activeTextEditor.document);
        if (!(activeFileInfo instanceof pddl_workspace_2.HappeningsInfo)) {
            throw new Error('Active document cannot be debugged.');
        }
        const happeningsInfo = activeFileInfo;
        const context = workspaceUtils_1.getDomainAndProblemForHappenings(happeningsInfo, this.pddlWorkspace.pddlWorkspace);
        return {
            domain: context.domain,
            problem: context.problem,
            happenings: happeningsInfo
        };
    }
    async startDebugging() {
        const context = await this.getActiveContext();
        let folder; // so far there is no configuration to resolve
        const debugConfiguration = {
            "name": "PDDL Plan Happenings F5",
            "type": "pddl-happenings",
            "request": "launch",
            "program": context.happenings.fileUri.fsPath,
            "domain": context.domain.fileUri.fsPath,
            "problem": context.problem.fileUri.fsPath
        };
        await vscode.debug.startDebugging(folder, debugConfiguration);
    }
}
exports.Debugging = Debugging;
class PddlPlanDebugConfigurationProvider {
    /**
     * Massage a debug configuration just before a debug session is being launched,
     * e.g. add all missing attributes to the debug configuration.
     */
    async resolveDebugConfiguration(_folder, config, token) {
        if (token === null || token === void 0 ? void 0 : token.isCancellationRequested) {
            return undefined;
        }
        // if launch.json is missing or empty
        if (!config.type && !config.request && !config.name) {
            const editor = vscode.window.activeTextEditor;
            if (editor && editor.document.languageId === pddl_workspace_1.HAPPENINGS) {
                config.type = 'pddl-happenings';
                config.name = 'PDDL: Plan Happenings (from context menu)';
                config.request = 'launch';
                config.program = '${file}';
                config.domain = 'domain.pddl';
                config.problem = '${fileBasenameNoExtension}.pddl';
                config.stopOnEntry = true;
            }
        }
        if (!config.program) {
            await vscode.window.showInformationMessage("Cannot find a program to debug");
            return undefined; // abort launch
        }
        if (EMBED_DEBUG_ADAPTER) {
            // start port listener on launch of first debug session
            if (!this._server) {
                // start listening on a random port
                this._server = Net.createServer(socket => {
                    const session = new PlanDebugSession_1.PlanDebugSession();
                    session.setRunAsServer(true);
                    session.start(socket, socket);
                }).listen(0);
            }
            // make VS Code connect to debug server instead of launching debug adapter
            config.debugServer = this._server.address().port;
        }
        return config;
    }
    dispose() {
        if (this._server) {
            this._server.close();
        }
    }
}
//# sourceMappingURL=debugging.js.map