/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnostics = void 0;
const vscode_1 = require("vscode");
const Authentication_1 = require("../util/Authentication");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const validator_1 = require("./validator");
const ValidatorService_1 = require("./ValidatorService");
const ValidatorExecutable_1 = require("./ValidatorExecutable");
const configuration_1 = require("../configuration/configuration");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const utils_1 = require("../utils");
const validatorUtils_1 = require("./validatorUtils");
/**
 * Listens to updates to PDDL files and performs detailed parsing and syntactical analysis and report problems as `Diagnostics`.
 */
class Diagnostics extends vscode_1.Disposable {
    constructor(codePddlWorkspace, diagnosticCollection, pddlConfiguration, planValidator, happeningsValidator) {
        super(() => { console.log('Diagnostics disposed'); }); //this.codePddlWorkspace.pddlWorkspace.removeAllListeners()); // this was probably too harsh
        this.codePddlWorkspace = codePddlWorkspace;
        this.diagnosticCollection = diagnosticCollection;
        this.pddlConfiguration = pddlConfiguration;
        this.planValidator = planValidator;
        this.happeningsValidator = happeningsValidator;
        this.defaultTimerDelayInSeconds = 3;
        this.pddlParserSettings = pddlConfiguration.getParserSettings();
        this.codePddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.UPDATED, (fileInfo) => {
            // clear diagnostics for that file
            this.clearDiagnostics(fileInfo.fileUri);
            this.scheduleValidation();
        });
        this.codePddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.REMOVING, (doc) => this.clearDiagnostics(doc.fileUri));
        vscode_1.workspace.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration(configuration_1.PDDL_PARSER)
                || e.affectsConfiguration(configuration_1.CONF_PDDL + '.' + configuration_1.VALIDATION_PATH)) {
                this.handleConfigurationChange();
            }
        });
    }
    scheduleValidation() {
        this.cancelScheduledValidation();
        const timerDelayInSeconds = this.pddlParserSettings.delayInSecondsBeforeParsing || this.defaultTimerDelayInSeconds;
        this.timeout = setTimeout(() => this.validateAllDirty(), timerDelayInSeconds * 1000);
    }
    cancelScheduledValidation() {
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
    }
    validateAllDirty() {
        // find all dirty unknown files
        const dirtyUnknowns = this.codePddlWorkspace.pddlWorkspace.getAllFilesIf(fileInfo => fileInfo.isUnknownPddl() && fileInfo.getStatus() === pddl_workspace_2.FileStatus.Parsed);
        // validate unknown files (those where the header does not parse)
        dirtyUnknowns.forEach(file => this.validateUnknownFile(file));
        // find all dirty domains
        const dirtyDomains = this.codePddlWorkspace.pddlWorkspace.getAllFilesIf(fileInfo => fileInfo.isDomain() && fileInfo.getStatus() === pddl_workspace_2.FileStatus.Parsed);
        if (dirtyDomains.length > 0) {
            const firstDirtyDomain = dirtyDomains[0];
            // if there was more than one domain schedule further validation
            const scheduleFurtherValidation = dirtyDomains.length > 1;
            this.validatePddlDocument(firstDirtyDomain, scheduleFurtherValidation);
        }
        // find all dirty problems
        const dirtyProblems = this.codePddlWorkspace.pddlWorkspace.getAllFilesIf(fileInfo => fileInfo.isProblem() && fileInfo.getStatus() === pddl_workspace_2.FileStatus.Parsed);
        if (dirtyProblems.length > 0) {
            const firstDirtyProblem = dirtyProblems[0];
            // if there was more than one domain schedule further validation
            const scheduleFurtherValidation = dirtyProblems.length > 1;
            this.validatePddlDocument(firstDirtyProblem, scheduleFurtherValidation);
        }
        // find all dirty plans
        const dirtyPlans = this.codePddlWorkspace.pddlWorkspace.getAllFilesIf(fileInfo => fileInfo.isPlan() && fileInfo.getStatus() === pddl_workspace_2.FileStatus.Parsed);
        if (dirtyPlans.length > 0) {
            const firstDirtyPlan = dirtyPlans[0];
            // if there was more than one dirty file, schedule further validation
            const scheduleFurtherValidation = dirtyPlans.length > 1;
            this.validatePlan(firstDirtyPlan, scheduleFurtherValidation);
        }
        // find all dirty happenings
        const dirtyHappenings = this.codePddlWorkspace.pddlWorkspace.getAllFilesIf(fileInfo => fileInfo.isHappenings() && fileInfo.getStatus() === pddl_workspace_2.FileStatus.Parsed);
        if (dirtyHappenings.length > 0) {
            const firstDirtyHappenings = dirtyHappenings[0];
            // if there was more than one dirty file, schedule further validation
            const scheduleFurtherValidation = dirtyHappenings.length > 1;
            this.validateHappenings(firstDirtyHappenings, scheduleFurtherValidation);
        }
    }
    handleConfigurationChange() {
        this.pddlParserSettings = this.pddlConfiguration.getParserSettings();
        this.revalidateAll();
    }
    revalidateAll() {
        // mark all files as dirty
        this.codePddlWorkspace.pddlWorkspace.folders.forEach(folder => {
            folder.files
                .forEach(f => {
                if (f.getStatus() !== pddl_workspace_2.FileStatus.Dirty) {
                    f.setStatus(pddl_workspace_2.FileStatus.Parsed);
                }
            });
        });
        // revalidate all files
        this.cancelScheduledValidation(); // ... and validate immediately
        this.validateAllDirty();
    }
    validatePddlDocumentByUri(fileUri, scheduleFurtherValidation) {
        const fileInfo = this.codePddlWorkspace.pddlWorkspace.getFileInfo(fileUri);
        if (!fileInfo) {
            throw new Error(`File not found in the PDDL workspace: ${fileUri}`);
        }
        this.validatePddlDocument(fileInfo, scheduleFurtherValidation);
    }
    validatePlan(planInfo, scheduleFurtherValidation) {
        if (planInfo === null || planInfo === undefined) {
            return;
        }
        if (!this.planValidator.testConfiguration()) {
            return;
        }
        // mark the file as under validation
        planInfo.setStatus(pddl_workspace_2.FileStatus.Validating);
        console.log(`Validating ${planInfo.name} plan.`);
        this.planValidator.validatePlanAndReportDiagnostics(planInfo, false, (diagnostics) => {
            // Send the computed diagnostics to VSCode.
            this.sendDiagnostics(diagnostics);
            planInfo.setStatus(pddl_workspace_2.FileStatus.Validated);
            if (scheduleFurtherValidation) {
                this.scheduleValidation();
            }
        }, (err) => {
            vscode_1.window.showErrorMessage("Error during plan validation: " + err);
            console.warn("Error during plan validation: " + err);
            // var showNever = false;
            // this.pddlConfiguration.suggestNewValidatorConfiguration(showNever);
        });
    }
    validateHappenings(happeningsInfo, scheduleFurtherValidation) {
        if (happeningsInfo === null || happeningsInfo === undefined) {
            return;
        }
        // mark the file as under validation
        happeningsInfo.setStatus(pddl_workspace_2.FileStatus.Validating);
        console.log(`Validating ${happeningsInfo.name} plan.`);
        this.happeningsValidator.validateAndReportDiagnostics(happeningsInfo, false, (diagnostics) => {
            // Send the computed diagnostics to VSCode.
            this.sendDiagnostics(diagnostics);
            happeningsInfo.setStatus(pddl_workspace_2.FileStatus.Validated);
            if (scheduleFurtherValidation) {
                this.scheduleValidation();
            }
        }, (err) => {
            vscode_1.window.showErrorMessage(err);
            console.warn(err);
        });
    }
    validatePddlDocument(fileInfo, scheduleFurtherValidation) {
        if (fileInfo === null || fileInfo === undefined) {
            console.log('File not found in the workspace.');
            return;
        }
        // console.log(`Validating '${fileInfo.fileUri}' file.`);
        // detect parsing and pre-processing issues
        if (fileInfo.getParsingProblems().length > 0) {
            const parsingProblems = new Map();
            parsingProblems.set(fileInfo.fileUri.toString(), validatorUtils_1.toDiagnosticsFromParsingProblems(fileInfo.getParsingProblems()));
            this.sendDiagnostics(parsingProblems);
            return;
        }
        if (fileInfo.isDomain()) {
            const domainInfo = fileInfo;
            const problemFiles = this.codePddlWorkspace.pddlWorkspace.getProblemFiles(domainInfo);
            this.validateDomainAndProblems(domainInfo, problemFiles, scheduleFurtherValidation);
        }
        else if (fileInfo.isProblem()) {
            const problemInfo = fileInfo;
            const domainFile = this.getDomainFileFor(problemInfo);
            if (domainFile) {
                this.validateDomainAndProblems(domainFile, [problemInfo], scheduleFurtherValidation);
            }
        }
        else {
            // this should not happen ?!
        }
    }
    validateDomainAndProblems(domainInfo, problemFiles, scheduleFurtherValidation) {
        if (!this.pddlConfiguration.getParserPath()) {
            // suggest the user to update the settings
            const showNever = true;
            this.pddlConfiguration.suggestNewParserConfiguration(showNever);
            return;
        }
        // mark the files that they are under validation
        domainInfo.setStatus(pddl_workspace_2.FileStatus.Validating);
        problemFiles.forEach(p => p.setStatus(pddl_workspace_2.FileStatus.Validating));
        const validator = this.createValidator(this.pddlConfiguration.getParserPath());
        if (!validator) {
            return;
        }
        validator.validate(domainInfo, problemFiles, (diagnostics) => {
            // Send the computed diagnostics to VSCode.
            this.sendDiagnostics(diagnostics);
            if (scheduleFurtherValidation) {
                this.scheduleValidation();
            }
        }, (err) => {
            vscode_1.window.showErrorMessage(err);
            console.warn(err);
            const showNever = false;
            this.pddlConfiguration.suggestNewParserConfiguration(showNever);
        });
    }
    createValidator(newParserPath) {
        if (!this.validator || this.validator.path !== newParserPath
            || (this.validator instanceof ValidatorExecutable_1.ValidatorExecutable) && (this.validator.syntax !== this.pddlParserSettings.executableOptions ||
                this.validator.customPattern !== this.pddlParserSettings.problemPattern)) {
            if (utils_1.isHttp(newParserPath)) {
                // is a service
                const authentication = new Authentication_1.SAuthentication(this.pddlParserSettings.serviceAuthenticationUrl, this.pddlParserSettings.serviceAuthenticationRequestEncoded, this.pddlParserSettings.serviceAuthenticationClientId, this.pddlParserSettings.serviceAuthenticationCallbackPort, this.pddlParserSettings.serviceAuthenticationTimeoutInMs, this.pddlParserSettings.serviceAuthenticationTokensvcUrl, this.pddlParserSettings.serviceAuthenticationTokensvcApiKey, this.pddlParserSettings.serviceAuthenticationTokensvcAccessPath, this.pddlParserSettings.serviceAuthenticationTokensvcValidatePath, this.pddlParserSettings.serviceAuthenticationTokensvcCodePath, this.pddlParserSettings.serviceAuthenticationTokensvcRefreshPath, this.pddlParserSettings.serviceAuthenticationTokensvcSvctkPath, this.pddlParserSettings.serviceAuthenticationRefreshToken, this.pddlParserSettings.serviceAuthenticationAccessToken, this.pddlParserSettings.serviceAuthenticationSToken);
                return this.validator = new ValidatorService_1.ValidatorService(newParserPath, this.pddlParserSettings.serviceAuthenticationEnabled, authentication);
            }
            else {
                return this.validator = new ValidatorExecutable_1.ValidatorExecutable(newParserPath, this.pddlParserSettings.executableOptions, this.pddlParserSettings.problemPattern);
            }
        }
        else {
            return this.validator;
        }
    }
    sendDiagnostics(diagnostics) {
        diagnostics.forEach((diagnostics, fileUri) => this.diagnosticCollection.set(vscode_1.Uri.parse(fileUri), diagnostics));
    }
    getDomainFileFor(problemFile) {
        try {
            return workspaceUtils_1.getDomainFileForProblem(problemFile, this.codePddlWorkspace);
        }
        catch (err) {
            if (err instanceof workspaceUtils_1.NoDomainAssociated) {
                this.sendDiagnosticInfo(utils_1.toUri(problemFile.fileUri), err.message, workspaceUtils_1.NoDomainAssociated.DIAGNOSTIC_CODE);
                problemFile.setStatus(pddl_workspace_2.FileStatus.Validated);
                return undefined;
            }
            throw err;
        }
    }
    sendDiagnosticInfo(fileUri, message, code) {
        this.sendDiagnostic(fileUri, message, vscode_1.DiagnosticSeverity.Information, code);
    }
    sendDiagnostic(fileUri, message, severity, code) {
        const diagnostic = new vscode_1.Diagnostic(validator_1.Validator.createLineRange(0), message, severity);
        if (code !== undefined && code !== null) {
            diagnostic.code = code;
        }
        this.diagnosticCollection.set(fileUri, [diagnostic]);
    }
    clearDiagnostics(fileUri) {
        this.diagnosticCollection.delete(fileUri);
    }
    validateUnknownFile(fileInfo) {
        fileInfo.setStatus(pddl_workspace_2.FileStatus.Validating);
        const firstLine = pddl_workspace_2.stripComments(fileInfo.getText()).replace(/^\s+/g, '').split('\n')[0];
        this.sendDiagnostic(fileInfo.fileUri, `Cannot recognize whether this is a domain or problem: ${firstLine}`, vscode_1.DiagnosticSeverity.Error, 'CONTENT_NOT_RECOGNIZED');
        fileInfo.setStatus(pddl_workspace_2.FileStatus.Validated);
    }
}
exports.Diagnostics = Diagnostics;
//# sourceMappingURL=Diagnostics.js.map