/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.HappeningsValidator = exports.PDDL_HAPPENINGS_VALIDATE = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const process = require("child_process");
const pddl_workspace_1 = require("pddl-workspace");
const ai_planning_val_1 = require("ai-planning-val");
const path_1 = require("path");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const validatorUtils_1 = require("./validatorUtils");
exports.PDDL_HAPPENINGS_VALIDATE = 'pddl.happenings.validate';
/**
 * Delegate for parsing and validating Plan Happenings files.
 */
class HappeningsValidator {
    constructor(output, codePddlWorkspace, plannerConfiguration, context) {
        this.output = output;
        this.codePddlWorkspace = codePddlWorkspace;
        this.plannerConfiguration = plannerConfiguration;
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.PDDL_HAPPENINGS_VALIDATE, async () => {
            if (vscode_1.window.activeTextEditor && workspaceUtils_1.isHappenings(vscode_1.window.activeTextEditor.document)) {
                try {
                    const outcome = await this.validateTextDocument(vscode_1.window.activeTextEditor.document);
                    if (outcome.getError()) {
                        vscode_1.window.showErrorMessage(outcome.getError());
                    }
                }
                catch (ex) {
                    vscode_1.window.showErrorMessage("Happenings validation failed: " + ex);
                    return;
                }
            }
            else {
                vscode_1.window.showErrorMessage("There is no happenings file open.");
                return;
            }
        }));
    }
    async validateTextDocument(happeningsDocument) {
        const planFileInfo = await this.codePddlWorkspace.upsertAndParseFile(happeningsDocument);
        if (!planFileInfo) {
            throw new Error("Cannot open or parse plan file: " + happeningsDocument.uri.fsPath);
        }
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        return this.validateAndReportDiagnostics(planFileInfo, true, () => { }, () => { });
    }
    async validateAndReportDiagnostics(happeningsInfo, showOutput, onSuccess, onError) {
        if (happeningsInfo.getParsingProblems().length > 0) {
            const diagnostics = happeningsInfo.getParsingProblems()
                .map(problem => validatorUtils_1.createDiagnosticFromParsingProblem(problem));
            const outcome = HappeningsValidationOutcome.failedWithDiagnostics(happeningsInfo, diagnostics);
            onSuccess(outcome.getDiagnostics());
            return outcome;
        }
        let context = null;
        try {
            context = workspaceUtils_1.getDomainAndProblemForHappenings(happeningsInfo, this.codePddlWorkspace.pddlWorkspace);
        }
        catch (err) {
            const outcome = HappeningsValidationOutcome.info(happeningsInfo, err);
            onSuccess(outcome.getDiagnostics());
            return outcome;
        }
        // are the actions in the plan declared in the domain?
        const actionNameDiagnostics = this.validateActionNames(context.domain, context.problem, happeningsInfo);
        if (actionNameDiagnostics.length) {
            const errorOutcome = HappeningsValidationOutcome.failedWithDiagnostics(happeningsInfo, actionNameDiagnostics);
            onSuccess(errorOutcome.getDiagnostics());
            return errorOutcome;
        }
        // are the actions start times monotonically increasing?
        const actionTimeDiagnostics = this.validateActionTimes(happeningsInfo);
        if (actionTimeDiagnostics.length) {
            const errorOutcome = HappeningsValidationOutcome.failedWithDiagnostics(happeningsInfo, actionTimeDiagnostics);
            onSuccess(errorOutcome.getDiagnostics());
            return errorOutcome;
        }
        const valStepPath = await this.plannerConfiguration.getValStepPath();
        //const valVerbose = this.plannerConfiguration.getValStepVerbose();
        if (!valStepPath) {
            onSuccess(HappeningsValidationOutcome.unknown(happeningsInfo, "ValStep not configured.").getDiagnostics());
        }
        // copy editor content to temp files to avoid using out-of-date content on disk
        const domainFilePath = await ai_planning_val_1.Util.toPddlFile('domain', context.domain.getText());
        const problemFilePath = await ai_planning_val_1.Util.toPddlFile('problem', context.problem.getText());
        const happeningsConverter = new ai_planning_val_1.HappeningsToValStep();
        happeningsConverter.convertAllHappenings(happeningsInfo);
        const valSteps = happeningsConverter.getExportedText(true);
        const args = [domainFilePath, problemFilePath];
        const child = process.spawnSync(valStepPath, args, { cwd: path_1.dirname(happeningsInfo.fileUri.fsPath), input: valSteps });
        if (showOutput) {
            this.output.appendLine(valStepPath + ' ' + args.join(' '));
        }
        const output = child.stdout.toString();
        if (showOutput) {
            this.output.appendLine(output);
        }
        if (showOutput && child.stderr && child.stderr.length) {
            this.output.append('Error:');
            this.output.appendLine(child.stderr.toString());
        }
        const outcome = this.analyzeOutput(happeningsInfo, child.error, output);
        if (child.error) {
            if (showOutput) {
                this.output.appendLine(`Error: name=${child.error.name}, message=${child.error.message}`);
            }
            onError(child.error.name);
        }
        else {
            onSuccess(outcome.getDiagnostics());
        }
        if (showOutput) {
            this.output.appendLine(`Exit code: ${child.status}`);
            this.output.show();
        }
        return outcome;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    analyzeOutput(happeningsInfo, error, _output) {
        if (error) {
            return HappeningsValidationOutcome.failed(happeningsInfo, error);
        }
        return HappeningsValidationOutcome.valid(happeningsInfo);
    }
    /**
     * Validate that plan steps match domain actions
     * @param domain domain file
     * @param problem problem file
     * @param happeningsInfo happeningsInfo
     */
    validateActionNames(domain, problem, happeningsInfo) {
        return happeningsInfo.getHappenings()
            .filter(happening => !this.isDomainAction(domain, problem, happening))
            .map(happening => { var _a; return new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine((_a = happening.lineIndex) !== null && _a !== void 0 ? _a : 0), `Action '${happening.getAction()}' not known by the domain '${domain.name}'`, vscode_1.DiagnosticSeverity.Error); });
    }
    /**
     * Validate that plan step times are monotonically increasing
     * @param domain domain file
     * @param problem problem file
     * @param happeningsInfo happeningsInfo
     */
    validateActionTimes(happeningsInfo) {
        return happeningsInfo.getHappenings()
            .slice(1)
            .filter((happening, index) => !this.isTimeMonotonociallyIncreasing(happeningsInfo.getHappenings()[index], happening))
            .map(happening => { var _a; return new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine((_a = happening.lineIndex) !== null && _a !== void 0 ? _a : 0), `Action '${happening.getAction()}' time ${happening.getTime()} is before the preceding action time`, vscode_1.DiagnosticSeverity.Error); });
    }
    isDomainAction(domain, problem, happening) {
        const allActionNames = domain.getActions().map(a => { var _a, _b; return (_b = (_a = a.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()) !== null && _b !== void 0 ? _b : 'undefined'; }).concat(problem.getSupplyDemands().map(sd => sd.getName().toLowerCase()));
        return allActionNames.includes(happening.getAction().toLowerCase());
    }
    isTimeMonotonociallyIncreasing(first, second) {
        return first.getTime() <= second.getTime();
    }
}
exports.HappeningsValidator = HappeningsValidator;
class HappeningsValidationOutcome {
    constructor(happeningsInfo, diagnostics, error) {
        this.happeningsInfo = happeningsInfo;
        this.diagnostics = diagnostics;
        this.error = error;
    }
    getError() {
        return this.error;
    }
    getDiagnostics() {
        const diagnostics = new Map();
        diagnostics.set(this.happeningsInfo.fileUri.toString(), this.diagnostics);
        return diagnostics;
    }
    static goalNotAttained(happeningsInfo) {
        var _a, _b;
        const errorLine = happeningsInfo.getHappenings().length > 0 ? (_b = (_a = happeningsInfo.getHappenings().slice(-1).pop()) === null || _a === void 0 ? void 0 : _a.lineIndex) !== null && _b !== void 0 ? _b : 0 + 1 : 0;
        const error = "Plan does not reach the goal.";
        const diagnostics = [validatorUtils_1.createDiagnostic(errorLine, 0, error, vscode_1.DiagnosticSeverity.Warning)];
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics, error);
    }
    /**
     * Creates validation outcomes for invalid plan i.e. plans that do not parse or do not correspond to the domain/problem file.
     */
    static invalidPlanDescription(happeningsInfo) {
        const error = "Invalid plan description.";
        const diagnostics = [validatorUtils_1.createDiagnostic(0, 0, error, vscode_1.DiagnosticSeverity.Error)];
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics, error);
    }
    /**
     * Creates validation outcomes for valid plan, which does not reach the goal.
     */
    static valid(happeningsInfo) {
        return new HappeningsValidationOutcome(happeningsInfo, [], undefined);
    }
    static failed(happeningsInfo, error) {
        const message = "Validate tool failed. " + error.message;
        const diagnostics = [validatorUtils_1.createDiagnostic(0, 0, message, vscode_1.DiagnosticSeverity.Error)];
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics, message);
    }
    static info(happeningsInfo, error) {
        const message = error.message;
        const diagnostics = [validatorUtils_1.createDiagnostic(0, 0, message, vscode_1.DiagnosticSeverity.Information)];
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics, message);
    }
    static failedWithDiagnostics(happeningsInfo, diagnostics) {
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics);
    }
    static failedAtTime(happeningsInfo, timeStamp, repairHints) {
        let errorLine = 0;
        const stepAtTimeStamp = happeningsInfo.getHappenings()
            .find(happening => pddl_workspace_1.PlanStep.equalsWithin(happening.getTime(), timeStamp, 1e-4));
        if (stepAtTimeStamp && stepAtTimeStamp.lineIndex !== undefined) {
            errorLine = stepAtTimeStamp.lineIndex;
        }
        const diagnostics = repairHints.map(hint => new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine(errorLine), hint, vscode_1.DiagnosticSeverity.Warning));
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics);
    }
    static unknown(happeningsInfo, message = "Unknown error.") {
        const diagnostics = [new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine(0), message, vscode_1.DiagnosticSeverity.Warning)];
        return new HappeningsValidationOutcome(happeningsInfo, diagnostics, message);
    }
}
//# sourceMappingURL=HappeningsValidator.js.map