/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanValidator = exports.PDDL_PLAN_VALIDATE = void 0;
const vscode_1 = require("vscode");
const process = require("child_process");
const pddl_workspace_1 = require("pddl-workspace");
const ai_planning_val_1 = require("ai-planning-val");
const path_1 = require("path");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const utils_1 = require("../utils");
const valCommand_1 = require("../validation/valCommand");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const validatorUtils_1 = require("./validatorUtils");
exports.PDDL_PLAN_VALIDATE = 'pddl.plan.validate';
/**
 * Delegate for parsing and validating plans..
 */
class PlanValidator {
    constructor(output, codePddlWorkspace, plannerConfiguration, context) {
        this.output = output;
        this.codePddlWorkspace = codePddlWorkspace;
        this.plannerConfiguration = plannerConfiguration;
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.PDDL_PLAN_VALIDATE, async (planUri) => this.validateActiveDocument(planUri).catch(utils_1.showError)));
    }
    async validateActiveDocument(planUri) {
        let planDocument;
        if (planUri) {
            planDocument = await vscode_1.workspace.openTextDocument(planUri);
        }
        else {
            if (vscode_1.window.activeTextEditor) {
                planDocument = vscode_1.window.activeTextEditor.document;
            }
            else {
                throw new Error(`No active editor and no document uri provided.`);
            }
        }
        if (!workspaceUtils_1.isPlan(planDocument)) {
            return;
        }
        if (planDocument) {
            if (!await this.testConfiguration()) {
                return;
            }
            try {
                const outcome = await this.validatePlanDocument(planDocument);
                if (outcome.getError()) {
                    // do not open the _Problems_ pane, unless you can direct the error there
                    // commands.executeCommand('workbench.actions.view.problems');
                    throw new Error(outcome.getError());
                }
            }
            catch (ex) {
                console.error(ex);
                throw new Error("Plan validation failed: " + ex);
            }
        }
        else {
            throw new Error("There is no plan file open.");
        }
    }
    async testConfiguration() {
        const validatePath = this.plannerConfiguration.getValidatorPath();
        if (!validatePath || validatePath.length === 0) {
            vscode_1.commands.executeCommand(valCommand_1.VAL_DOWNLOAD_COMMAND);
            return false;
        }
        else {
            return true;
        }
    }
    async validatePlanDocument(planDocument) {
        const planFileInfo = await this.codePddlWorkspace.upsertAndParseFile(planDocument);
        if (!planFileInfo) {
            throw new Error("Cannot open or parse plan file from: " + planDocument.uri.fsPath);
        }
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        return this.validatePlanAndReportDiagnostics(planFileInfo, true, () => { }, () => { });
    }
    async validatePlanAndReportDiagnostics(planInfo, showOutput, onSuccess, onError) {
        const epsilon = this.plannerConfiguration.getEpsilonTimeStep();
        const validatePath = this.plannerConfiguration.getValidatorPath();
        if (!validatePath) {
            throw new Error(`Validate executable path not configured.`);
        }
        let context;
        try {
            context = workspaceUtils_1.getDomainAndProblemForPlan(planInfo, this.codePddlWorkspace.pddlWorkspace);
        }
        catch (err) {
            const outcome = PlanValidationOutcome.failed(planInfo, err);
            onSuccess(outcome.getDiagnostics());
            return outcome;
        }
        // are the actions in the plan declared in the domain?
        const actionNameDiagnostics = this.validateActionNames(context.domain, context.problem, planInfo);
        if (actionNameDiagnostics.length) {
            const errorOutcome = PlanValidationOutcome.failedWithDiagnostics(planInfo, actionNameDiagnostics);
            onSuccess(errorOutcome.getDiagnostics());
            return errorOutcome;
        }
        // are the actions start times monotonically increasing?
        const actionTimeDiagnostics = this.validateActionTimes(planInfo);
        if (actionTimeDiagnostics.length) {
            const errorOutcome = PlanValidationOutcome.failedWithDiagnostics(planInfo, actionTimeDiagnostics);
            onSuccess(errorOutcome.getDiagnostics());
            return errorOutcome;
        }
        // copy editor content to temp files to avoid using out-of-date content on disk
        const domainFilePath = await ai_planning_val_1.Util.toPddlFile('domain', context.domain.getText());
        const problemFilePath = await ai_planning_val_1.Util.toPddlFile('problem', context.problem.getText());
        const planFilePath = await ai_planning_val_1.Util.toPddlFile('plan', planInfo.getText());
        const args = ['-t', epsilon.toString(), '-v', domainFilePath, problemFilePath, planFilePath];
        const workingDir = this.createWorkingFolder(planInfo.fileUri);
        const child = process.spawnSync(validatePath, args, { cwd: workingDir });
        if (showOutput) {
            this.output.appendLine(validatePath + ' ' + args.join(' '));
        }
        let outcome;
        if (child.error) {
            if (showOutput) {
                this.output.appendLine(`Error: name=${child.error.name}, message=${child.error.message}`);
            }
            onError(child.error.name);
            outcome = PlanValidationOutcome.failed(planInfo, child.error);
            onSuccess(outcome.getDiagnostics());
        }
        else {
            const output = child.stdout.toString();
            if (showOutput) {
                this.output.appendLine(output);
            }
            if (showOutput && child.stderr && child.stderr.length) {
                this.output.append('Error:');
                this.output.appendLine(child.stderr.toString());
            }
            outcome = this.analyzeOutput(planInfo, child.stderr.toString(), child.error, output);
            onSuccess(outcome.getDiagnostics());
        }
        if (showOutput) {
            this.output.appendLine(`Exit code: ${child.status}`);
            this.output.show();
        }
        return outcome;
    }
    createWorkingFolder(planUri) {
        if (planUri.scheme === "file") {
            return path_1.dirname(planUri.fsPath);
        }
        const workspaceFolder = vscode_1.workspace.getWorkspaceFolder(planUri);
        if (workspaceFolder) {
            return workspaceFolder.uri.fsPath;
        }
        if (vscode_1.workspace.workspaceFolders && vscode_1.workspace.workspaceFolders.length > 0) {
            return vscode_1.workspace.workspaceFolders[0].uri.fsPath;
        }
        return ".";
    }
    analyzeOutput(planInfo, stderr, error, output) {
        if (error) {
            return PlanValidationOutcome.failed(planInfo, error);
        }
        if (output.match("Plan failed to execute") || output.match("Goal not satisfied")) {
            const failurePattern = /Checking next happening \(time (\d+.\d+)\)/g;
            let result;
            let timeStamp = -1;
            while ((result = failurePattern.exec(output)) !== null) {
                timeStamp = parseFloat(result[1]);
            }
            const match = output.match(/Plan Repair Advice:([\s\S]+)Failed plans:/);
            if (match) {
                return PlanValidationOutcome.failedAtTime(planInfo, timeStamp, match[1].trim().split('\n'));
            }
            else {
                return PlanValidationOutcome.failedAtTime(planInfo, timeStamp, ["Unidentified error. Run the 'PDDL: Validate plan' command for more info."]);
            }
        }
        if (output.match("Bad plan description!")) {
            return PlanValidationOutcome.invalidPlanDescription(planInfo);
        }
        else if (output.match("Plan valid")) {
            return PlanValidationOutcome.valid(planInfo);
        }
        if (stderr === null || stderr === void 0 ? void 0 : stderr.trim()) {
            return PlanValidationOutcome.otherError(planInfo, stderr.trim());
        }
        return PlanValidationOutcome.unknown(planInfo);
    }
    /**
     * Validate that plan steps match domain actions
     * @param domain domain file
     * @param problem problem file
     * @param plan plan
     */
    validateActionNames(domain, problem, plan) {
        return plan.getSteps()
            .filter(step => !this.isDomainAction(domain, problem, step))
            .map(step => { var _a; return new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine((_a = step.lineIndex) !== null && _a !== void 0 ? _a : 0), `Action '${step.getActionName()}' not known by the domain '${domain.name}'`, vscode_1.DiagnosticSeverity.Error); });
    }
    /**
     * Validate that plan step times are monotonically increasing
     * @param domain domain file
     * @param problem problem file
     * @param plan plan
     */
    validateActionTimes(plan) {
        return plan.getSteps()
            .slice(1)
            .filter((step, index) => !this.isTimeMonotonicallyIncreasing(plan.getSteps()[index], step))
            .map(step => { var _a; return new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine((_a = step.lineIndex) !== null && _a !== void 0 ? _a : 0), `Action '${step.getActionName()}' time ${step.getStartTime()} is before the preceding action time`, vscode_1.DiagnosticSeverity.Error); });
    }
    isDomainAction(domain, problem, step) {
        const allActionNames = domain.getActions().map(a => { var _a, _b; return (_b = (_a = a.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()) !== null && _b !== void 0 ? _b : 'undefined'; }).concat(problem.getSupplyDemands().map(sd => sd.getName().toLowerCase()));
        return allActionNames.includes(step.getActionName().toLowerCase());
    }
    isTimeMonotonicallyIncreasing(first, second) {
        return first.getStartTime() <= second.getStartTime();
    }
}
exports.PlanValidator = PlanValidator;
class PlanValidationOutcome {
    constructor(planInfo, diagnostics, error) {
        this.planInfo = planInfo;
        this.diagnostics = diagnostics;
        this.error = error;
    }
    getError() {
        return this.error;
    }
    getDiagnostics() {
        const diagnostics = new Map();
        diagnostics.set(this.planInfo.fileUri.toString(), this.diagnostics);
        return diagnostics;
    }
    static goalNotAttained(planInfo) {
        var _a, _b;
        const errorLine = planInfo.getSteps().length > 0 ? ((_b = (_a = planInfo.getSteps().slice(-1).pop()) === null || _a === void 0 ? void 0 : _a.lineIndex) !== null && _b !== void 0 ? _b : -1) + 1 : 0;
        const error = "Plan does not reach the goal.";
        const diagnostics = [validatorUtils_1.createDiagnostic(errorLine, 0, error, vscode_1.DiagnosticSeverity.Warning)];
        return new PlanValidationOutcome(planInfo, diagnostics, error);
    }
    /**
     * Creates validation outcomes for invalid plan i.e. plans that do not parse or do not correspond to the domain/problem file.
     */
    static invalidPlanDescription(planInfo) {
        const error = "Invalid plan description.";
        const diagnostics = [validatorUtils_1.createDiagnostic(0, 0, error, vscode_1.DiagnosticSeverity.Error)];
        return new PlanValidationOutcome(planInfo, diagnostics, error);
    }
    /**
     * Creates validation outcomes for valid plan, which does not reach the goal.
     */
    static valid(planInfo) {
        return new PlanValidationOutcome(planInfo, [], undefined);
    }
    static failed(planInfo, error) {
        const message = "Validate tool failed. " + error.message;
        const diagnostic = validatorUtils_1.createDiagnostic(0, 0, message, vscode_1.DiagnosticSeverity.Error);
        if (error instanceof workspaceUtils_1.NoProblemAssociated) {
            diagnostic.code = workspaceUtils_1.NoProblemAssociated.DIAGNOSTIC_CODE;
        }
        else if (error instanceof workspaceUtils_1.NoDomainAssociated) {
            diagnostic.code = workspaceUtils_1.NoDomainAssociated.DIAGNOSTIC_CODE;
        }
        return new PlanValidationOutcome(planInfo, [diagnostic], message);
    }
    static failedWithDiagnostics(planInfo, diagnostics) {
        return new PlanValidationOutcome(planInfo, diagnostics);
    }
    static failedAtTime(planInfo, timeStamp, repairHints) {
        let errorLine = 0;
        const stepAtTimeStamp = planInfo.getSteps()
            .find(step => pddl_workspace_1.PlanStep.equalsWithin(step.getStartTime(), timeStamp, 1e-4));
        if (stepAtTimeStamp && stepAtTimeStamp.lineIndex !== undefined) {
            errorLine = stepAtTimeStamp.lineIndex;
        }
        const diagnostics = repairHints.map(hint => new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine(errorLine), hint, vscode_1.DiagnosticSeverity.Warning));
        return new PlanValidationOutcome(planInfo, diagnostics);
    }
    static otherError(planInfo, error) {
        const diagnostics = [new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine(0), `${error}. Run the 'PDDL: Validate plan' command for more information.`, vscode_1.DiagnosticSeverity.Error)];
        return new PlanValidationOutcome(planInfo, diagnostics, error);
    }
    static unknown(planInfo) {
        const diagnostics = [new vscode_1.Diagnostic(validatorUtils_1.createRangeFromLine(0), "Unknown error. Run the 'PDDL: Validate plan' command for more information.", vscode_1.DiagnosticSeverity.Warning)];
        return new PlanValidationOutcome(planInfo, diagnostics, "Unknown error.");
    }
}
//# sourceMappingURL=PlanValidator.js.map