/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidatorExecutable = void 0;
const validator_1 = require("./validator");
const ai_planning_val_1 = require("ai-planning-val");
const ai_planning_val_2 = require("ai-planning-val");
const validatorUtils_1 = require("./validatorUtils");
class ValidatorExecutable extends validator_1.Validator {
    constructor(path, syntax, customPattern) {
        super(path);
        this.syntax = syntax;
        this.customPattern = customPattern;
        this.parser = new ConfiguredParser({ executablePath: this.path, customPattern, syntax });
    }
    async validate(domainInfo, problemFiles, onSuccess, onError) {
        if (!problemFiles.length) {
            try {
                const parsingErrors = await this.parser.validate(domainInfo);
                onSuccess.apply(this, [validatorUtils_1.toDiagnostics(parsingErrors, [domainInfo])]);
            }
            catch (err) {
                onError.apply(this, [err]);
            }
        }
        else {
            problemFiles.forEach(async (problemFile) => {
                // todo: the issues in the domain file should only be output once, not as many times as there are problem files
                try {
                    const parsingErrors = await this.parser.validate(domainInfo, problemFile);
                    onSuccess.apply(this, [validatorUtils_1.toDiagnostics(parsingErrors, [domainInfo, problemFile])]);
                }
                catch (err) {
                    onError.apply(this, [err]);
                }
            });
        }
    }
}
exports.ValidatorExecutable = ValidatorExecutable;
class ConfiguredParser extends ai_planning_val_2.Parser {
    constructor(options) {
        super(options);
        this.customPattern = options.customPattern;
        this.args = options.syntax.split(' ');
    }
    getSyntax() {
        return this.args;
    }
    createPatternMatchers(context) {
        const filePaths = context.fileNameMap.getFilePaths();
        const patterns = [
            // popf pattern
            new ai_planning_val_1.ProblemPattern(`/^($(filePaths))\\s*:\\s*line\\s*:\\s*(\\d*)\\s*:\\s*(Error|Warning)\\s*:\\s*(.*)$/gmi/1,3,2,0,4`, filePaths),
            // pddl4j pattern
            new ai_planning_val_1.ProblemPattern(`/(error|warning) at line (\\d+), column (\\d+), file \\(($(filePaths))\\)\\s*:\\s*(.+)/ig/4,1,2,3,5`, filePaths),
        ];
        if (this.customPattern) {
            patterns.push(new ai_planning_val_1.ProblemPattern(this.customPattern, filePaths));
        }
        return patterns;
    }
}
//# sourceMappingURL=ValidatorExecutable.js.map