/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PddlFormatProvider = void 0;
const pddl_workspace_1 = require("pddl-workspace");
const vscode_1 = require("vscode");
const utils_1 = require("../utils");
const PddlOnTypeFormatter_1 = require("./PddlOnTypeFormatter");
class PddlFormatProvider {
    constructor(pddlWorkspace) {
        this.pddlWorkspace = pddlWorkspace;
    }
    async provideDocumentRangeFormattingEdits(document, range, options, token) {
        var _a;
        const fileInfo = await ((_a = this.pddlWorkspace) === null || _a === void 0 ? void 0 : _a.upsertAndParseFile(document));
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (fileInfo && (fileInfo.getLanguage() !== pddl_workspace_1.PddlLanguage.PDDL)) {
            return undefined;
        }
        const tree = this.getSyntaxTree(fileInfo, document);
        return new PddlFormatter(document, range, options, token).format(tree.getRootNode());
    }
    async provideDocumentFormattingEdits(document, options, token) {
        var _a;
        const fileInfo = await ((_a = this.pddlWorkspace) === null || _a === void 0 ? void 0 : _a.upsertAndParseFile(document));
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (fileInfo && (fileInfo.getLanguage() !== pddl_workspace_1.PddlLanguage.PDDL)) {
            return undefined;
        }
        const tree = this.getSyntaxTree(fileInfo, document);
        const fullRange = document.validateRange(new vscode_1.Range(new vscode_1.Position(0, 0), new vscode_1.Position(document.lineCount, Number.MAX_VALUE)));
        return new PddlFormatter(document, fullRange, options, token).format(tree.getRootNode());
    }
    getSyntaxTree(fileInfo, document) {
        let tree;
        if (fileInfo && (fileInfo instanceof pddl_workspace_1.DomainInfo)) {
            tree = fileInfo.syntaxTree;
        }
        else if (fileInfo && (fileInfo instanceof pddl_workspace_1.ProblemInfo)) {
            tree = fileInfo.syntaxTree;
        }
        else {
            tree = new pddl_workspace_1.parser.PddlSyntaxTreeBuilder(document.getText()).getTree();
        }
        return tree;
    }
}
exports.PddlFormatProvider = PddlFormatProvider;
class PddlFormatter {
    constructor(document, range, options, token) {
        this.document = document;
        this.options = options;
        this.token = token;
        this.edits = [];
        this.firstOffset = document.offsetAt(range.start);
        this.lastOffset = document.offsetAt(range.end);
    }
    getLastChild(node) {
        let children = node === null || node === void 0 ? void 0 : node.getChildren();
        while (children && (children.length > 0)) {
            node = children[children.length - 1];
            children = node === null || node === void 0 ? void 0 : node.getChildren();
        }
        return node;
    }
    format(node) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        if (node.getStart() > this.lastOffset || this.token.isCancellationRequested) {
            return this.edits;
        }
        const nextSibling = (_a = node.getFollowingSibling()) !== null && _a !== void 0 ? _a : (_b = node.getParent()) === null || _b === void 0 ? void 0 : _b.getFollowingSibling(undefined, node);
        const precedingSibling = node.getPrecedingSibling();
        const precedingNode = precedingSibling && this.getLastChild(precedingSibling);
        if (node.getStart() >= this.firstOffset || node.includesIndex(this.firstOffset)) {
            if (node.isType(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator)) {
                if (precedingNode === null || precedingNode === void 0 ? void 0 : precedingNode.isNotType(pddl_workspace_1.parser.PddlTokenType.Whitespace)) {
                    this.breakAndIndent(node);
                }
            }
            else if (node.getToken().type === pddl_workspace_1.parser.PddlTokenType.CloseBracket) {
                if (precedingNode === null || precedingNode === void 0 ? void 0 : precedingNode.isNotType(pddl_workspace_1.parser.PddlTokenType.Whitespace)) {
                    const openBracketToken = (_d = (_c = node.getParent()) === null || _c === void 0 ? void 0 : _c.getToken().tokenText) !== null && _d !== void 0 ? _d : '';
                    if (['(:requirements', '(:domain'].includes(openBracketToken)) {
                        // do nothing
                    }
                    else if (openBracketToken.startsWith('(:')
                        || ['(define'].includes(openBracketToken)) {
                        this.breakAndIndent(node, -1);
                    }
                }
            }
            else if (node.isType(pddl_workspace_1.parser.PddlTokenType.Whitespace)) {
                if (node.getParent() && ['(increase', '(decrease', '(assign'].includes(node.getParent().getToken().tokenText)) {
                    if (((_f = (_e = node.getParent()) === null || _e === void 0 ? void 0 : _e.length) !== null && _f !== void 0 ? _f : 0) > 50) {
                        this.breakAndIndent(node);
                    }
                    else {
                        this.replace(node, ' ');
                    }
                }
                else if (nextSibling === null || nextSibling === void 0 ? void 0 : nextSibling.isType(pddl_workspace_1.parser.PddlTokenType.Comment)) {
                    if (node.getText().includes('\n')) {
                        this.breakAndIndent(node);
                    }
                    else {
                        this.replace(node, ' ');
                    }
                }
                else if (node.getParent() && ['(:types', '(:objects', '(:constants'].includes(node.getParent().getToken().tokenText)) {
                    if ((nextSibling === null || nextSibling === void 0 ? void 0 : nextSibling.isType(pddl_workspace_1.parser.PddlTokenType.Dash)) || (precedingSibling === null || precedingSibling === void 0 ? void 0 : precedingSibling.isType(pddl_workspace_1.parser.PddlTokenType.Dash))) {
                        this.replace(node, ' ');
                    }
                    else if (precedingSibling === null || precedingSibling === void 0 ? void 0 : precedingSibling.isType(pddl_workspace_1.parser.PddlTokenType.Comment)) {
                        this.breakAndIndent(node);
                    }
                    else if (nextSibling === null || nextSibling === void 0 ? void 0 : nextSibling.isType(pddl_workspace_1.parser.PddlTokenType.CloseBracket)) {
                        this.breakAndIndent(node, -1);
                    }
                    else {
                        const precedingNodes = node.getPrecedingSiblings()
                            .filter(n => n.isNoneOf([pddl_workspace_1.parser.PddlTokenType.Comment, pddl_workspace_1.parser.PddlTokenType.Whitespace]));
                        if (precedingNodes.length === 0) {
                            this.breakAndIndent(node);
                        }
                        else if (precedingNodes.length > 2 && this.areTypes(precedingNodes.slice(-2), [pddl_workspace_1.parser.PddlTokenType.Dash, pddl_workspace_1.parser.PddlTokenType.Other])) {
                            this.breakAndIndent(node);
                        }
                        else {
                            this.replace(node, ' ');
                        }
                    }
                }
                else if (nextSibling === undefined) {
                    this.replace(node, '');
                }
                else if (nextSibling.isType(pddl_workspace_1.parser.PddlTokenType.CloseBracket)) {
                    if (node.getText().includes('\n')) {
                        this.breakAndIndent(node, -1);
                    }
                    else {
                        this.replace(node, '');
                    }
                }
                else if (nextSibling.isAnyOf([pddl_workspace_1.parser.PddlTokenType.Dash, pddl_workspace_1.parser.PddlTokenType.Other, pddl_workspace_1.parser.PddlTokenType.Parameter])) {
                    this.replace(node, ' ');
                }
                else if (nextSibling.isAnyOf([pddl_workspace_1.parser.PddlTokenType.OpenBracket, pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, pddl_workspace_1.parser.PddlTokenType.Keyword])) {
                    if (nextSibling.isType(pddl_workspace_1.parser.PddlTokenType.Keyword)) {
                        const requirementsAncestor = node.findAncestor(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /:requirements/);
                        if (requirementsAncestor) {
                            this.replace(node, ' ');
                        }
                        else {
                            this.breakAndIndent(node);
                        }
                    }
                    else if (((_g = node.getParent()) === null || _g === void 0 ? void 0 : _g.isNumericExpression()) || ((_h = node.getParent()) === null || _h === void 0 ? void 0 : _h.isLogicalExpression()) || ((_j = node.getParent()) === null || _j === void 0 ? void 0 : _j.isTemporalExpression())) {
                        if (node.getText().includes('\n')) {
                            this.breakAndIndent(node);
                        }
                        else {
                            this.replace(node, ' ');
                        }
                    }
                    else if (['(domain', '(problem'].includes(nextSibling.getToken().tokenText)) {
                        this.replace(node, ' ');
                    }
                    else {
                        if (node.getParent()
                            && [':parameters', ':duration', ':precondition', ':condition', ':effect'].includes(node.getParent().getToken().tokenText)) {
                            this.replace(node, ' ');
                        }
                        else {
                            this.breakAndIndent(node);
                        }
                    }
                }
            }
        }
        node.getChildren().forEach(child => this.format(child));
        return this.edits;
    }
    areTypes(nodes, expectedTypes) {
        if (nodes.length !== expectedTypes.length) {
            throw new Error(`argument lengths are not matching`);
        }
        const actualTypes = nodes.map(n => n.getToken().type);
        for (let i = 0; i < actualTypes.length; i++) {
            if (actualTypes[i] !== expectedTypes[i]) {
                return false;
            }
        }
        return true;
    }
    breakAndIndent(node, offset = 0) {
        const level = node.getAncestors([pddl_workspace_1.parser.PddlTokenType.OpenBracket, pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator]).length;
        if (node.isType(pddl_workspace_1.parser.PddlTokenType.Whitespace)) {
            this.replace(node, this.ends(node.getText(), { min: 1, max: 2 }) + PddlOnTypeFormatter_1.PddlOnTypeFormatter.createIndent('', level + offset, this.options));
        }
        else {
            const newText = this.eol() + PddlOnTypeFormatter_1.PddlOnTypeFormatter.createIndent('', level + offset, this.options);
            this.edits.push(vscode_1.TextEdit.insert(this.document.positionAt(node.getStart()), newText));
        }
    }
    replace(node, newText) {
        this.edits.push(vscode_1.TextEdit.replace(utils_1.nodeToRange(this.document, node), newText));
    }
    /** @returns the endline characters only, but at least the `min` count */
    ends(text, options) {
        var _a;
        // strip off all other characters than the bare `\n`
        let endls = text.replace(/[^\n]/g, '');
        let endlCount = (endls.match(/\n/g) || []).length;
        // remove excess line breaks
        while (endlCount > ((_a = options.max) !== null && _a !== void 0 ? _a : Number.MAX_VALUE)) {
            endls = endls.replace('\n', '');
            endlCount--;
        }
        // replace all `\n` by the actual document's eol sequence
        endls = endls.split('\n').join(this.eol());
        if (endlCount < options.min) {
            return endls + this.eol().repeat(options.min - endlCount);
        }
        else {
            return endls;
        }
    }
    eol() {
        switch (this.document.eol) {
            case vscode_1.EndOfLine.LF:
                return '\n';
            case vscode_1.EndOfLine.CRLF:
                return '\r\n';
        }
    }
}
//# sourceMappingURL=PddlFormatProvider.js.map