/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.OverviewPage = exports.LAST_SHOWN_OVERVIEW_PAGE = exports.SHOULD_SHOW_OVERVIEW_PAGE = void 0;
const vscode_1 = require("vscode");
const configuration_1 = require("../configuration/configuration");
const path = require("path");
const utils_1 = require("../utils");
const valCommand_1 = require("../validation/valCommand");
const PTestCommands_1 = require("../ptest/PTestCommands");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const PlannersConfiguration_1 = require("../configuration/PlannersConfiguration");
const workspaceFs_1 = require("../util/workspaceFs");
const webviewUtils_1 = require("../webviewUtils");
exports.SHOULD_SHOW_OVERVIEW_PAGE = 'shouldShowOverviewPage';
exports.LAST_SHOWN_OVERVIEW_PAGE = 'lastShownOverviewPage';
const LATER = 'LATER';
const NEVER = 'NEVER';
const ACCEPTED = 'ACCEPTED';
class OverviewPage {
    constructor(context, pddlConfiguration, plannersConfiguration, val) {
        this.context = context;
        this.pddlConfiguration = pddlConfiguration;
        this.plannersConfiguration = plannersConfiguration;
        this.val = val;
        this.iconsInstalled = false;
        this.ICONS_EXTENSION_NAME = "vscode-icons-team.vscode-icons";
        this.NEXT_TIP_TO_SHOW = 'nextTipToShow';
        this.ACCEPTED_TO_WRITE_A_REVIEW = 'acceptedToWriteAReview';
        this.VIEWS = "views";
        this.CONTENT_FOLDER = path.join(this.VIEWS, "overview");
        this.COMMON_FOLDER = path.join(this.VIEWS, "common");
        this.tipDisplayed = 0;
        vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.showOverview", () => this.showWelcomePage(true));
        vscode_1.workspace.onDidChangeConfiguration(() => this.scheduleUpdatePageConfiguration(), undefined, this.context.subscriptions);
        vscode_1.workspace.onDidChangeWorkspaceFolders(() => this.scheduleUpdatePageConfiguration(), undefined, this.context.subscriptions);
        vscode_1.extensions.onDidChange(() => this.updateIconsAlerts(), this.context.subscriptions);
        this.updateIconsAlerts();
    }
    async showWelcomePage(showAnyway) {
        if (this.webViewPanel) {
            this.webViewPanel.reveal();
        }
        else {
            if (showAnyway || this.beenAWhile()) {
                this.createWelcomePage(false);
            }
        }
    }
    beenAWhile() {
        const lastTimeShown = this.context.globalState.get(exports.LAST_SHOWN_OVERVIEW_PAGE, new Date(2000, 0, 1).toString());
        const hoursSinceLastShow = (Date.now() - Date.parse(lastTimeShown)) / 1000 / 60 / 60;
        return hoursSinceLastShow > 24;
    }
    async createWelcomePage(showOnTop) {
        const iconUri = this.context.asAbsolutePath('images/icon.png');
        const webViewPanel = vscode_1.window.createWebviewPanel("pddl.Welcome", "PDDL Overview", {
            viewColumn: vscode_1.ViewColumn.Active,
            preserveFocus: !showOnTop
        }, {
            retainContextWhenHidden: true,
            enableFindWidget: true,
            enableCommandUris: true,
            enableScripts: true,
            localResourceRoots: [
                vscode_1.Uri.file(this.context.asAbsolutePath(this.VIEWS)),
                vscode_1.Uri.file(this.context.asAbsolutePath("images"))
            ]
        });
        const html = await this.getHtml(webViewPanel.webview);
        webViewPanel.webview.html = html;
        webViewPanel.iconPath = vscode_1.Uri.file(iconUri);
        webViewPanel.onDidDispose(() => this.webViewPanel = undefined, undefined, this.context.subscriptions);
        webViewPanel.webview.onDidReceiveMessage(message => this.handleMessage(message), undefined, this.context.subscriptions);
        webViewPanel.onDidChangeViewState(() => this.scheduleUpdatePageConfiguration());
        this.webViewPanel = webViewPanel;
        this.context.subscriptions.push(this.webViewPanel);
        // record the last date the page was shown on top
        this.context.globalState.update(exports.LAST_SHOWN_OVERVIEW_PAGE, new Date(Date.now()));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async handleMessage(message) {
        var _a;
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'onload':
                this.scheduleUpdatePageConfiguration();
                this.showNextHint();
                break;
            case 'shouldShowOverview':
                this.context.globalState.update(exports.SHOULD_SHOW_OVERVIEW_PAGE, message.value);
                break;
            case 'tryHelloWorld':
                this.helloWorld().catch(utils_1.showError);
                break;
            case 'openNunjucksSample':
                this.openNunjucksSample().catch(utils_1.showError);
                break;
            case 'clonePddlSamples':
                vscode_1.commands.executeCommand("git.clone", "https://github.com/jan-dolejsi/vscode-pddl-samples.git");
                break;
            case 'selectPlanner':
                this.plannersConfiguration.setSelectedPlanner(message.value, this.workspaceFolder).catch(utils_1.showError);
                break;
            case 'deletePlanner':
                vscode_1.commands.executeCommand('pddl.deletePlanner', message.value);
                break;
            case 'configurePlanner':
                vscode_1.commands.executeCommand('pddl.configurePlanner', message.value);
                break;
            case 'showConfiguration':
                vscode_1.commands.executeCommand('pddl.showPlannerConfiguration', message.value);
                break;
            case 'plannerOutputTarget':
                this.pddlConfiguration.updateEffectiveConfiguration("pddlPlanner", "executionTarget", message.value);
                break;
            case 'workspaceFolderSelected':
                const workspaceFolderUriAsString = message.workspaceFolderUri;
                this.workspaceFolder = vscode_1.workspace.getWorkspaceFolder(vscode_1.Uri.parse(workspaceFolderUriAsString));
                this.scheduleUpdatePageConfiguration();
                break;
            case 'installIcons':
                try {
                    await vscode_1.commands.executeCommand("workbench.extensions.installExtension", this.ICONS_EXTENSION_NAME);
                }
                catch (err) {
                    vscode_1.window.showErrorMessage("Could not install the VS Code Icons extension: " + err);
                }
                break;
            case 'enableIcons':
                await vscode_1.workspace.getConfiguration().update("workbench.iconTheme", "vscode-icons", vscode_1.ConfigurationTarget.Global);
                break;
            case 'downloadVal':
                const options = { bypassConsent: message.informedDecision };
                await vscode_1.commands.executeCommand(valCommand_1.VAL_DOWNLOAD_COMMAND, options);
                break;
            case 'enableFormatOnType':
                this.pddlConfiguration.setEditorFormatOnType(true, { forPddlOnly: message.forPddlOnly });
                break;
            case 'hintOk':
                this.hintAcknowledged();
                break;
            case 'hintLater':
                this.hintHide();
                break;
            case 'hintNext':
                this.hintShowNext();
                break;
            case 'feedbackAccepted':
                this.feedbackAccepted();
                break;
            case 'feedbackLater':
                this.feedbackAskLater();
                break;
            case 'feedbackNever':
                this.feedbackAskNever();
                break;
            default:
                if ((_a = message.command) === null || _a === void 0 ? void 0 : _a.startsWith('command:')) {
                    const command = message.command;
                    await vscode_1.commands.executeCommand(command.substr('command:'.length));
                }
                else {
                    console.warn('Unexpected command: ' + message.command);
                }
        }
    }
    async showNextHint() {
        const tipsUri = vscode_1.Uri.joinPath(this.context.extensionUri, 'tips.html');
        const tips = (await vscode_1.workspace.fs.readFile(tipsUri)).toString().split("\n");
        let nextTipToShow = this.context.globalState.get(this.NEXT_TIP_TO_SHOW, 0);
        for (let index = nextTipToShow; index < tips.length; index++) {
            const tip = tips[index];
            // skip tips that were removed subsequently as obsolete
            if (tip.trim() === "") {
                nextTipToShow++;
                continue;
            }
            this.tipDisplayed = nextTipToShow;
            this.showTip(tip);
            return;
        }
        if (nextTipToShow === tips.length) {
            this.showTip(undefined);
            this.askForReview();
        }
        this.context.globalState.update(this.NEXT_TIP_TO_SHOW, nextTipToShow);
    }
    showTip(tip) {
        var _a;
        (_a = this.webViewPanel) === null || _a === void 0 ? void 0 : _a.webview.postMessage({
            command: 'showHint',
            hint: tip
        });
    }
    hintShowNext() {
        this.context.globalState.update(this.NEXT_TIP_TO_SHOW, this.tipDisplayed + 1);
        this.showNextHint();
    }
    hintHide() {
        this.showTip(undefined);
    }
    hintAcknowledged() {
        this.context.globalState.update(this.NEXT_TIP_TO_SHOW, this.tipDisplayed + 1);
        this.showTip(undefined);
    }
    async askForReview() {
        // what was the user response last time?
        const accepted = this.context.globalState.get(this.ACCEPTED_TO_WRITE_A_REVIEW, LATER);
        if (accepted === LATER) {
            this.showFeedbackRequest(true);
        }
    }
    feedbackAccepted() {
        const reviewPage = 'https://marketplace.visualstudio.com/items?itemName=jan-dolejsi.pddl#review-details';
        vscode_1.commands.executeCommand('vscode.open', vscode_1.Uri.parse(reviewPage));
        this.context.globalState.update(this.ACCEPTED_TO_WRITE_A_REVIEW, ACCEPTED);
        this.showFeedbackRequest(false);
    }
    feedbackAskNever() {
        this.context.globalState.update(this.ACCEPTED_TO_WRITE_A_REVIEW, NEVER);
        this.showFeedbackRequest(false);
    }
    feedbackAskLater() {
        this.context.globalState.update(this.ACCEPTED_TO_WRITE_A_REVIEW, LATER);
        this.showFeedbackRequest(false);
    }
    showFeedbackRequest(shouldShow) {
        var _a;
        (_a = this.webViewPanel) === null || _a === void 0 ? void 0 : _a.webview.postMessage({
            command: 'showFeedbackRequest',
            visible: shouldShow
        });
    }
    async helloWorld() {
        const sampleDocuments = await this.createSample('helloWorld', 'Hello World!');
        if (!sampleDocuments) {
            return;
        } // canceled
        const documentsToOpen = await this.openSampleFiles(sampleDocuments, ['domain.pddl', 'problem.pddl']);
        const workingDirectory = path.dirname(documentsToOpen[0].fileName);
        vscode_1.commands.executeCommand("pddl.planAndDisplayResult", documentsToOpen[0].uri, documentsToOpen[1].uri, workingDirectory, "");
    }
    async openNunjucksSample() {
        const sampleDocuments = await this.createSample('nunjucks', 'Nunjucks template sample');
        if (!sampleDocuments) {
            return;
        } // canceled
        // let documentsToOpen = 
        await this.openSampleFiles(sampleDocuments, ['domain.pddl', 'problem.pddl', 'problem0.json']);
        const ptestJsonName = '.ptest.json';
        const ptestJson = sampleDocuments.find(doc => path.basename(doc.fileName) === ptestJsonName);
        if (!ptestJson) {
            throw new Error("Could not find " + ptestJsonName);
        }
        const generatedProblemUri = ptestJson.uri.with({ fragment: '0' });
        await vscode_1.commands.executeCommand(PTestCommands_1.PTEST_VIEW, generatedProblemUri);
        // let workingDirectory = path.dirname(documentsToOpen[0].fileName);
        // commands.executeCommand("pddl.planAndDisplayResult", documentsToOpen[0].uri, generatedProblemUri, workingDirectory, "");
    }
    async openSampleFiles(sampleDocuments, fileNamesToOpen) {
        const documentsToOpen = fileNamesToOpen
            .map(fileName => sampleDocuments.find(doc => path.basename(doc.fileName) === fileName));
        // were all files found?
        if (documentsToOpen.some(v => !v)) {
            throw new Error('One or more sample files were not found: ' + fileNamesToOpen);
        }
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const validDocumentsToOpen = documentsToOpen.map(v => v);
        for (let index = 0; index < validDocumentsToOpen.length; index++) {
            const doc = sampleDocuments[index];
            const viewColumn = this.indexToViewColumn(index);
            await vscode_1.window.showTextDocument(doc, { viewColumn: viewColumn, preview: false });
        }
        return validDocumentsToOpen;
    }
    indexToViewColumn(index) {
        switch (index) {
            case 0: return vscode_1.ViewColumn.One;
            case 1: return vscode_1.ViewColumn.Two;
            case 2: return vscode_1.ViewColumn.Three;
            case 3: return vscode_1.ViewColumn.Four;
            default: return vscode_1.ViewColumn.Five;
        }
    }
    async createSample(subDirectory, sampleName) {
        let folder;
        if (!vscode_1.workspace.workspaceFolders || vscode_1.workspace.workspaceFolders.length === 0) {
            const folders = await vscode_1.window.showOpenDialog({ canSelectFiles: false, canSelectFolders: true, canSelectMany: false, openLabel: `Select folder for the '${sampleName}' sample...` });
            if (folders) {
                folder = folders[0];
            }
        }
        else if (vscode_1.workspace.workspaceFolders.length === 1) {
            folder = vscode_1.workspace.workspaceFolders[0].uri;
        }
        else {
            const selectedFolder = await vscode_1.window.showWorkspaceFolderPick({ placeHolder: `Select workspace folder for the '${sampleName}' sample...` });
            folder = selectedFolder === null || selectedFolder === void 0 ? void 0 : selectedFolder.uri;
        }
        if (!folder) {
            return undefined;
        }
        const sampleFiles = await vscode_1.workspace.fs.readDirectory(vscode_1.Uri.joinPath(this.context.extensionUri, this.CONTENT_FOLDER, subDirectory));
        const sampleFileNames = sampleFiles
            .map(child => child[0]); // pick up the file name
        const sampleDocuments = [];
        for (const sampleFile of sampleFileNames) {
            const shouldWrite = await this.overwriteIfExists(folder, sampleFile);
            if (shouldWrite === undefined) {
                return undefined;
            }
            const sampleTargetPath = vscode_1.Uri.joinPath(folder, sampleFile);
            if (shouldWrite) {
                const sampleResourcePath = vscode_1.Uri.joinPath(this.context.extensionUri, this.CONTENT_FOLDER, subDirectory, sampleFile);
                await vscode_1.workspace.fs.copy(sampleResourcePath, sampleTargetPath, { overwrite: true });
            }
            const sampleDocument = await vscode_1.workspace.openTextDocument(sampleTargetPath);
            sampleDocuments.push(sampleDocument);
        }
        return sampleDocuments;
    }
    /**
     * Checks whether the file already exists and decides whether to overwrite it.
     * @param folder folder uri
     * @param sampleFile sample file to write
     * @returns undefined, if the user cancels.
     */
    async overwriteIfExists(folder, sampleFile) {
        const sampleTargetPath = vscode_1.Uri.joinPath(folder, sampleFile);
        if (await workspaceFs_1.exists(sampleTargetPath)) {
            const overwrite = "Overwrite";
            const answer = await vscode_1.window.showWarningMessage(`File '${sampleFile}' already exists.`, { modal: true }, overwrite, "Skip");
            if (answer === undefined) {
                // canceled by the user
                return undefined;
            }
            return answer === overwrite;
        }
        else {
            // it does not exist
            return true;
        }
    }
    async getHtml(webview) {
        return webviewUtils_1.getWebViewHtml(this.context, {
            relativePath: this.CONTENT_FOLDER,
            htmlFileName: 'overview.html',
            allowUnsafeInlineScripts: true,
            fonts: [
                vscode_1.Uri.file(path.join("..", "..", this.COMMON_FOLDER, "codicon.ttf"))
            ]
        }, webview);
    }
    updateIconsAlerts() {
        this.iconsInstalled = vscode_1.extensions.getExtension(this.ICONS_EXTENSION_NAME) !== undefined;
        this.scheduleUpdatePageConfiguration();
    }
    scheduleUpdatePageConfiguration(options) {
        if (options === null || options === void 0 ? void 0 : options.immediate) {
            this.updatePageConfiguration();
        }
        else {
            if (this.updateTimeout) {
                this.updateTimeout.refresh();
            }
            else {
                this.updateTimeout = setTimeout(() => {
                    this.updatePageConfiguration().catch(utils_1.showError);
                }, 500);
            }
        }
    }
    async updatePageConfiguration() {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (this.updateTimeout) {
            clearTimeout(this.updateTimeout);
            this.updateTimeout = undefined;
        }
        if (!this.webViewPanel || !this.webViewPanel.active) {
            return false;
        }
        let planners;
        let plannersConfigError;
        this.workspaceFolder = (_a = this.workspaceFolder) !== null && _a !== void 0 ? _a : (vscode_1.workspace.workspaceFolders && vscode_1.workspace.workspaceFolders[0]);
        try {
            planners = this.plannersConfiguration.getPlanners(this.workspaceFolder);
        }
        catch (err) {
            plannersConfigError = (_b = err.message) !== null && _b !== void 0 ? _b : err;
            console.log(plannersConfigError);
        }
        const message = {
            command: 'updateConfiguration',
            workspaceFolders: (_d = (_c = vscode_1.workspace.workspaceFolders) === null || _c === void 0 ? void 0 : _c.map(wf => this.toWireWorkspaceFolder(wf))) !== null && _d !== void 0 ? _d : [],
            selectedWorkspaceFolder: this.workspaceFolder && this.toWireWorkspaceFolder(this.workspaceFolder),
            planners: planners !== null && planners !== void 0 ? planners : [],
            selectedPlanner: (_e = this.plannersConfiguration.getSelectedPlanner(this.workspaceFolder)) === null || _e === void 0 ? void 0 : _e.configuration.title,
            plannersConfigError: plannersConfigError,
            plannerOutputTarget: vscode_1.workspace.getConfiguration(configuration_1.PDDL_PLANNER, this.workspaceFolder).get(PlannersConfiguration_1.EXECUTION_TARGET, PlannersConfiguration_1.DEF_PLANNER_OUTPUT_TARGET),
            parser: this.pddlConfiguration.getParserPath(this.workspaceFolder),
            validator: this.pddlConfiguration.getValidatorPath(this.workspaceFolder),
            imagesPath: webviewUtils_1.asWebviewUri(vscode_1.Uri.file(this.context.asAbsolutePath('images')), this.webViewPanel.webview).toString(),
            shouldShow: this.context.globalState.get(exports.SHOULD_SHOW_OVERVIEW_PAGE, true),
            autoSave: vscode_1.workspace.getConfiguration("files", this.workspaceFolder).get("autoSave", "off"),
            showInstallIconsAlert: !this.iconsInstalled,
            showEnableIconsAlert: this.iconsInstalled && vscode_1.workspace.getConfiguration().get("workbench.iconTheme") !== "vscode-icons",
            downloadValAlert: !this.pddlConfiguration.getValidatorPath(this.workspaceFolder) || !(await this.val.isInstalled()),
            updateValAlert: await this.val.isNewValVersionAvailable(),
            showEnableFormatterAlert: !this.pddlConfiguration.getEditorFormatOnType()
            // todo: workbench.editor.revealIfOpen
        };
        return (_h = (_g = (_f = this === null || this === void 0 ? void 0 : this.webViewPanel) === null || _f === void 0 ? void 0 : _f.webview) === null || _g === void 0 ? void 0 : _g.postMessage(message)) !== null && _h !== void 0 ? _h : false;
    }
    toWireWorkspaceFolder(workspaceFolder) {
        return {
            name: workspaceFolder.name,
            uri: workspaceFolder.uri.toString()
        };
    }
}
exports.OverviewPage = OverviewPage;
//# sourceMappingURL=OverviewPage.js.map