/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.StartUp = void 0;
const vscode_1 = require("vscode");
const semver_1 = require("semver");
const OverviewPage_1 = require("./OverviewPage");
const fs = require("fs");
const ValDownloadReminder_1 = require("../validation/ValDownloadReminder");
const ExtensionInfo_1 = require("../configuration/ExtensionInfo");
class StartUp {
    constructor(context, pddlConfiguration, plannersConfiguration, val) {
        this.context = context;
        this.pddlConfiguration = pddlConfiguration;
        this.val = val;
        this.WHATS_NEW_SHOWN_FOR_VERSION = 'whatsNewShownForVersion';
        this.NEVER_AUTO_SAVE = 'neverAutoSave';
        this.overviewPage = new OverviewPage_1.OverviewPage(context, this.pddlConfiguration, plannersConfiguration, this.val);
    }
    atStartUp() {
        this.showOverviewPage();
        this.showWhatsNew();
        this.suggestFolderIsOpen();
        this.suggestAutoSave();
        new ValDownloadReminder_1.ValDownloadReminder(this.context, this.val).suggestValDownloadConfigurationIfAbsent();
    }
    suggestFolderIsOpen() {
        // The PDDL extension works best if you open VS Code in a specific folder. Use File > Open Folder ...
    }
    async suggestAutoSave() {
        if (this.context.globalState.get(this.NEVER_AUTO_SAVE, false)) {
            return;
        }
        const option = "files.autoSave";
        if (vscode_1.workspace.getConfiguration().get(option) === "off") {
            const changeConfigurationOption = { title: "Configure auto-save" };
            const notNow = { title: "Not now" };
            const never = { title: "Do not ask again" };
            const options = [changeConfigurationOption, notNow, never];
            const choice = await vscode_1.window.showInformationMessage("Switching on `File > Auto Save` saves you from constant file saving when working with command-line tools.", ...options);
            switch (choice) {
                case changeConfigurationOption:
                    vscode_1.workspace.getConfiguration().update(option, "afterDelay", vscode_1.ConfigurationTarget.Global);
                    break;
                case never:
                    this.context.globalState.update(this.NEVER_AUTO_SAVE, true);
                case notNow:
                default:
                    // do nothing
                    break;
            }
        }
    }
    async showWhatsNew() {
        const currentVersion = new ExtensionInfo_1.ExtensionInfo(ExtensionInfo_1.ExtensionInfo.EXTENSION_ID).getVersion();
        const lastValue = this.context.globalState.get(this.WHATS_NEW_SHOWN_FOR_VERSION, "0.0.0");
        const lastInstalledDiff = semver_1.diff(currentVersion, lastValue);
        if (lastInstalledDiff === null) {
            // same version
            return;
        }
        if (['major', 'minor'].includes(lastInstalledDiff)) {
            if (true) {
                this.showChangeLog();
            }
            else {
                const changeLog = this.context.asAbsolutePath('CHANGELOG.html');
                const html = (await fs.promises.readFile(changeLog, { encoding: "utf-8" })).toString();
                const webViewPanel = vscode_1.window.createWebviewPanel("pddl.WhatsNew", "PDDL: What's new?", vscode_1.ViewColumn.Active, {
                    retainContextWhenHidden: true,
                    enableFindWidget: true,
                    enableCommandUris: false,
                    enableScripts: true
                });
                webViewPanel.webview.html = html;
                this.context.subscriptions.push(webViewPanel);
            }
            this.context.globalState.update(this.WHATS_NEW_SHOWN_FOR_VERSION, currentVersion);
        }
        else {
            const SEE = `See what is new`;
            const LATER = "Later";
            const answer = await vscode_1.window.showInformationMessage(`PDDL Extension was updated to ${currentVersion}.`, SEE, LATER);
            switch (answer) {
                case LATER:
                    // do nothing
                    break;
                case SEE:
                    this.showChangeLog();
                // intentionally falling through to 'default'
                default:
                    this.context.globalState.update(this.WHATS_NEW_SHOWN_FOR_VERSION, currentVersion);
            }
        }
    }
    showChangeLog() {
        const changeLogMd = this.context.asAbsolutePath('CHANGELOG.md');
        vscode_1.commands.executeCommand('markdown.showPreview', vscode_1.Uri.file(changeLogMd), null, {
            sideBySide: false,
            locked: true
        });
    }
    showOverviewPage() {
        const shouldShow = this.context.globalState.get(OverviewPage_1.SHOULD_SHOW_OVERVIEW_PAGE, true);
        if (shouldShow) {
            this.overviewPage.showWelcomePage(false);
        }
    }
}
exports.StartUp = StartUp;
//# sourceMappingURL=StartUp.js.map