/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainTypesView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
const view_1 = require("./view");
const DomainView_1 = require("./DomainView");
const CONTENT = path.join('views', 'modelView', 'static');
const PDDL_DOMAIN_TYPES_PREVIEW_COMMAND = "pddl.domain.types.preview";
const PDDL_DOMAIN_TYPES_INSET_COMMAND = "pddl.domain.types.inset";
class DomainTypesView extends DomainView_1.DomainView {
    constructor(context, codePddlWorkspace) {
        super(context, codePddlWorkspace, new DomainTypesRenderer(), {
            content: CONTENT,
            viewCommand: PDDL_DOMAIN_TYPES_PREVIEW_COMMAND,
            insetViewCommand: PDDL_DOMAIN_TYPES_INSET_COMMAND,
            insetHeight: 5,
            webviewType: 'domainTypesPreview',
            webviewHtmlPath: 'graphView.html',
            webviewOptions: {
                enableFindWidget: true,
                // enableCommandUris: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                localResourceRoots: [
                    vscode_1.Uri.file(context.asAbsolutePath("views"))
                ]
            }
        }, {});
    }
    async provideCodeLenses(document, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        const domain = await this.parseDomain(document);
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (!domain) {
            return [];
        }
        const defineNode = domain.syntaxTree.getDefineNodeOrThrow();
        const typesNode = defineNode.getFirstChild(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /\s*:types/i);
        if (typesNode) {
            return [
                new view_1.DocumentCodeLens(document, utils_1.nodeToRange(document, typesNode))
            ];
        }
        else {
            return [];
        }
    }
    async resolveCodeLens(codeLens, token) {
        if (!(codeLens instanceof view_1.DocumentCodeLens)) {
            return undefined;
        }
        if (token.isCancellationRequested) {
            return undefined;
        }
        const domain = await this.parseDomain(codeLens.getDocument());
        if (!domain) {
            return undefined;
        }
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (codeLens instanceof view_1.DocumentInsetCodeLens) {
            codeLens.command = { command: PDDL_DOMAIN_TYPES_INSET_COMMAND, title: 'Show hierarchy inset', arguments: [codeLens.getDocument().uri, codeLens.getLine()] };
            return codeLens;
        }
        else {
            codeLens.command = { command: PDDL_DOMAIN_TYPES_PREVIEW_COMMAND, title: 'Show hierarchy', arguments: [codeLens.getDocument().uri] };
            return codeLens;
        }
    }
    createPreviewPanelTitle(uri) {
        return `:types in '${path.basename(uri.fsPath)}'`;
    }
    async handleOnLoad(panel) {
        await panel.postMessage('setInverted', { value: true });
        return super.handleOnLoad(panel);
    }
}
exports.DomainTypesView = DomainTypesView;
class DomainTypesRenderer {
    render(context, domain, options) {
        const renderer = new DomainTypesRendererDelegate(context, domain, options);
        return {
            nodes: renderer.getNodes(),
            relationships: renderer.getRelationships()
        };
    }
}
class DomainTypesRendererDelegate {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    constructor(_context, domain, _options) {
        this.nodes = new Map();
        this.relationships = [];
        domain.getTypesInclObject().forEach((t, index) => this.nodes.set(t, index));
        domain.getTypeInheritance().getEdges().forEach(edge => this.addEdge(edge));
    }
    addEdge(edge) {
        this.relationships.push(this.toEdge(edge));
    }
    getNodes() {
        return [...this.nodes.entries()].map(entry => this.toNode(entry));
    }
    toNode(entry) {
        const [entryLabel, entryId] = entry;
        return { id: entryId, label: entryLabel };
    }
    toEdge(edge) {
        const [from, to] = edge;
        return { from: this.getNodeId(from), to: this.getNodeId(to), label: 'extends' };
    }
    getNodeId(from) {
        const nodeId = this.nodes.get(from);
        if (nodeId === undefined) {
            return -1;
        }
        else {
            return nodeId;
        }
    }
    getRelationships() {
        return this.relationships;
    }
}
//# sourceMappingURL=DomainTypesView.js.map