/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
const DomainViewPanel_1 = require("./DomainViewPanel");
const view_1 = require("./view");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const webviewUtils_1 = require("../webviewUtils");
const VIEWS = "views";
const COMMON_FOLDER = path.join(VIEWS, "common");
/**
 * Base-class for different domain views.
 */
class DomainView extends vscode_1.Disposable {
    constructor(context, codePddlWorkspace, renderer, options, rendererOptions) {
        super(() => this.dispose());
        this.context = context;
        this.codePddlWorkspace = codePddlWorkspace;
        this.renderer = renderer;
        this.options = options;
        this.rendererOptions = rendererOptions;
        this._onDidChangeCodeLenses = new vscode_1.EventEmitter();
        this.onDidChangeCodeLenses = this._onDidChangeCodeLenses.event;
        this.webviewPanels = new utils_1.UriMap();
        this.webviewInsets = new utils_1.UriMap();
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(options.viewCommand, async (domainUri) => {
            const domainDocument = await getDomainDocument(domainUri);
            if (domainDocument) {
                return this.revealOrCreatePreview(domainDocument, vscode_1.ViewColumn.Beside).catch(utils_1.showError);
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(options.insetViewCommand, async (domainUri, line) => {
            if (vscode_1.window.activeTextEditor && domainUri && line) {
                if (domainUri.toString() === vscode_1.window.activeTextEditor.document.uri.toString()) {
                    this.showInset(vscode_1.window.activeTextEditor, line, options.insetHeight).catch(utils_1.showError);
                }
            }
        }));
        codePddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.UPDATED, (fileInfo) => {
            if (fileInfo.isDomain()) {
                this.refreshDomain(fileInfo);
            }
        });
    }
    async refreshDomain(domainInfo) {
        const domainUri = utils_1.toUri(domainInfo.fileUri);
        // if no panel was created, skip
        if (!this.webviewPanels.get(domainUri) && !this.webviewInsets.get(domainUri)) {
            return;
        }
        let panelsToRefresh = [];
        // update the panel
        const panel = this.webviewPanels.get(domainUri);
        if (panel) {
            panelsToRefresh.push(panel);
        }
        // update all the insets
        const insets = this.webviewInsets.get(domainUri);
        if (insets) {
            panelsToRefresh = panelsToRefresh.concat([...insets.values()]);
        }
        panelsToRefresh.forEach(async (panel) => {
            panel.setDomain(domainInfo);
            await this.refreshPanelContent(panel);
        });
        if (panelsToRefresh.length > 0) {
            this._onDidChangeCodeLenses.fire();
        }
    }
    refreshPanel(panel) {
        if (panel.getNeedsRebuild() && panel.getPanel().isVisible()) {
            this.refreshPanelContent(panel);
        }
    }
    async setup(previewPanel, domainInfo) {
        if (!previewPanel.getPanel().html) {
            previewPanel.getPanel().html = "Please wait...";
        }
        previewPanel.setNeedsRebuild(false);
        previewPanel.getPanel().html =
            await this.generateHtml(previewPanel);
        previewPanel.setDomain(domainInfo);
    }
    async refreshPanelContent(previewPanel) {
        previewPanel.setNeedsRebuild(false);
        return this.updateContentData(previewPanel.getDomain(), previewPanel);
    }
    async showInset(editor, line, height) {
        console.log(`todo: revealOrCreateInset ${editor} line ${line} height ${height}`);
    }
    async expandInset(panel) {
        console.log(`todo: expand inset ${panel.uri}`);
    }
    async revealOrCreatePreview(doc, displayColumn) {
        const domainInfo = await this.parseDomain(doc);
        if (!domainInfo) {
            return;
        }
        let previewPanel = this.webviewPanels.get(doc.uri);
        if (previewPanel && previewPanel.getPanel().canReveal()) {
            previewPanel.getPanel().reveal(displayColumn);
            await this.refreshDomain(domainInfo);
        }
        else {
            previewPanel = await this.createPreviewPanelForDocument(domainInfo, displayColumn);
            this.webviewPanels.set(doc.uri, previewPanel);
        }
    }
    async createPreviewPanelForDocument(domainInfo, displayColumn) {
        const domainUri = utils_1.toUri(domainInfo.fileUri);
        const previewTitle = this.createPreviewPanelTitle(domainUri);
        const webViewPanel = vscode_1.window.createWebviewPanel(this.options.webviewType, previewTitle, displayColumn, this.options.webviewOptions);
        webViewPanel.iconPath = vscode_1.Uri.file(this.context.asAbsolutePath("images/icon.png"));
        const panel = new DomainViewPanel_1.DomainViewPanel(domainUri, new view_1.WebviewPanelAdapter(webViewPanel));
        // when the user closes the tab, remove the panel
        webViewPanel.onDidDispose(() => this.webviewPanels.delete(domainUri), undefined, this.context.subscriptions);
        // when the pane becomes visible again, refresh it
        webViewPanel.onDidChangeViewState(() => this.refreshPanel(panel));
        webViewPanel.webview.onDidReceiveMessage(e => this.handleMessageCore(panel, e), undefined, this.context.subscriptions);
        await this.setup(panel, domainInfo);
        return panel;
    }
    async generateHtml(viewPanel) {
        const error = viewPanel.getError();
        if (error) {
            return error.message;
        }
        else {
            return webviewUtils_1.getWebViewHtml(this.context, {
                fonts: [
                    vscode_1.Uri.file(path.join("..", "..", "..", COMMON_FOLDER, "codicon.ttf"))
                ],
                relativePath: this.options.content,
                htmlFileName: this.options.webviewHtmlPath
            }, viewPanel.getPanel().webview);
        }
    }
    async updateContentData(domain, panel) {
        return panel.postMessage('updateContent', {
            data: this.renderer.render(this.context, domain, this.rendererOptions)
        });
    }
    async parseDomain(domainDocument) {
        const fileInfo = await this.codePddlWorkspace.upsertAndParseFile(domainDocument);
        if (!(fileInfo === null || fileInfo === void 0 ? void 0 : fileInfo.isDomain())) {
            return undefined;
        }
        return fileInfo;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    handleMessage(_panel, _message) {
        return false;
    }
    async handleOnLoad(panel) {
        await panel.postMessage('setIsInset', { value: panel.getPanel().isInset });
        return await this.refreshPanelContent(panel);
    }
    async handleMessageCore(panel, message) {
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'close':
                panel.close();
                break;
            case 'expand':
                this.expandInset(panel);
                break;
            case 'onload':
                await this.handleOnLoad(panel);
                break;
            default:
                if (!this.handleMessage(panel, message)) {
                    console.warn('Unexpected command: ' + message.command);
                }
        }
    }
}
exports.DomainView = DomainView;
async function getDomainDocument(dotDocumentUri) {
    if (dotDocumentUri) {
        return await vscode_1.workspace.openTextDocument(dotDocumentUri);
    }
    else {
        if (vscode_1.window.activeTextEditor !== undefined && workspaceUtils_1.isPddl(vscode_1.window.activeTextEditor.document)) {
            return vscode_1.window.activeTextEditor.document;
        }
        else {
            return undefined;
        }
    }
}
//# sourceMappingURL=DomainView.js.map