/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.HappeningsPlanExporter = void 0;
const PlanExporter_1 = require("./PlanExporter");
const pddl_workspace_1 = require("pddl-workspace");
/** Exports .happenings file to a .plan file by pairing happenings, if possible. */
class HappeningsPlanExporter extends PlanExporter_1.AbstractPlanExporter {
    constructor(happeningsDocument, epsilon) {
        super();
        this.happeningsDocument = happeningsDocument;
        this.epsilon = epsilon;
        this.makespan = 0;
        this.decimals = -Math.log10(epsilon);
    }
    getDefaultPlanPath() {
        return PlanExporter_1.AbstractPlanExporter.replaceExtension(this.happeningsDocument.uri.fsPath, '.plan');
    }
    getPlanText() {
        const happeningsText = this.happeningsDocument.getText();
        const happeningsParser = new pddl_workspace_1.parser.PlanHappeningsBuilder(this.epsilon);
        happeningsParser.tryParseFile(happeningsText);
        const happenings = happeningsParser.getHappenings();
        let planSteps = [];
        happenings.forEach((happening, happeningIndex) => {
            switch (happening.getType()) {
                case pddl_workspace_1.HappeningType.INSTANTANEOUS:
                    planSteps.push(new pddl_workspace_1.PlanStep(happening.getTime(), happening.getFullActionName(), false, undefined, -1));
                    break;
                case pddl_workspace_1.HappeningType.START:
                    const end = this.findEnd(happenings, happening, happeningIndex + 1);
                    const duration = end === undefined ? 0 : end.getTime() - happening.getTime();
                    planSteps.push(new pddl_workspace_1.PlanStep(happening.getTime(), happening.getFullActionName(), true, duration, -1));
                    break;
            }
        });
        planSteps = planSteps.sort(this.comparePlanSteps);
        const meta = pddl_workspace_1.parser.PddlPlanParser.parsePlanMeta(happeningsText);
        const endl = '\n';
        return pddl_workspace_1.parser.PddlPlanParser.getPlanMeta(meta.domainName, meta.problemName, endl)
            + endl
            + planSteps.map(step => step.toPddl()).join(endl);
    }
    findEnd(happenings, start, fromIndex) {
        return happenings.slice(fromIndex).find(h => h.belongsTo(start));
    }
    formatTime(time) {
        return Number(time).toFixed(this.decimals);
    }
    comparePlanSteps(h1, h2) {
        if (h1.getStartTime() > h2.getStartTime()) {
            return 1;
        }
        else if (h1.getStartTime() < h2.getStartTime()) {
            return -1;
        }
        return 0;
    }
}
exports.HappeningsPlanExporter = HappeningsPlanExporter;
//# sourceMappingURL=HappeningsPlanExporter.js.map