/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanExporter = exports.AbstractPlanExporter = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const path_1 = require("path");
const ExportUtil_1 = require("./ExportUtil");
const utils_1 = require("../utils");
class AbstractPlanExporter {
    async export() {
        const defaultPlanPath = this.getDefaultPlanPath();
        const options = {
            saveLabel: "Save plan as...",
            filters: {
                "Plan": ["plan"]
            },
            defaultUri: vscode_1.Uri.file(defaultPlanPath)
        };
        try {
            const uri = await vscode_1.window.showSaveDialog(options);
            if (uri === undefined) {
                return;
            } // canceled by user
            await ExportUtil_1.exportToAndShow(this.getPlanText(), uri);
        }
        catch (ex) {
            vscode_1.window.showErrorMessage(`Cannot export plan: ${ex}`);
        }
    }
    static replaceExtension(path, extension) {
        const pathObj = path_1.parse(path);
        const origExt = pathObj.ext;
        pathObj.ext = extension;
        pathObj.base = pathObj.base.replace(new RegExp(origExt + '$'), pathObj.ext);
        return path_1.format(pathObj);
    }
}
exports.AbstractPlanExporter = AbstractPlanExporter;
class PlanExporter extends AbstractPlanExporter {
    constructor(plan) {
        super();
        this.plan = plan;
    }
    getDefaultPlanPath() {
        if (!this.plan.problem) {
            throw new Error(`Problem not specified.`);
        }
        let baseUri = utils_1.toUri(this.plan.problem.fileUri);
        if (this.plan.problem.fileUri.scheme === 'http') {
            baseUri = vscode_1.Uri.file(baseUri.path);
        }
        return PlanExporter.replaceExtension(baseUri.with({ scheme: 'file' }).fsPath, '.plan');
    }
    getPlanText() {
        var _a, _b, _c, _d;
        let planText = pddl_workspace_1.parser.PddlPlanParser.getPlanMeta((_b = (_a = this.plan.domain) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : 'undefined', (_d = (_c = this.plan.problem) === null || _c === void 0 ? void 0 : _c.name) !== null && _d !== void 0 ? _d : 'undefined', '\n') +
            `
${this.plan.getText()}

; Makespan: ${this.plan.makespan}`;
        if (this.plan.metric !== null && this.plan.metric !== undefined) {
            planText += `\n; Metric: ${this.plan.metric}`;
        }
        if (this.plan.statesEvaluated !== undefined) {
            planText += `\n; States evaluated: ${this.plan.statesEvaluated}`;
        }
        return planText;
    }
}
exports.PlanExporter = PlanExporter;
//# sourceMappingURL=PlanExporter.js.map