/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanHappeningsExporter = void 0;
const vscode_1 = require("vscode");
const PlanExporter_1 = require("./PlanExporter");
const pddl_workspace_1 = require("pddl-workspace");
const ExportUtil_1 = require("./ExportUtil");
/**
 * Converts plan to happenings while preserving comments.
 */
class PlanHappeningsExporter {
    constructor(planDocument, epsilon) {
        this.planDocument = planDocument;
        this.epsilon = epsilon;
        this.comments = '';
        this.happeningsText = '';
        this.enqueuedHappenings = [];
        this.makespan = 0;
        this.actionCounter = new Map();
        this.decimals = -Math.log10(epsilon);
        this.computeHappeningsText(this.planDocument.getText());
    }
    async export() {
        const defaultPlanHappeningsPath = PlanExporter_1.PlanExporter.replaceExtension(this.planDocument.uri.fsPath, '.happenings');
        const options = {
            saveLabel: "Save plan happenings as...",
            filters: {
                "Plan Happenings": ["happenings"]
            },
            defaultUri: vscode_1.Uri.file(defaultPlanHappeningsPath)
        };
        try {
            const uri = await vscode_1.window.showSaveDialog(options);
            if (uri === undefined) {
                return;
            } // canceled by user
            await ExportUtil_1.exportToAndShow(this.happeningsText, uri);
        }
        catch (ex) {
            vscode_1.window.showErrorMessage(`Cannot export plan to happenings: ${ex}`);
        }
    }
    computeHappeningsText(planText) {
        const planParser = new pddl_workspace_1.parser.PddlPlanParser();
        const planBuilder = new pddl_workspace_1.parser.PddlPlanBuilder(this.epsilon);
        planText.split('\n').forEach((line, index) => {
            if (line.match(/^\s*;/)) {
                // comment lines are accumulated and flushed later 
                this.comments += line + '\n';
            }
            else if (line.match(/^\s*$/)) {
                this.comments += '\n';
            }
            else {
                if (this.comments.length) {
                    // flush non-trailing comments
                    this.flushComments();
                }
                const happening = this.parseStepAndEnqueueEnd(line, index, planParser, planBuilder);
                if (happening) {
                    this.flushHappeningsBefore(happening.time);
                    this.outputHappening(happening);
                }
                else {
                    console.log("Error parsing happening: " + line);
                }
            }
        });
        // flush enqueued ends
        this.enqueuedHappenings.forEach(happening => this.outputHappening(happening));
        this.enqueuedHappenings = [];
        // flush trailing comments into the output
        this.flushComments();
    }
    parseStepAndEnqueueEnd(line, lineIndex, planParser, planBuilder) {
        var _a;
        const planStep = planParser.parse(line, lineIndex, planBuilder);
        if (planStep) {
            // this line is a plan step
            const actionName = planStep.getFullActionName();
            const count = this.getActionCount(actionName);
            const thisHappening = new Happening(planStep.getStartTime(), actionName, planStep.isDurative ? HappeningType.Start : HappeningType.Instantaneous, count);
            if (planStep.isDurative) {
                const endTime = planStep.getStartTime() + ((_a = planStep.getDuration()) !== null && _a !== void 0 ? _a : 0);
                const endHappening = new Happening(endTime, actionName, HappeningType.End, count);
                this.enqueue(endHappening);
            }
            return thisHappening;
        }
        else {
            this.happeningsText += `; Warning: line did not parse: ${line}`;
            return null;
        }
    }
    getActionCount(action) {
        let prevCount = -1;
        const actionCount = this.actionCounter.get(action);
        if (actionCount !== undefined) {
            prevCount = actionCount;
        }
        const newCount = prevCount + 1;
        this.actionCounter.set(action, newCount);
        return newCount;
    }
    enqueue(endHappening) {
        this.enqueuedHappenings.push(endHappening);
        this.enqueuedHappenings = this.enqueuedHappenings.sort(this.compareHappenings);
    }
    flushHappeningsBefore(time) {
        while (this.enqueuedHappenings.length > 0 && this.enqueuedHappenings[0].time <= time) {
            this.outputHappening(this.enqueuedHappenings[0]);
            this.enqueuedHappenings.splice(0, 1);
        }
    }
    flushComments() {
        this.happeningsText += this.comments;
        this.comments = '';
    }
    outputHappening(happening) {
        let happeningQualifier = '';
        switch (happening.type) {
            case HappeningType.Start:
                happeningQualifier = "start ";
                break;
            case HappeningType.End:
                happeningQualifier = "end ";
                break;
        }
        let countString = '';
        if (happening.count > 0) {
            countString = ` #${happening.count + 1}`;
        }
        this.happeningsText += `${this.formatTime(happening.time)}: ${happeningQualifier}(${happening.name})${countString}\n`;
    }
    formatTime(time) {
        return Number(time).toFixed(this.decimals);
    }
    compareHappenings(h1, h2) {
        if (h1.time > h2.time) {
            return 1;
        }
        else if (h1.time < h2.time) {
            return -1;
        }
        return 0;
    }
}
exports.PlanHappeningsExporter = PlanHappeningsExporter;
class Happening {
    constructor(time, name, type, count) {
        this.time = time;
        this.name = name;
        this.type = type;
        this.count = count;
    }
}
var HappeningType;
(function (HappeningType) {
    HappeningType[HappeningType["Start"] = 0] = "Start";
    HappeningType[HappeningType["End"] = 1] = "End";
    HappeningType[HappeningType["Instantaneous"] = 2] = "Instantaneous";
})(HappeningType || (HappeningType = {}));
//# sourceMappingURL=PlanHappeningsExporter.js.map