"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlannerConfigurationSelector = void 0;
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
const vscode_1 = require("vscode");
const path = require("path");
const pddl_planning_service_client_1 = require("pddl-planning-service-client");
class PlannerConfigurationSelector {
    constructor(problemPath) {
        this.problemPath = problemPath;
    }
    async getConfiguration() {
        const selectedItem = await vscode_1.window.showQuickPick([useDefaultsItem, selectedConfigurationItem], { placeHolder: 'Select planner configuration from a .json file...' });
        if (!selectedItem) {
            return null;
        }
        switch (selectedItem) {
            case useDefaultsItem:
                return PlannerConfigurationSelector.DEFAULT;
            case selectedConfigurationItem:
                return await this.selectConfigurationFile();
            default:
                if (selectedConfigurationItem instanceof PlannerConfigurationUriItem) {
                    return selectedConfigurationItem.uri;
                }
                else {
                    throw new Error("Unexpected selected item type: " + typeof (selectedConfigurationItem));
                }
        }
    }
    async selectConfigurationFile() {
        const selectedUris = await vscode_1.window.showOpenDialog({
            canSelectMany: false, filters: {
                'Planner Configuration JSON': ['json']
            },
            defaultUri: this.problemPath
        });
        if (!selectedUris) {
            return null;
        }
        return selectedUris[0];
    }
    static async loadConfiguration(configurationUri, timeout) {
        if (configurationUri.toString() === PlannerConfigurationSelector.DEFAULT.toString()) {
            return pddl_planning_service_client_1.PlannerAsyncService.createDefaultConfiguration(timeout);
        }
        else {
            const configurationDoc = await vscode_1.workspace.openTextDocument(configurationUri);
            const configurationString = configurationDoc.getText();
            return JSON.parse(configurationString);
        }
    }
}
exports.PlannerConfigurationSelector = PlannerConfigurationSelector;
PlannerConfigurationSelector.DEFAULT = vscode_1.Uri.parse("configuration:/default");
class PlannerConfigurationItem {
    constructor(label, description) {
        this.label = label;
        this.description = description;
    }
}
class PlannerConfigurationUriItem extends PlannerConfigurationItem {
    constructor(uri) {
        super(path.basename(uri.fsPath), path.dirname(uri.fsPath));
        this.uri = uri;
    }
}
const useDefaultsItem = new PlannerConfigurationItem("Use defaults");
const selectedConfigurationItem = new PlannerConfigurationItem("Select a configuration file...");
//# sourceMappingURL=PlannerConfigurationSelector.js.map