/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Planning = exports.PDDL_PLAN_AND_DISPLAY = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const pddl_planning_service_client_1 = require("pddl-planning-service-client");
const path = require("path");
const path_1 = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const configuration_1 = require("../configuration/configuration");
const PlannerExecutable_1 = require("./PlannerExecutable");
const Authentication_1 = require("../util/Authentication");
const PlanningResult_1 = require("./PlanningResult");
const PlanExporter_1 = require("./PlanExporter");
const PlanHappeningsExporter_1 = require("./PlanHappeningsExporter");
const HappeningsPlanExporter_1 = require("./HappeningsPlanExporter");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const PlanView_1 = require("../planView/PlanView");
const PlannerUserOptionsSelector_1 = require("./PlannerUserOptionsSelector");
const PlannerConfigurationSelector_1 = require("./PlannerConfigurationSelector");
const AssociationProvider_1 = require("../workspace/AssociationProvider");
const utils_1 = require("../utils");
const PlannersConfiguration_1 = require("../configuration/PlannersConfiguration");
const plannerConfigurations_1 = require("../configuration/plannerConfigurations");
const PDDL_STOP_PLANNER = 'pddl.stopPlanner';
const PDDL_CONVERT_PLAN_TO_HAPPENINGS = 'pddl.convertPlanToHappenings';
const PDDL_CONVERT_HAPPENINGS_TO_PLAN = 'pddl.convertHappeningsToPlan';
exports.PDDL_PLAN_AND_DISPLAY = 'pddl.planAndDisplayResult';
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function instanceOfEditorId(obj) {
    return 'groupId' in obj;
}
/**
 * Delegate for handling requests to run the planner and visualize the plans.
 */
class Planning {
    constructor(codePddlWorkspace, pddlConfiguration, plannersConfiguration, context) {
        this.codePddlWorkspace = codePddlWorkspace;
        this.pddlConfiguration = pddlConfiguration;
        this.plannersConfiguration = plannersConfiguration;
        this.context = context;
        this.planner = null;
        this.plans = [];
        this.planningProcessKilled = false;
        this.optionProviders = [];
        this._onPlansFound = new vscode_1.EventEmitter();
        this.PLANNING_SERVICE_CONSENTS = "planningServiceConsents";
        this.userOptionsProvider = new PlannerUserOptionsSelector_1.PlannerUserOptionsSelector();
        this.output = vscode_1.window.createOutputChannel("Planner output");
        context.subscriptions.push(this.planView = new PlanView_1.PlanView(context, codePddlWorkspace));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.PDDL_PLAN_AND_DISPLAY, async (domainUri, payload, workingFolder, options) => {
            if (payload && instanceOfEditorId(payload)) {
                // triggered from the editor tab
                await this.planFromDocument(await vscode_1.workspace.openTextDocument(domainUri)).catch(utils_1.showError);
            }
            else if (Array.isArray(payload)) {
                // multi-selected multiple files in the Explorer
                const selectedFiles = payload;
                this.planByUris(selectedFiles, workingFolder, options).catch(utils_1.showError);
            }
            else if (payload) {
                // payload should be Uri; we ruled out other options
                await this.planByUri(domainUri, payload, workingFolder, options).catch(utils_1.showError);
            }
            else {
                await this.plan().catch(utils_1.showError);
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_STOP_PLANNER, () => this.stopPlanner()));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PlanView_1.PDDL_EXPORT_PLAN, (plan) => {
            if (plan) {
                new PlanExporter_1.PlanExporter(plan).export();
            }
            else {
                vscode_1.window.showErrorMessage("There is no plan open, or the selected plan does not exist.");
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_CONVERT_PLAN_TO_HAPPENINGS, async () => {
            if (vscode_1.window.activeTextEditor && workspaceUtils_1.isPlan(vscode_1.window.activeTextEditor.document)) {
                const epsilon = pddlConfiguration.getEpsilonTimeStep();
                new PlanHappeningsExporter_1.PlanHappeningsExporter(vscode_1.window.activeTextEditor.document, epsilon).export();
            }
            else {
                vscode_1.window.showErrorMessage("Active document is not a plan.");
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PDDL_CONVERT_HAPPENINGS_TO_PLAN, async () => {
            if (vscode_1.window.activeTextEditor && workspaceUtils_1.isHappenings(vscode_1.window.activeTextEditor.document)) {
                const epsilon = pddlConfiguration.getEpsilonTimeStep();
                new HappeningsPlanExporter_1.HappeningsPlanExporter(vscode_1.window.activeTextEditor.document, epsilon).export();
            }
            else {
                vscode_1.window.showErrorMessage("Active document is not a happening.");
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.syntaxTree", () => {
            if (vscode_1.window.activeTextEditor && workspaceUtils_1.isPddl(vscode_1.window.activeTextEditor.document)) {
                const index = vscode_1.window.activeTextEditor.document.offsetAt(vscode_1.window.activeTextEditor.selection.active);
                const pddlSyntaxTreeBuilder = new pddl_workspace_1.parser.PddlSyntaxTreeBuilder(vscode_1.window.activeTextEditor.document.getText());
                this.output.appendLine('');
                this.output.appendLine("PDDL Syntax Tree:");
                this.output.appendLine(pddlSyntaxTreeBuilder.getTreeAsString());
                const breadcrumbs = pddlSyntaxTreeBuilder.getBreadcrumbs(index);
                this.output.appendLine('');
                this.output.appendLine("PDDL Parser Breadcrumbs:");
                breadcrumbs.forEach(b => this.output.appendLine(b.toString()));
                this.output.show();
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PlannersConfiguration_1.PDDL_CONFIGURE_PLANNER_OUTPUT_TARGET, () => vscode_1.commands.executeCommand(configuration_1.PDDL_CONFIGURE_COMMAND, configuration_1.PDDL_PLANNER + '.' + PlannersConfiguration_1.EXECUTION_TARGET)));
    }
    addOptionsProvider(optionsProvider) {
        this.optionProviders.push(optionsProvider);
    }
    providePlannerOptions(context) {
        return this.optionProviders.map(provider => provider.providePlannerOptions(context));
    }
    /**
     * Invokes the planner in context of model specified via file URIs.
     * @param domainUri domain file uri
     * @param problemUri problem file uri
     * @param workingFolder working folder
     * @param options planner options
     */
    async planByUri(domainUri, problemUri, workingFolder, options) {
        const domainDocument = await vscode_1.workspace.openTextDocument(domainUri);
        const problemDocument = await vscode_1.workspace.openTextDocument(problemUri);
        const domainInfo = await this.codePddlWorkspace.upsertAndParseFile(domainDocument);
        const problemInfo = await this.codePddlWorkspace.upsertAndParseFile(problemDocument);
        this.planExplicit(domainInfo, problemInfo, workingFolder, options);
    }
    async planByUris(selectedFiles, workingFolder, options) {
        if (selectedFiles.length === 1) {
            await this.planFromDocument(await vscode_1.workspace.openTextDocument(selectedFiles[0])).catch(utils_1.showError);
        }
        else if (selectedFiles.length === 2) {
            let problemFileInfo;
            let domainFileInfo;
            for (let i = 0; i < selectedFiles.length; i++) {
                const selectedFile = selectedFiles[i];
                const selectedDoc = await vscode_1.workspace.openTextDocument(selectedFile);
                const fileInfo = await this.codePddlWorkspace.upsertAndParseFile(selectedDoc);
                if (fileInfo === undefined) {
                    throw new Error(`Selected file is not a PDDL document: ${selectedDoc.fileName}`);
                }
                if (fileInfo.isDomain()) {
                    if (domainFileInfo !== undefined) {
                        throw new Error('Two domain files were selected. Select a domain file and one problem file.');
                    }
                    domainFileInfo = fileInfo;
                }
                else if (fileInfo.isProblem()) {
                    if (problemFileInfo !== undefined) {
                        throw new Error('Two problem files were selected. Select a domain file and one problem file.');
                    }
                    problemFileInfo = fileInfo;
                }
                else {
                    throw new Error(`File not supported: ${fileInfo.name}. Select a domain file and one problem file.`);
                }
            }
            if (!domainFileInfo || !problemFileInfo) {
                throw new Error('Select a domain file and one problem file.');
            }
            await this.planExplicit(domainFileInfo, problemFileInfo, workingFolder, options);
        }
    }
    /**
     * Invokes the planner in the context of the currently opened files in the workspace.
     */
    async plan() {
        if (this.planner) {
            vscode_1.window.showErrorMessage("Planner is already running. Stop it using the Cancel button in the progress notification, or using the 'PDDL: Stop planner' command or wait for it to finish.");
            return;
        }
        if (!vscode_1.window.activeTextEditor) {
            vscode_1.window.showErrorMessage("Active document is not a PDDL file.");
            return;
        }
        const activeDocument = vscode_1.window.activeTextEditor.document;
        return this.planFromDocument(activeDocument);
    }
    async planFromDocument(activeDocument) {
        if (!activeDocument) {
            return;
        }
        const activeFileInfo = await this.codePddlWorkspace.upsertAndParseFile(activeDocument);
        if (activeFileInfo === undefined) {
            throw new Error('Selected file is not a PDDL document.');
        }
        let problemFileInfo;
        let domainFileInfo;
        this.output.clear();
        if (activeFileInfo.isProblem()) {
            problemFileInfo = activeFileInfo;
            // find domain file(s)
            const domainFiles = this.codePddlWorkspace.getDomainFilesFor(problemFileInfo);
            if (domainFiles.length === 1) {
                domainFileInfo = domainFiles[0];
            }
            else if (domainFiles.length !== 1) {
                const workspaceFolder = vscode_1.workspace.getWorkspaceFolder(activeDocument.uri);
                const domainFileUri = await workspaceUtils_1.selectFile({
                    language: pddl_workspace_1.PddlLanguage.PDDL,
                    promptMessage: 'Select the matching domain file...',
                    findPattern: AssociationProvider_1.AssociationProvider.PDDL_PATTERN,
                    fileOpenLabel: 'Select',
                    fileOpenFilters: { 'PDDL Domain Files': ['pddl'] },
                    workspaceFolder: workspaceFolder
                }, domainFiles);
                if (!domainFileUri) {
                    return;
                } // was canceled
                domainFileInfo = domainFiles.find(doc => doc.fileUri.toString() === domainFileUri.toString())
                    || this.codePddlWorkspace.pddlWorkspace.getFileInfo(domainFileUri)
                    || await this.parseDomain(domainFileUri);
            }
            else {
                vscode_1.window.showInformationMessage(`Ensure a domain '${problemFileInfo.domainName}' from the same folder is open in the editor.`);
                return;
            }
        }
        else if (activeFileInfo.isDomain()) {
            domainFileInfo = activeFileInfo;
            const problemFiles = this.codePddlWorkspace.pddlWorkspace.getProblemFiles(domainFileInfo);
            if (problemFiles.length === 1) {
                problemFileInfo = problemFiles[0];
            }
            else if (problemFiles.length > 1) {
                const problemFilePicks = problemFiles.map(info => new ProblemFilePickItem(info));
                const selectedProblemFilePick = await vscode_1.window.showQuickPick(problemFilePicks, { placeHolder: "Select problem file:" });
                if (!selectedProblemFilePick) {
                    return;
                } // was canceled
                problemFileInfo = selectedProblemFilePick.problemInfo;
            }
            else {
                vscode_1.window.showInformationMessage("Ensure a corresponding problem file is open in the editor.");
                return;
            }
        }
        else {
            vscode_1.window.showInformationMessage("Selected file does not appear to be a valid PDDL domain or problem file.");
            return;
        }
        const workingDirectory = this.establishWorkingDirectory(activeDocument, problemFileInfo, domainFileInfo);
        await this.planExplicit(domainFileInfo, problemFileInfo, workingDirectory);
    }
    get onPlansFound() { return this._onPlansFound.event; }
    establishWorkingDirectory(activeDocument, problemFileInfo, domainFileInfo) {
        let workingDirectory = "";
        if (activeDocument.uri.scheme === "file") {
            workingDirectory = path_1.dirname(activeDocument.fileName);
        }
        else if (problemFileInfo.fileUri.scheme === "file") {
            workingDirectory = path_1.dirname(problemFileInfo.fileUri.fsPath);
        }
        else if (domainFileInfo.fileUri.scheme === "file") {
            workingDirectory = path_1.dirname(domainFileInfo.fileUri.fsPath);
        }
        return workingDirectory;
    }
    async parseDomain(domainFileUri) {
        const fileInfo = await this.codePddlWorkspace.upsertAndParseFile(await vscode_1.workspace.openTextDocument(domainFileUri));
        if (fileInfo && !fileInfo.isDomain()) {
            throw new Error("Selected file is not a domain file.");
        }
        else {
            return fileInfo;
        }
    }
    /**
     * Invokes the planner and visualize the plan(s).
     * @param domainFileInfo domain
     * @param problemFileInfo problem
     * @param workingDirectory workflow folder for auxiliary output files
     * @param options planner options
     */
    async planExplicit(domainFileInfo, problemFileInfo, workingDirectory, options) {
        const planParser = new pddl_workspace_1.parser.PddlPlannerOutputParser(domainFileInfo, problemFileInfo, { epsilon: this.pddlConfiguration.getEpsilonTimeStep() }, plans => this.visualizePlans(plans));
        workingDirectory = await this.adjustWorkingFolder(workingDirectory);
        this.planner = await this.createPlanner(workingDirectory, options);
        if (!this.planner) {
            return;
        }
        const planner = this.planner;
        if (planner instanceof vscode_1.Disposable) {
            this.context.subscriptions.push(planner);
        }
        this.planningProcessKilled = false;
        if (!this.isSearchDebugger()) {
            this.output.show(true);
        }
        else {
            vscode_1.commands.executeCommand('pddl.searchDebugger.start');
        }
        vscode_1.window.withProgress({
            location: vscode_1.ProgressLocation.Notification,
            title: `Searching for plans for domain ${domainFileInfo.name} and problem ${problemFileInfo.name}`,
            cancellable: true,
        }, (progress, token) => {
            token.onCancellationRequested(() => {
                this.planningProcessKilled = true;
                this.stopPlanner();
            });
            this.progressUpdater = new ElapsedTimeProgressUpdater(progress, token);
            return planner.plan(domainFileInfo, problemFileInfo, planParser, this);
        })
            .then(plans => this.onPlannerFinished(plans), reason => this.onPlannerFailed(reason, planner).catch(utils_1.showError));
    }
    isSearchDebugger() {
        return vscode_1.workspace.getConfiguration("pddlPlanner").get("executionTarget") === "Search debugger";
    }
    onPlannerFinished(plans) {
        if (!this.progressUpdater) {
            return;
        }
        const elapsedTime = this.progressUpdater.getElapsedTimeInMilliSecs();
        this.progressUpdater.setFinished();
        const result = this.planningProcessKilled ? PlanningResult_1.PlanningResult.killed() : PlanningResult_1.PlanningResult.success(plans, elapsedTime);
        this._onPlansFound.fire(result);
        this.planner = null;
        this.output.appendLine(`Planner found ${plans.length} plan(s) in ${this.progressUpdater.getElapsedTimeInMilliSecs() / 1000}secs.`);
        this.visualizePlans(plans);
    }
    /**
     * Called when the planner run fails.
     * @param reason reason of the failure
     * @param failedPlanner planner that failed
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async onPlannerFailed(reason, failedPlanner) {
        var _a, _b, _c, _d;
        if (!this.progressUpdater) {
            return;
        }
        this.progressUpdater.setFinished();
        this._onPlansFound.fire(PlanningResult_1.PlanningResult.failure(reason.toString()));
        this.planner = null;
        console.error(reason);
        // does the planner provide any trouble-shooting options?
        const troubleShootingInfo = await ((_b = (_a = failedPlanner.providerConfiguration.provider) === null || _a === void 0 ? void 0 : _a.troubleshoot) === null || _b === void 0 ? void 0 : _b.call(_a, failedPlanner, reason));
        const options = [
            {
                title: failedPlanner.providerConfiguration.configuration.canConfigure ?
                    "Re-configure the planner" : "Show configuration",
                action: () => {
                    // todo, if there is only one configuration of the kind, launch the configuration directly
                    vscode_1.commands.executeCommand("pddl.showOverview");
                }
            }
        ];
        if (troubleShootingInfo === null || troubleShootingInfo === void 0 ? void 0 : troubleShootingInfo.options) {
            [...troubleShootingInfo.options.keys()]
                .forEach(title => {
                options.push({ title: title, action: troubleShootingInfo.options.get(title) });
            });
        }
        options.push({ title: "Dismiss", isCloseAffordance: true });
        const message = ((_c = troubleShootingInfo === null || troubleShootingInfo === void 0 ? void 0 : troubleShootingInfo.info) !== null && _c !== void 0 ? _c : '') + '\n\n\nError: ' + ((_d = reason.message) !== null && _d !== void 0 ? _d : reason);
        const selection = await vscode_1.window.showErrorMessage(message, ...options);
        if (selection === null || selection === void 0 ? void 0 : selection.action) {
            selection.action(failedPlanner);
        }
    }
    async adjustWorkingFolder(workingDirectory) {
        var _a;
        if (!workingDirectory) {
            return "";
        }
        // the working directory may be virtual, replace it
        if (!await pddl_workspace_1.utils.afs.exists(workingDirectory)) {
            if ((_a = vscode_1.workspace.workspaceFolders) === null || _a === void 0 ? void 0 : _a.length) {
                return vscode_1.workspace.workspaceFolders[0].uri.fsPath;
            }
            else {
                return "";
            }
        }
        else {
            return workingDirectory;
        }
    }
    getPlans() {
        return this.plans;
    }
    /**
     * Creates the right planner wrapper according to the current configuration.
     *
     * @param workingDirectory directory where planner creates output files by default
     * @param options planner options or a path of a configuration file
     * @returns `Planner` instance of the configured planning engine
     */
    async createPlanner(workingDirectory, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const scopedPlannerConfiguration = await this.plannersConfiguration.getOrAskSelectedPlanner(vscode_1.workspace.getWorkspaceFolder(vscode_1.Uri.file(workingDirectory)));
        if (!scopedPlannerConfiguration) {
            return null;
        }
        const plannerConfiguration = scopedPlannerConfiguration.configuration;
        if (!await this.verifyConsentForSendingPddl(plannerConfiguration)) {
            return null;
        }
        if (plannerConfiguration.url !== undefined) {
            const useAuthentication = this.pddlConfiguration.isPddlPlannerServiceAuthenticationEnabled();
            let authentication = undefined;
            if (useAuthentication) {
                const configuration = this.pddlConfiguration.getPddlPlannerServiceAuthenticationConfiguration();
                authentication = new Authentication_1.SAuthentication(configuration.url, configuration.requestEncoded, configuration.clientId, configuration.callbackPort, configuration.timeoutInMs, configuration.tokensvcUrl, configuration.tokensvcApiKey, configuration.tokensvcAccessPath, configuration.tokensvcValidatePath, configuration.tokensvcCodePath, configuration.tokensvcRefreshPath, configuration.tokensvcSvctkPath, configuration.refreshToken, configuration.accessToken, configuration.sToken);
            }
            if (plannerConfiguration.url.endsWith("/solve")) {
                options = await this.getPlannerLineOptions(plannerConfiguration, options);
                if (options === undefined) {
                    return null;
                }
                const plannerRunConfiguration = {
                    options: options,
                    authentication: authentication
                };
                return (_c = (_b = (_a = this.codePddlWorkspace.pddlWorkspace.getPlannerRegistrar()
                    .getPlannerProvider(new pddl_workspace_1.planner.PlannerKind(plannerConfiguration.kind))) === null || _a === void 0 ? void 0 : _a.createPlanner) === null || _b === void 0 ? void 0 : _b.call(_a, plannerConfiguration, plannerRunConfiguration)) !== null && _c !== void 0 ? _c : plannerConfigurations_1.SolveServicePlannerProvider.createDefaultPlanner(plannerConfiguration, plannerRunConfiguration);
            }
            else if (plannerConfiguration.url.endsWith("/request")) {
                const configurationUri = options ? this.toAbsoluteUri(options, workingDirectory) : await new PlannerConfigurationSelector_1.PlannerConfigurationSelector(vscode_1.Uri.file(workingDirectory)).getConfiguration();
                if (!configurationUri) {
                    return null;
                } // canceled by user
                const plannerRunConfiguration = await PlannerConfigurationSelector_1.PlannerConfigurationSelector.loadConfiguration(configurationUri, pddl_planning_service_client_1.PlannerAsyncService.DEFAULT_TIMEOUT);
                plannerRunConfiguration.authentication = authentication;
                return (_f = (_e = (_d = this.codePddlWorkspace.pddlWorkspace.getPlannerRegistrar()
                    .getPlannerProvider(new pddl_workspace_1.planner.PlannerKind(plannerConfiguration.kind))) === null || _d === void 0 ? void 0 : _d.createPlanner) === null || _e === void 0 ? void 0 : _e.call(_d, plannerConfiguration, plannerRunConfiguration)) !== null && _f !== void 0 ? _f : plannerConfigurations_1.RequestServicePlannerProvider.createDefaultPlanner(plannerConfiguration, plannerRunConfiguration);
            }
            else {
                throw new Error(`Planning service not supported: ${plannerConfiguration.url}. Only /solve or /request service endpoints are supported.`);
            }
        }
        else if (plannerConfiguration.path) {
            options = await this.getPlannerLineOptions(plannerConfiguration, options);
            if (options === undefined) {
                return null;
            }
            const plannerRunConfiguration = {
                options: options,
                workingDirectory: workingDirectory,
                plannerSyntax: plannerConfiguration.syntax
            };
            const providerConfiguration = {
                configuration: plannerConfiguration
            };
            return (_j = (_h = (_g = this.codePddlWorkspace.pddlWorkspace.getPlannerRegistrar()
                .getPlannerProvider(new pddl_workspace_1.planner.PlannerKind(plannerConfiguration.kind))) === null || _g === void 0 ? void 0 : _g.createPlanner) === null || _h === void 0 ? void 0 : _h.call(_g, plannerConfiguration, plannerRunConfiguration)) !== null && _j !== void 0 ? _j : new PlannerExecutable_1.PlannerExecutable(plannerConfiguration.path, plannerRunConfiguration, providerConfiguration);
        }
        else {
            throw new Error(`Planner configuration must define at least one of the properties 'url' or 'path': ${plannerConfiguration.title}`);
        }
    }
    toAbsoluteUri(configPath, workingDirectory) {
        const absoluteConfigPath = path.isAbsolute(configPath) ?
            configPath :
            path.join(workingDirectory, configPath);
        return vscode_1.Uri.file(absoluteConfigPath);
    }
    async getPlannerLineOptions(configuration, options) {
        if (options === undefined) {
            const plannerProvider = this.codePddlWorkspace.pddlWorkspace.getPlannerRegistrar()
                .getPlannerProvider(new pddl_workspace_1.planner.PlannerKind(configuration.kind));
            return await this.userOptionsProvider.getPlannerOptions(plannerProvider);
        }
        else {
            return options;
        }
    }
    async verifyConsentForSendingPddl(plannerConfiguration) {
        if (plannerConfiguration.url !== undefined && utils_1.isHttp(plannerConfiguration.url)) {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const consents = this.context.globalState.get(this.PLANNING_SERVICE_CONSENTS, {});
            if (consents[plannerConfiguration.url]) {
                return true;
            }
            else {
                const answer = await vscode_1.window.showWarningMessage("Confirm you want to send this PDDL to " + plannerConfiguration.url, {
                    modal: true
                }, "Yes, send my PDDL to this service.", "No, I do not want to send this PDDL to this service.");
                const consentGiven = answer !== undefined && answer.toLowerCase().startsWith("yes");
                consents[plannerConfiguration.url] = consentGiven;
                this.context.globalState.update(this.PLANNING_SERVICE_CONSENTS, consents);
                return consentGiven;
            }
        }
        else {
            return true;
        }
    }
    stopPlanner() {
        try {
            if (this.planner) {
                this.planner.stop();
                this.planner = null;
                this.output.appendLine('Process killing requested.');
            }
        }
        catch (ex) {
            console.error(ex);
        }
    }
    handleOutput(outputText) {
        this.output.append(outputText);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    handlePlan(_plan) {
        // todo: this shall be implemented, when the planning service can return multiple plans
        throw new Error("Method not implemented.");
    }
    visualizePlans(plans) {
        // visualize if either
        // 1. plan(s) are different
        // 2. the plan preview was closed/disposed meanwhile
        if (!this.arePlanListsEqual(this.plans, plans) || !this.planView.hasPlannerOutput()) {
            this.plans = [...plans]; // making a copy of the list, so the above comparison works
            this.planView.setPlannerOutput(plans, !this.isSearchDebugger());
        }
    }
    arePlanListsEqual(plans1, plans2) {
        if (plans1.length !== plans2.length) {
            return false;
        }
        for (let i = 0; i < plans1.length; i++) {
            if (!this.arePlansEqual(plans1[i], plans2[i])) {
                return false;
            }
        }
        return true;
    }
    arePlansEqual(plan1, plan2) {
        if (plan1.steps.length !== plan2.steps.length) {
            return false;
        }
        for (let i = 0; i < plan1.steps.length; i++) {
            if (!plan1.steps[i].equals(plan2.steps[i], this.pddlConfiguration.getEpsilonTimeStep())) {
                return false;
            }
        }
        return true;
    }
    static q(path) {
        return path.includes(' ') ? `"${path}"` : path;
    }
}
exports.Planning = Planning;
class ElapsedTimeProgressUpdater {
    constructor(progress, token) {
        this.progress = progress;
        this.token = token;
        this.startTime = new Date();
        this.finished = false;
        this.reportProgress();
    }
    getElapsedTimeInMilliSecs() {
        return new Date().getTime() - this.startTime.getTime();
    }
    reportProgress() {
        if (this.token.isCancellationRequested || this.finished) {
            return;
        }
        setTimeout(() => {
            const elapsedTime = new Date(this.getElapsedTimeInMilliSecs());
            this.progress.report({ message: "Elapsed time: " + elapsedTime.toISOString().substr(11, 8) });
            this.reportProgress();
        }, 1000);
    }
    setFinished() {
        this.finished = true;
    }
}
class ProblemFilePickItem {
    constructor(problemInfo) {
        this.label = pddl_workspace_1.PddlWorkspace.getFileInfoName(problemInfo);
        this.description = pddl_workspace_1.PddlWorkspace.getFolderPath(problemInfo.fileUri);
        this.problemInfo = problemInfo;
    }
}
//# sourceMappingURL=planning.js.map