/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratedDocumentContentProvider = void 0;
const vscode_1 = require("vscode");
const path_1 = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const pddl_workspace_3 = require("pddl-workspace");
const utils_1 = require("../utils");
/**
 * Content provider for the problem file generated from a template.
 */
class GeneratedDocumentContentProvider {
    constructor(outputWindow, pddlWorkspace) {
        this.outputWindow = outputWindow;
        this.pddlWorkspace = pddlWorkspace;
        this._onDidChange = new vscode_1.EventEmitter();
        this.onDidChange = this._onDidChange.event;
        this.uriMap = new Map();
        pddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.UPDATED, (fileInfo) => {
            // if the URI corresponds one that was already rendered from template, fire event
            this.uriMap.forEach((testCase, uri) => {
                if (testCase.getProblemUri().toString() === fileInfo.fileUri.toString()) {
                    this.changed(vscode_1.Uri.parse(uri));
                }
            });
        });
        vscode_1.workspace.onDidChangeTextDocument(e => {
            // check if the changing document represents input data for a pre-processor
            this.uriMap.forEach((testCase, uri) => {
                var _a;
                const inputChanged = (_a = testCase.getPreProcessor()) === null || _a === void 0 ? void 0 : _a.getInputFiles().some(inputFileName => e.document.fileName.endsWith(inputFileName));
                if (inputChanged) {
                    this.changed(vscode_1.Uri.parse(uri));
                }
            });
        });
    }
    changed(uri) {
        this._onDidChange.fire(uri);
    }
    mapUri(test) {
        const problemTemplatePath = test.getProblemUri().fsPath;
        const testIdx = test.getUriOrThrow().fragment;
        let problemPath;
        const problemTemplateWithoutExtension = problemTemplatePath.replace('.pddl', '');
        if (test.getLabel()) {
            problemPath = path_1.join(problemTemplateWithoutExtension + ` (${this.sanitizePath(test.getLabel())}).pddl`);
        }
        else {
            problemPath = problemTemplateWithoutExtension + ` (${testIdx}).pddl`;
        }
        const uri = vscode_1.Uri.file(problemPath).with({ scheme: 'tpddl' });
        this.uriMap.set(uri.toString(), test);
        return uri;
    }
    sanitizePath(path) {
        return path.split(/[\\\/]/).join('_');
    }
    async provideTextDocumentContent(uri, token) {
        var _a, _b;
        const test = this.uriMap.get(uri.toString());
        if (!test) {
            return `Document ${uri.toString()} not found in the content provider's map.`;
        }
        if (token.isCancellationRequested) {
            return "";
        }
        const problemDocument = await vscode_1.workspace.openTextDocument(test.getProblemUri());
        const documentText = problemDocument.getText();
        try {
            const preProcessedProblemText = (_b = await ((_a = test.getPreProcessor()) === null || _a === void 0 ? void 0 : _a.transform(documentText, path_1.dirname(utils_1.assertDefined(test.getManifest(), `Test ${test.getLabel()} should have manifest defined.`).path), this.outputWindow))) !== null && _b !== void 0 ? _b : "No pre-processor configured.";
            // force parsing of the generated problem
            this.pddlWorkspace.pddlWorkspace.upsertFile(uri, pddl_workspace_2.PddlLanguage.PDDL, 0, preProcessedProblemText, new pddl_workspace_3.SimpleDocumentPositionResolver(preProcessedProblemText), true);
            return preProcessedProblemText;
        }
        catch (ex) {
            return `;;Problem file '${path_1.basename(uri.fsPath)}' failed to generate: ${ex.message}\n\n` + documentText;
        }
    }
}
exports.GeneratedDocumentContentProvider = GeneratedDocumentContentProvider;
//# sourceMappingURL=GeneratedDocumentContentProvider.js.map