/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PTestExplorer = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const path_1 = require("path");
const fs_1 = require("fs");
const jsonc_parser_1 = require("jsonc-parser");
const vscode_uri_1 = require("vscode-uri");
const pddl_workspace_1 = require("pddl-workspace");
const Test_1 = require("./Test");
const PTestTreeDataProvider_1 = require("./PTestTreeDataProvider");
const GeneratedDocumentContentProvider_1 = require("./GeneratedDocumentContentProvider");
const PlanningResult_1 = require("../planning/PlanningResult");
const TestsManifest_1 = require("./TestsManifest");
const PTestReport_1 = require("./PTestReport");
const utils_1 = require("../utils");
const PTestCommands_1 = require("./PTestCommands");
const configuration_1 = require("../configuration/configuration");
const ManifestGenerator_1 = require("./ManifestGenerator");
const PlanView_1 = require("../planView/PlanView");
/**
 * PDDL Test Explorer pane.
 */
class PTestExplorer {
    constructor(context, pddlContext, codePddlWorkspace, planning) {
        this.context = context;
        this.pddlContext = pddlContext;
        this.codePddlWorkspace = codePddlWorkspace;
        this.planning = planning;
        this.pTestTreeDataProvider = new PTestTreeDataProvider_1.PTestTreeDataProvider(pddlContext);
        this.pTestViewer = vscode_1.window.createTreeView('pddl.tests.explorer', { treeDataProvider: this.pTestTreeDataProvider, showCollapseAll: true });
        this.subscribe(this.pTestViewer);
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.refresh', () => this.pTestTreeDataProvider.refresh()));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.run', node => this.runTest(node).catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.runAll', node => this.findAndRunTests(node).catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PTestCommands_1.PTEST_VIEW, nodeOrUri => {
            if (nodeOrUri instanceof vscode_1.Uri) {
                this.openTestByUri(nodeOrUri).catch(utils_1.showError);
            }
            else {
                this.openTest(nodeOrUri).catch(utils_1.showError);
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PTestCommands_1.PTEST_VIEW_PROBLEM, test => this.openProblemFile(test, vscode_1.ViewColumn.Beside).catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.viewDefinition', node => this.openDefinition(node).catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.viewExpectedPlans', node => this.openExpectedPlans(node).catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.problemSaveAs', () => this.saveProblemAs().catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PTestCommands_1.PTEST_REVEAL, nodeUri => this.pTestViewer.reveal(this.pTestTreeDataProvider.findNodeByResourceOrThrow(nodeUri), { select: true, expand: true }).then(() => { return; }, utils_1.showError)));
        this.manifestGenerator = new ManifestGenerator_1.ManifestGenerator(this.codePddlWorkspace.pddlWorkspace, this.pddlContext);
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.tests.createAll', () => this.generateAllManifests().catch(utils_1.showError)));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PlanView_1.PDDL_SAVE_AS_EXPECTED_PLAN, plan => this.manifestGenerator.createPlanAssertion(plan).catch(utils_1.showError)));
        this.subscribe(this.report = new PTestReport_1.PTestReport(context, this.planning.output));
        this.generatedDocumentContentProvider = new GeneratedDocumentContentProvider_1.GeneratedDocumentContentProvider(this.planning.output, this.codePddlWorkspace);
        this.subscribe(vscode_1.workspace.registerTextDocumentContentProvider('tpddl', this.generatedDocumentContentProvider));
    }
    getTreeDataProvider() {
        return this.pTestTreeDataProvider;
    }
    async generateAllManifests() {
        const manifests = await this.manifestGenerator.generateAll();
        this.getTreeDataProvider().refresh();
        return manifests;
    }
    subscribe(disposable) {
        this.pddlContext.subscriptions.push(disposable);
    }
    async saveProblemAs() {
        var _a, _b;
        const generatedDocument = (_a = vscode_1.window.activeTextEditor) === null || _a === void 0 ? void 0 : _a.document;
        if (generatedDocument === undefined) {
            return;
        }
        const options = {
            saveLabel: "Save as PDDL problem",
            filters: {
                "PDDL": ["pddl"]
            },
            defaultUri: generatedDocument.uri.with({ scheme: 'file' })
        };
        try {
            const uri = await vscode_1.window.showSaveDialog(options);
            if (!uri) {
                return;
            }
            const newDocument = await vscode_1.workspace.openTextDocument(uri.with({ scheme: 'untitled' }));
            const editor = await vscode_1.window.showTextDocument(newDocument, (_b = vscode_1.window.activeTextEditor) === null || _b === void 0 ? void 0 : _b.viewColumn);
            await editor.edit(edit => edit.insert(new vscode_1.Position(0, 0), generatedDocument.getText()));
        }
        catch (ex) {
            console.log(ex);
        }
    }
    async openDefinition(node) {
        var _a;
        if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Test) {
            const test = Test_1.Test.fromUri(node.resource, this.pddlContext);
            if (test === undefined) {
                throw new Error("No test found at: " + node.resource);
            }
            const manifest = test.getManifest();
            if (!manifest) {
                throw new Error(`Test ${test.getLabel()} is not associated to any manifest`);
            }
            const manifestDocument = await vscode_1.workspace.openTextDocument(manifest.uri);
            // use jsonc-parser to find the element in the JSON DOM
            const manifestText = (await vscode_1.workspace.fs.readFile(vscode_1.Uri.file(manifest.path))).toString();
            const rootNode = jsonc_parser_1.parseTree(manifestText);
            const jsonTestNode = rootNode && jsonc_parser_1.findNodeAtLocation(rootNode, ["cases", (_a = test.getIndex()) !== null && _a !== void 0 ? _a : 0]);
            const selection = jsonTestNode && utils_1.jsonNodeToRange(manifestDocument, jsonTestNode);
            await vscode_1.window.showTextDocument(manifestDocument.uri, { preview: true, viewColumn: vscode_1.ViewColumn.One, selection: selection });
        }
        else if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Manifest) {
            const manifestDocument = await vscode_1.workspace.openTextDocument(node.resource);
            await vscode_1.window.showTextDocument(manifestDocument.uri, { preview: true, viewColumn: vscode_1.ViewColumn.One });
        }
    }
    async openExpectedPlans(node) {
        if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Test) {
            const test = Test_1.Test.fromUri(node.resource, this.pddlContext);
            // assert that everything exists
            if (!test) {
                return;
            }
            await this.assertValid(test);
            if (!test.hasExpectedPlans()) {
                await vscode_1.window.showInformationMessage("Test has no expected plans defined.");
            }
            else {
                const previewOnly = test.getExpectedPlans().length === 1;
                test.getExpectedPlans().forEach(async (expectedPlan) => {
                    const path = test.toAbsolutePath(expectedPlan);
                    const planDocument = await vscode_1.workspace.openTextDocument(path);
                    await vscode_1.window.showTextDocument(planDocument.uri, { preview: previewOnly, viewColumn: vscode_1.ViewColumn.Three });
                });
            }
        }
    }
    async openTest(node) {
        if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Test) {
            this.openTestByUri(node.resource);
        }
    }
    async openTestByUri(testCaseUri) {
        const test = Test_1.Test.fromUri(testCaseUri, this.pddlContext);
        // assert that everything exists
        if (!test) {
            return;
        }
        await this.assertValid(test);
        const domainDocument = await vscode_1.workspace.openTextDocument(test.getDomainUri());
        await vscode_1.window.showTextDocument(domainDocument.uri, { preview: true, viewColumn: vscode_1.ViewColumn.One });
        await this.openProblemFile(test, vscode_1.ViewColumn.Two);
    }
    async openProblemFile(test, column) {
        let uri;
        if (test.getPreProcessor()) {
            uri = this.generatedDocumentContentProvider.mapUri(test);
            // notify the window (if any) that the document will change
            this.generatedDocumentContentProvider.changed(uri);
        }
        else {
            const problemDocument = await vscode_1.workspace.openTextDocument(test.getProblemUri());
            uri = problemDocument.uri;
        }
        await vscode_1.window.showTextDocument(uri, { preview: true, viewColumn: column });
    }
    /**
     * Finds all tests in given scope and executes them.
     * @param node user-selected tree node, or `null` for all workspace folders
     */
    async findAndRunTests(node) {
        const allManifests = [];
        await this.findTests(node, allManifests);
        const contextPath = node ?
            vscode_1.workspace.asRelativePath(node.resource.fsPath) :
            "this workspace";
        this.runTests(allManifests, contextPath);
    }
    /**
     * Walks the tree from the given node recursively and adds manifests to `allManifests`.
     * @param node tree node from where to start walking
     * @param allManifests all manifests found so far
     */
    async findTests(node, allManifests) {
        if (!node || node.kind === PTestTreeDataProvider_1.PTestNodeKind.Directory) {
            const children = await this.pTestTreeDataProvider.getChildren(node);
            for (const child of children) {
                await this.findTests(child, allManifests);
            }
        }
        else if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Manifest) {
            const manifest = TestsManifest_1.TestsManifest.load(node.resource.fsPath, this.pddlContext);
            allManifests.push(manifest);
        }
    }
    runTests(manifests, contextPath) {
        const testCount = manifests.map(m => m.testCases.length).reduce((previousValue, currentValue) => previousValue + currentValue, 0);
        this.report.clearAndShow();
        vscode_1.window.withProgress({
            location: vscode_1.ProgressLocation.Notification,
            title: `Running tests from ${contextPath}`,
            cancellable: true
        }, (progress, token) => {
            return new Promise(async (resolve, reject) => {
                try {
                    for (const manifest of manifests) {
                        this.report.startingManifest(manifest);
                        for (let caseIndex = 0; caseIndex < manifest.testCases.length; caseIndex++) {
                            if (token.isCancellationRequested) {
                                this.planning.handleOutput('Canceled by user.\n');
                                reject();
                                break;
                            }
                            const test = manifest.testCases[caseIndex];
                            progress.report({ message: 'Test case: ' + test.getLabel(), increment: 100.0 / testCount });
                            try {
                                await this.executeTest(test);
                            }
                            catch (e) {
                                console.log(e);
                            }
                        }
                        this.report.finishedManifest(manifest);
                    }
                    resolve(null); // should be resolve(void);
                }
                finally {
                    this.report.show();
                }
            });
        });
    }
    async runTest(node) {
        if (node.kind === PTestTreeDataProvider_1.PTestNodeKind.Test) {
            const test = Test_1.Test.fromUri(node.resource, this.pddlContext);
            // assert that everything exists
            if (!test) {
                return;
            }
            try {
                await this.executeTest(test);
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    executeTest(test) {
        let problemUri;
        if (test.getPreProcessor()) {
            problemUri = this.generatedDocumentContentProvider.mapUri(test);
            // notify the window (if any) that the document may have changed
            this.generatedDocumentContentProvider.changed(problemUri);
        }
        else {
            problemUri = test.getProblemUri();
        }
        this.setTestOutcome(test, Test_1.TestOutcome.IN_PROGRESS);
        return new Promise(async (resolve, reject) => {
            try {
                const testValid = await this.assertValid(test);
                if (!testValid) {
                    this.outputTestResult(test, Test_1.TestOutcome.SKIPPED, Number.NaN, "Invalid test definition");
                    reject(new Error('Invalid test ' + test.getLabel()));
                    return;
                }
            }
            catch (ex) {
                reject(ex);
                return;
            }
            const resultSubscription = this.planning.onPlansFound(result => {
                var _a, _b;
                resultSubscription.dispose();
                if (result.outcome === PlanningResult_1.PlanningOutcome.FAILURE) {
                    this.outputTestResult(test, Test_1.TestOutcome.FAILED, result.elapsedTime, result.error);
                    reject(new Error((_a = result.error) !== null && _a !== void 0 ? _a : "Unknown error while planning."));
                    return;
                }
                else if (result.outcome === PlanningResult_1.PlanningOutcome.KILLED) {
                    this.outputTestResult(test, Test_1.TestOutcome.SKIPPED, result.elapsedTime, 'Killed by the user.');
                    resolve(false);
                    return;
                }
                else if (result.plans.length === 0) {
                    this.outputTestResult(test, Test_1.TestOutcome.FAILED, result.elapsedTime, 'No plan found.');
                    resolve(false);
                    return;
                }
                if (test.hasExpectedPlans()) {
                    try {
                        this.assertMatchesAnExpectedPlan(result, test);
                    }
                    catch (err) {
                        this.outputTestResult(test, Test_1.TestOutcome.FAILED, result.elapsedTime, (_b = "Failed to compare plan to expected plans. Error: " + err.message) !== null && _b !== void 0 ? _b : err);
                    }
                }
                else {
                    this.outputTestResult(test, Test_1.TestOutcome.SUCCESS, result.elapsedTime);
                }
                resolve(true);
            });
            try {
                const cwd = test.getManifest() ? path_1.dirname(test.getManifest().path) : '.';
                await vscode_1.commands.executeCommand('pddl.planAndDisplayResult', test.getDomainUri(), problemUri, cwd, test.getOptions());
            }
            catch (e) {
                this.setTestOutcome(test, Test_1.TestOutcome.FAILED);
                reject(e);
                return;
            }
        });
    }
    assertMatchesAnExpectedPlan(result, test) {
        const success = result.plans.every(plan => test.getExpectedPlans()
            .map(expectedPlanFileName => test.toAbsolutePath(expectedPlanFileName))
            .some(expectedPlanPath => this.areSame(plan, this.loadPlan(expectedPlanPath))));
        if (success) {
            this.outputTestResult(test, Test_1.TestOutcome.SUCCESS, result.elapsedTime);
        }
        else {
            this.outputTestResult(test, Test_1.TestOutcome.FAILED, result.elapsedTime, "Actual plan is NOT matching any of the expected plans.");
        }
    }
    outputTestResult(test, outcome, elapsedTime, error) {
        this.setTestOutcome(test, outcome);
        this.report.outputTestResult(test, outcome, elapsedTime, error);
    }
    setTestOutcome(test, testOutcome) {
        this.pTestTreeDataProvider.setTestOutcome(test, testOutcome);
    }
    areSame(actualPlan, expectedPlan) {
        // todo: refer to VAlStep to check that plan is equivalent, rather than same
        if (actualPlan.steps.length !== expectedPlan.steps.length) {
            return false;
        }
        if (actualPlan.steps.length === 0) {
            return true;
        }
        const epsilon = vscode_1.workspace.getConfiguration().get("pddlPlanner.epsilonTimeStep", configuration_1.DEFAULT_EPSILON);
        if (!pddl_workspace_1.PlanStep.equalsWithin(actualPlan.makespan, expectedPlan.makespan, epsilon)) {
            return false;
        }
        for (let index = 0; index < actualPlan.steps.length; index++) {
            const actualStep = actualPlan.steps[index];
            const expectedStep = expectedPlan.steps[index];
            if (!expectedStep.equals(actualStep, epsilon)) {
                return false;
            }
        }
        return true;
    }
    loadPlan(expectedPlanPath) {
        const expectedPlanText = fs_1.readFileSync(expectedPlanPath, { encoding: "utf-8" });
        const epsilon = vscode_1.workspace.getConfiguration().get("pddlPlanner.epsilonTimeStep", configuration_1.DEFAULT_EPSILON);
        return pddl_workspace_1.parser.PddlPlannerOutputParser.parseOnePlan(expectedPlanText, vscode_uri_1.URI.file(expectedPlanPath), epsilon);
    }
    async assertValid(test) {
        const domainExists = await this.assertFileExists(test.getDomainUri().fsPath, 'Domain', test.getDomain());
        const problemExists = await this.assertFileExists(test.getProblemUri().fsPath, 'Problem', test.getProblem());
        const expectedPlanPromises = test.getExpectedPlans().map(async (planPath) => await this.assertFileExists(test.toAbsolutePath(planPath), 'Test', planPath));
        const expectedPlansExist = await Promise.all(expectedPlanPromises);
        return domainExists && problemExists
            && expectedPlansExist.every(v => v);
    }
    async assertFileExists(path, resourceName, fileName) {
        const exists = await pddl_workspace_1.utils.afs.exists(path);
        if (!exists) {
            vscode_1.window.showErrorMessage(`${resourceName} file not found: ${fileName}`);
        }
        return exists;
    }
}
exports.PTestExplorer = PTestExplorer;
//# sourceMappingURL=PTestExplorer.js.map