/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestResult = exports.PTestReport = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const Test_1 = require("./Test");
const PTestReportView_1 = require("./PTestReportView");
const PTestCommands_1 = require("./PTestCommands");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
/** Gathers the output of running PDDL Test cases and summarizes them into a WebView table. */
class PTestReport {
    constructor(context, outputWindow) {
        this.context = context;
        this.outputWindow = outputWindow;
        this.manifests = new ManifestMap();
        vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(PTestCommands_1.PTEST_REPORT_VIEW, () => this.show());
    }
    dispose() {
        if (this.outputWindow) {
            this.outputWindow.dispose();
        }
    }
    startingManifest(manifest) {
        const manifestLocation = vscode_1.workspace.asRelativePath(manifest.path, true);
        this.output(`Executing tests from ${manifestLocation}.`);
        this.addManifestIfAbsent(manifest);
    }
    addManifestIfAbsent(manifest) {
        if (this.manifests.putIfAbsent(manifest, () => new TestResultMap())) {
            if (this.view) {
                this.view.updatePage();
            }
        }
    }
    upsertTestResult(test, result) {
        const testManifest = test.getManifest();
        if (!testManifest) {
            throw new Error(`Test ${test.getLabel()} has no manifest.`);
        }
        this.manifests.putIfAbsent(testManifest, () => new TestResultMap());
        const testMap = this.manifests.get(testManifest);
        testMap && testMap.set(test, result);
        if (this.view) {
            this.view.updatePage();
        }
    }
    /**
     * Marks manifest test as finished
     * @param manifest manifest whose tests just finished
     */
    finishedManifest(manifest) {
        const manifestLocation = vscode_1.workspace.asRelativePath(manifest.path, true);
        this.outputWindow.appendLine(`Finished executing tests from ${manifestLocation}.`);
    }
    getManifests() {
        return this.manifests.keyList();
    }
    getManifestTestResultsOrThrow(manifest) {
        return utils_1.assertDefined(this.manifests.get(manifest), `Manifest not found: ` + manifest.uri.toString());
    }
    getTestCases(manifest) {
        return this.getManifestTestResultsOrThrow(manifest).keyList();
    }
    getTestResultOrThrow(test) {
        const testManifest = test.getManifest();
        const testUri = test.getUri();
        if (!testManifest || !testUri) {
            throw new Error(`Test ${test.getLabel()} has no manifest or URI.`);
        }
        const results = this.getManifestTestResultsOrThrow(testManifest);
        return utils_1.assertDefined(results.get(test), `Test not found in the result set: ` + testUri.toString());
    }
    outputTestResult(test, outcome, elapsedTime, error) {
        let outcomeChar = String.fromCharCode(0x2591);
        switch (outcome) {
            case Test_1.TestOutcome.SUCCESS:
                outcomeChar = String.fromCharCode(0x2611);
                break;
            case Test_1.TestOutcome.SKIPPED:
                outcomeChar = String.fromCharCode(0x2610);
                break;
            case Test_1.TestOutcome.FAILED:
                outcomeChar = String.fromCharCode(0x2612);
                break;
            // failed assertion: 	U+01C2 450
            // ‼	Double exclamation mark	0923
        }
        let outputMessage = `${outcomeChar} ${test.getLabel()}`;
        if (!Number.isNaN(elapsedTime)) {
            outputMessage += ` (${elapsedTime / 1000.0} sec)`;
        }
        if (error) {
            outputMessage += `\n    ${error}`;
        }
        this.outputWindow.appendLine(outputMessage);
        if (outcome === Test_1.TestOutcome.FAILED) {
            this.outputWindow.show(true);
        }
        this.upsertTestResult(test, new TestResult(outcome, outcomeChar, elapsedTime, error));
    }
    clearAndShow() {
        this.outputWindow.clear();
        this.manifests.clear();
        if (this.view) {
            this.view.updatePage();
        }
        this.show();
    }
    output(message) {
        this.outputWindow.appendLine(message);
    }
    show() {
        this.outputWindow.show(true);
        if (!this.view) {
            this.view = new PTestReportView_1.PTestReportView(this.context, this);
        }
        this.view.showReport();
    }
}
exports.PTestReport = PTestReport;
class TestResult {
    constructor(outcome, outcomeChar, elapsedTime, error) {
        this.outcome = outcome;
        this.outcomeChar = outcomeChar;
        this.elapsedTime = elapsedTime;
        this.error = error;
    }
}
exports.TestResult = TestResult;
class ManifestMap extends pddl_workspace_1.utils.StringifyingMap {
    stringifyKey(key) {
        return key.uri.toString();
    }
}
class TestResultMap extends pddl_workspace_1.utils.StringifyingMap {
    stringifyKey(key) {
        return key.getUriOrThrow().toString();
    }
}
//# sourceMappingURL=PTestReport.js.map