/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.StudentSession = exports.Classroom = void 0;
const path = require("path");
const fs = require("fs");
const pddl_workspace_1 = require("pddl-workspace");
const SessionConfiguration_1 = require("./SessionConfiguration");
const vscode_1 = require("vscode");
/** Represents classroom of student sessions. */
class Classroom {
    constructor(templateSourceControl, studentSessions) {
        this.templateSourceControl = templateSourceControl;
        this.studentSessions = studentSessions;
        this.parentPath = Classroom.getTemplateParentPath(templateSourceControl);
    }
    async createWorkspace() {
        await Promise.all(this.studentSessions
            .map(async (studentSession) => await this.createFolderWithConfig(studentSession)));
        const templateFolderName = path.basename(this.templateSourceControl.getWorkspaceFolder().uri.fsPath);
        let folders = this.studentSessions.map(s => this.toWorkspaceFolderSpec(s));
        folders = [{ name: 'Template', path: templateFolderName }, ...folders];
        const workspaceObj = Object.create(null);
        workspaceObj["folders"] = folders;
        workspaceObj["settings"] = {};
        const workspaceAsString = JSON.stringify(workspaceObj, null, 4);
        const workspaceFilePath = await this.createNewWorkspaceFile();
        await fs.promises.writeFile(workspaceFilePath, workspaceAsString);
        return workspaceFilePath;
    }
    toWorkspaceFolderSpec(s) {
        return { name: s.identity.getEffectiveName(), path: s.identity.getEffectivePath() };
    }
    /**
     * Creates a new workspace file to ensure no previous classroom workspace configuration is overwritten.
     */
    async createNewWorkspaceFile() {
        let filePath;
        let fileAlreadyExists;
        let counter = 1;
        do {
            filePath = path.join(this.parentPath, `classroom${counter++}.code-workspace`);
            fileAlreadyExists = await pddl_workspace_1.utils.afs.exists(filePath);
        } while (fileAlreadyExists);
        return filePath;
    }
    static getTemplateParentPath(templateSourceControl) {
        return path.dirname(templateSourceControl.getWorkspaceFolder().uri.fsPath);
    }
    static getSessionPath(templateSourceControl, student) {
        const parentPath = Classroom.getTemplateParentPath(templateSourceControl);
        return path.join(parentPath, student.getEffectivePath());
    }
    async createFolderWithConfig(studentSession) {
        const folderPath = Classroom.getSessionPath(this.templateSourceControl, studentSession.identity);
        // todo: use workspace.fs.createDirectory
        await pddl_workspace_1.utils.afs.mkdirIfDoesNotExist(folderPath, 0o644);
        await SessionConfiguration_1.saveConfiguration(vscode_1.Uri.file(folderPath), studentSession.sessionConfiguration);
    }
}
exports.Classroom = Classroom;
class StudentSession {
    constructor(identity, sessionConfiguration) {
        this.identity = identity;
        this.sessionConfiguration = sessionConfiguration;
    }
}
exports.StudentSession = StudentSession;
//# sourceMappingURL=Classroom.js.map