"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.duplicateSession = exports.uploadSession = exports.getSession = exports.checkSession = exports.SessionRepository = exports.SESSION_SCHEME = exports.areIdentical = exports.SessionContent = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const path = require("path");
const utils_1 = require("../utils");
const SessionConfiguration_1 = require("./SessionConfiguration");
const httpUtils_1 = require("../httpUtils");
const PlanningDomains_1 = require("../catalog/PlanningDomains");
/** Represents one Planning.Domains session and meta-data. */
class SessionContent {
    constructor(hash, writeHash, versionDate, files, plugins) {
        this.hash = hash;
        this.writeHash = writeHash;
        this.versionDate = versionDate;
        this.files = files;
        this.plugins = plugins;
    }
    static from(configuration, versionDate) {
        if (configuration.files === undefined) {
            throw new Error("Failed assertion: SessionConfiguration.files undefined");
        }
        return new SessionContent(configuration.hash, configuration.writeHash, versionDate, configuration.files, new Map());
    }
    canCommit() {
        return this.writeHash !== null && this.writeHash !== undefined;
    }
    getHash() {
        var _a, _b;
        return (_b = (_a = this.writeHash) !== null && _a !== void 0 ? _a : this.hash) !== null && _b !== void 0 ? _b : utils_1.throwForUndefined("One of read/write hash codes must be set.");
    }
}
exports.SessionContent = SessionContent;
function areIdentical(first, second) {
    return utils_1.compareMaps(first, second);
}
exports.areIdentical = areIdentical;
exports.SESSION_SCHEME = 'planning.domains.session';
/** This binds the local and remote repository. */
class SessionRepository {
    constructor(workspaceFolder, session) {
        this.workspaceFolder = workspaceFolder;
        this.sessionHash = session.getHash();
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    provideOriginalResource(uri, _) {
        // converts the local file uri to planning.domains.session:sessionId/file.ext
        const workspaceFolder = vscode_1.workspace.getWorkspaceFolder(uri);
        if (workspaceFolder) {
            const fileName = vscode_1.workspace.asRelativePath(uri, false);
            return SessionRepository.createDocumentUri(workspaceFolder.uri, fileName);
        }
        else {
            return undefined;
        }
    }
    static createDocumentUri(workspaceFolder, fileName) {
        return workspaceFolder.with({ scheme: exports.SESSION_SCHEME, query: fileName });
    }
    /**
     * Creates a local file path in the local workspace that corresponds to the given file in the session.
     *
     * @param fileName session file name
     * @returns path of the locally cloned session file
     * @deprecated use createLocalResourceUri
     */
    createLocalResourcePath(fileName) {
        return path.join(this.workspaceFolder.uri.fsPath, fileName);
    }
    /**
     * Creates a local file URI in the local workspace that corresponds to the given file in the session.
     *
     * @param fileName session file name
     * @returns URI of the locally cloned session file
     */
    createLocalResourceUri(fileName) {
        return vscode_1.Uri.joinPath(this.workspaceFolder.uri, fileName);
    }
    /**
     * Returns write or read session hash
     */
    getSessionHash() {
        return this.sessionHash;
    }
}
exports.SessionRepository = SessionRepository;
const SESSION_URL = "http://editor.planning.domains/session/";
const SESSION_PLUGINS_PATTERN = /define\s*\(\s*function\s*\(\s*\)\s*\{\s*return\s*{\s*meta:\s*true\s*,\s*plugins\s*:\s*({[\S\s]*})\s*}\s*}\s*\);/ms;
const SESSION_DETAILS_PATTERN = /window\.session_details\s*=\s*{\s*(?:readwrite_hash:\s*"(\w+)"\s*,\s*)?read_hash:\s*"(\w+)"\s*,\s*last_change:\s*"([\w: \(\)\+]+)",?\s*};/;
/**
 * Tests whether the session exists, determines whether it is writable and what is the last change time.
 * @param sessionId session read/write hash code
 * @throws error when session does not exist, or session type is not recognized
 */
async function checkSession(sessionId) {
    const url = `${SESSION_URL}check/${sessionId}`;
    const response = await httpUtils_1.getJson(url);
    PlanningDomains_1.checkResponseForError(response);
    let sessionMode;
    switch (response["type"].toLowerCase()) {
        case "read":
            sessionMode = SessionConfiguration_1.SessionMode.READ_ONLY;
            break;
        case "readwrite":
            sessionMode = SessionConfiguration_1.SessionMode.READ_WRITE;
            break;
        default:
            throw new Error("Unexpected session type: " + response["type"]);
    }
    // last_change contains last session change time (round it down to seconds)
    const sessionVersionDate = Math.floor(Date.parse(response["last_change"]) / 1000) * 1000;
    return [sessionMode, sessionVersionDate];
}
exports.checkSession = checkSession;
async function getSession(sessionConfiguration) {
    const rawSession = await getRawSession(sessionConfiguration);
    const savedTabsJson = JSON.parse(rawSession.domainFilesAsString);
    const fileNames = Object.keys(savedTabsJson);
    const sessionFiles = new Map();
    fileNames.forEach(fileName => sessionFiles.set(fileName, savedTabsJson[fileName]));
    return new SessionContent(rawSession.readOnlyHash, rawSession.readWriteHash, rawSession.sessionDate, sessionFiles, rawSession.plugins);
}
exports.getSession = getSession;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function createSessionContent(pluginDefinitions) {
    const sessionDefinitionAsString = JSON.stringify(pluginDefinitions, null, 4);
    return `// Put this file online somewhere and import it as a plugin

	define(function () {
    return {
        meta: true,
		plugins:
${sessionDefinitionAsString}
	}
});
`;
}
async function uploadSession(session) {
    if (!session.writeHash) {
        throw new Error("Check if the session is writable first.");
    }
    const rawLatestSession = await getRawSession(session);
    // re-place the saved tabs in the plugins
    const newPluginList = [...rawLatestSession.plugins.keys()]
        .map(oldPluginName => {
        const oldPlugin = rawLatestSession.plugins.get(oldPluginName);
        let newPlugin;
        if (oldPlugin.name === SAVE_TABS_PLUGIN_NAME) {
            newPlugin = {
                name: oldPlugin.name,
                url: oldPlugin.url,
                settings: pddl_workspace_1.utils.serializationUtils.strMapToObj(session.files),
                settingsAsString: undefined
            };
        }
        else {
            newPlugin = oldPlugin;
        }
        return newPlugin;
    });
    // re-construct the session plugin definition
    const newPlugins = Object.create(null);
    newPluginList.forEach(plugin => {
        const newPlugin = Object.create(null);
        newPlugin["url"] = plugin.url;
        newPlugin["settings"] = plugin.settings;
        newPlugins[plugin.name] = newPlugin;
    });
    const newContent = createSessionContent(newPlugins);
    const postBody = Object.create(null);
    postBody["content"] = newContent;
    postBody["read_hash"] = session.hash;
    postBody["readwrite_hash"] = session.writeHash;
    const url = `${SESSION_URL}${session.writeHash}`;
    const postResult = await httpUtils_1.postJson(url, postBody);
    if (postResult["error"]) {
        throw new Error(postResult["message"]);
    }
    // get the latest session
    return getSession(session);
}
exports.uploadSession = uploadSession;
async function duplicateSession(session) {
    const rawLatestOrigSession = await getRawSession(session);
    // replace the session files
    const newFilesAsString = JSON.stringify(pddl_workspace_1.utils.serializationUtils.strMapToObj(session.files), null, 4);
    const newContent = rawLatestOrigSession.sessionContent
        .replace(rawLatestOrigSession.sessionDetails, '') // strip the window.session.details= assignment
        .replace(rawLatestOrigSession.domainFilesAsString, newFilesAsString);
    const postBody = Object.create(null);
    postBody["content"] = newContent;
    const postResult = await httpUtils_1.postJson(SESSION_URL, postBody);
    if (postResult["error"]) {
        throw new Error(postResult["message"]);
    }
    // get the latest session
    return postResult["result"]["readwrite_hash"];
}
exports.duplicateSession = duplicateSession;
const SAVE_TABS_PLUGIN_NAME = "save-tabs";
const SOLVER_PLUGIN_NAME = "solver";
/**
 * Fetches session from the planning.domains server.
 * @param sessionConfiguration session identity
 */
async function getRawSession(sessionConfiguration) {
    var _a;
    const url = sessionConfiguration.writeHash ?
        `${SESSION_URL}edit/${sessionConfiguration.writeHash}` :
        `${SESSION_URL}${sessionConfiguration.hash}`;
    const sessionContent = await httpUtils_1.getText(url);
    if (sessionContent.match(/not found/i)) {
        throw new Error(`Session ${(_a = sessionConfiguration.writeHash) !== null && _a !== void 0 ? _a : sessionConfiguration.hash} not found.`);
    }
    let sessionDetails;
    let sessionDate;
    let readWriteHash;
    let readOnlyHash;
    SESSION_DETAILS_PATTERN.lastIndex = 0;
    const matchDetails = SESSION_DETAILS_PATTERN.exec(sessionContent);
    if (matchDetails) {
        sessionDetails = matchDetails[0];
        readWriteHash = matchDetails[1];
        readOnlyHash = matchDetails[2];
        sessionDate = Date.parse(matchDetails[3]);
    }
    else {
        console.log("Malformed saved session. Could not extract session date. Session content:" + sessionContent);
        throw new Error("Malformed saved session. Could not extract session date.");
    }
    // extract plugins
    const plugins = new Map();
    SESSION_PLUGINS_PATTERN.lastIndex = 0;
    let pluginsMatch = SESSION_PLUGINS_PATTERN.exec(sessionContent);
    if (pluginsMatch = SESSION_PLUGINS_PATTERN.exec(sessionContent)) {
        const rawPlugins = JSON.parse(pluginsMatch[1]);
        [SAVE_TABS_PLUGIN_NAME, SOLVER_PLUGIN_NAME].forEach(pluginName => {
            if (rawPlugins.hasOwnProperty(pluginName)) {
                plugins.set(pluginName, toRawSessionPlugin(pluginName, rawPlugins[pluginName]));
            }
        });
    }
    else {
        console.log("Malformed saved session plugins. Could not extract session plugins. Session content:" + sessionContent);
        throw new Error("Malformed saved session. Could not extract session plugins.");
    }
    if (!plugins.has(SAVE_TABS_PLUGIN_NAME)) {
        throw new Error("Saved session contains no saved tabs.");
    }
    const domainFilesString = plugins.get(SAVE_TABS_PLUGIN_NAME).settingsAsString;
    return {
        sessionDetails: sessionDetails,
        sessionContent: sessionContent,
        sessionDate: sessionDate,
        readOnlyHash: readOnlyHash,
        readWriteHash: readWriteHash,
        domainFilesAsString: utils_1.assertDefined(domainFilesString, "save-tabs plugin settings"),
        plugins: plugins,
    };
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function toRawSessionPlugin(name, json) {
    return {
        name: name,
        url: json["url"],
        settings: json["settings"],
        settingsAsString: JSON.stringify(json["settings"], null, 4)
    };
}
//# sourceMappingURL=SessionRepository.js.map