"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionSourceControl = exports.SESSION_COMMAND_REFRESH_ALL = exports.SESSION_COMMAND_CHECKOUT = exports.SESSION_COMMAND_LOAD = void 0;
const vscode = require("vscode");
const SessionRepository_1 = require("./SessionRepository");
const path = require("path");
const utils_1 = require("../utils");
const SessionConfiguration_1 = require("./SessionConfiguration");
const configuration_1 = require("../configuration/configuration");
const workspaceFs_1 = require("../util/workspaceFs");
/**
 * Command for cloning a session to the local storage.
 * Arguments: sessionId?, localWorkspaceUri?
 */
exports.SESSION_COMMAND_LOAD = 'pddl.planning.domains.session.load';
/**
 * Command for downloading the latest version of the session to the local storage.
 */
exports.SESSION_COMMAND_CHECKOUT = 'pddl.planning.domains.session.checkout';
exports.SESSION_COMMAND_REFRESH_ALL = 'pddl.planning.domains.session.refresh_all';
var ChangedResourceState;
(function (ChangedResourceState) {
    ChangedResourceState[ChangedResourceState["New"] = 0] = "New";
    ChangedResourceState[ChangedResourceState["Deleted"] = 1] = "Deleted";
    ChangedResourceState[ChangedResourceState["Dirty"] = 2] = "Dirty";
})(ChangedResourceState || (ChangedResourceState = {}));
/** Handles the version control for session files. */
class SessionSourceControl {
    constructor(context, workspaceFolder, session, overwrite) {
        this.workspaceFolder = workspaceFolder;
        /** This represents the most recent session version on the server. */
        this.latestSessionVersionDate = Number.MIN_VALUE; // until actual value is established
        this._onRepositoryChange = new vscode.EventEmitter();
        this.isRefreshing = false;
        this.sessionScm = vscode.scm.createSourceControl('planningDomainsSession', 'Session #' + session.getHash(), workspaceFolder.uri);
        this.changedResources = this.sessionScm.createResourceGroup('workingTree', 'Changes');
        this.sessionRepository = new SessionRepository_1.SessionRepository(workspaceFolder, session);
        this.sessionScm.quickDiffProvider = this.sessionRepository;
        this.sessionScm.inputBox.placeholder = session.canCommit() ? '' : 'Read-only session!';
        this.sessionScm.inputBox.visible = false; // available from VS Code 1.45
        const fileSystemWatcher = vscode.workspace.createFileSystemWatcher(new vscode.RelativePattern(workspaceFolder, "*.*"));
        fileSystemWatcher.onDidChange(uri => this.onResourceChange(uri), context.subscriptions);
        fileSystemWatcher.onDidCreate(uri => this.onResourceChange(uri), context.subscriptions);
        fileSystemWatcher.onDidDelete(uri => this.onResourceChange(uri), context.subscriptions);
        context.subscriptions.push(this.sessionScm);
        context.subscriptions.push(fileSystemWatcher);
        // clone the session to the local workspace
        this.setSession(session, overwrite)
            .then(() => this.refresh()) //find out if there is a new version
            .catch(err => vscode.window.showErrorMessage(err));
    }
    static async fromSessionId(id, mode, context, workspaceFolder, overwrite) {
        const sessionConfiguration = SessionConfiguration_1.toSessionConfiguration(id, mode);
        return await SessionSourceControl.fromConfiguration(sessionConfiguration, workspaceFolder, context, overwrite);
    }
    static async fromConfiguration(configuration, workspaceFolder, context, overwrite) {
        const session = configuration.versionDate ?
            SessionRepository_1.SessionContent.from(configuration, configuration.versionDate) :
            await SessionRepository_1.getSession(configuration);
        return new SessionSourceControl(context, workspaceFolder, session, overwrite);
    }
    refreshStatusBar() {
        let title = "$(repo) ";
        if (this.session.canCommit()) {
            title += '$(pencil)';
        }
        title += utils_1.toFuzzyRelativeTime(this.session.versionDate);
        if (this.session.versionDate < this.latestSessionVersionDate) {
            title = `$(cloud-download) ${title} (stale)`;
        }
        if (this.getSourceControl().count) {
            title += " $(primitive-dot)";
        }
        const refreshTitle = `$(sync${this.isRefreshing ? '~spin' : ''})`;
        this.sessionScm.statusBarCommands = [
            {
                "command": exports.SESSION_COMMAND_CHECKOUT,
                "arguments": [this],
                "title": title,
                "tooltip": "Fetch latest version of the session files."
            },
            {
                "command": exports.SESSION_COMMAND_REFRESH_ALL,
                "title": refreshTitle,
                "tooltip": "Refresh status of all Planning.domains sessions"
            }
        ];
    }
    async commitAll() {
        if (!this.changedResources.resourceStates.length) {
            vscode.window.showErrorMessage("There is nothing to commit.");
            return;
        }
        if (!this.session.canCommit()) {
            vscode.window.showErrorMessage("This session was open as read-only! Save it using the Clone command.");
            return;
        }
        await this.refresh();
        if (this.session.versionDate < this.latestSessionVersionDate) {
            vscode.window.showErrorMessage("Checkout the latest session version before committing your changes.");
        }
        else {
            try {
                await vscode.window.withProgress({ location: vscode.ProgressLocation.SourceControl, title: 'Committing...' }, async (progress) => {
                    const currentSession = await this.getCurrentSession();
                    progress.report({ message: 'Latest session refreshed.', increment: 33 });
                    const newSessionContent = await SessionRepository_1.uploadSession(currentSession);
                    progress.report({ message: 'Session committed.', increment: 33 });
                    await this.setSession(newSessionContent, false);
                    progress.report({ message: 'Session re-downloaded.', increment: 34 });
                    this.sessionScm.inputBox.value = '';
                });
            }
            catch (ex) {
                vscode.window.showErrorMessage("Cannot commit changes to Planning.Domains session. " + ex.message);
            }
        }
    }
    /** Constructs a session instance from the current set of files in the workspace. */
    async getCurrentSession() {
        const newFiles = new Map();
        const localFileNames = await this.getLocalFileNames();
        for (const fileName of localFileNames) {
            newFiles.set(fileName, await this.getFileContent(fileName));
        }
        return new SessionRepository_1.SessionContent(this.session.hash, this.session.writeHash, this.session.versionDate, newFiles, new Map());
    }
    /**
     * Throws away all local changes and resets all files to the checked out version of the repository.
     */
    async resetFilesToCheckedOutVersion() {
        if (this.getLocalModifiedResources() && this.getLocalModifiedResources().length > 0) {
            const discardAnswer = "Discard";
            const answer = await vscode.window.showWarningMessage(`Discard ${this.getLocalModifiedResources().length} changes?`, { modal: true }, discardAnswer, "Cancel");
            if (answer !== discardAnswer) {
                return;
            }
        }
        this.session.files.forEach((fileContent, fileName) => this.resetFile(fileName, fileContent));
    }
    /** Resets the given local file content to the checked-out version. */
    async resetFile(fileName, fileContent) {
        const fileUri = this.sessionRepository.createLocalResourceUri(fileName);
        await vscode.workspace.fs.writeFile(fileUri, Buffer.from(fileContent, "utf-8"));
    }
    async getFileContent(fileName) {
        const fileUri = this.sessionRepository.createLocalResourceUri(fileName);
        const document = await vscode.workspace.openTextDocument(fileUri);
        return document.getText();
    }
    async tryCheckout(newVersion) {
        var _a;
        if (!Number.isFinite(this.latestSessionVersionDate)) {
            return;
        }
        if (newVersion === undefined) {
            const allVersions = [...new Set([this.session.versionDate, this.latestSessionVersionDate])]
                .map(ver => new VersionQuickPickItem(ver, ver === this.session.versionDate));
            const newVersionPick = await vscode.window.showQuickPick(allVersions, { canPickMany: false, placeHolder: 'Select a version...' });
            if (newVersionPick) {
                newVersion = newVersionPick.version;
            }
            else {
                return;
            }
        }
        if (newVersion === this.session.versionDate) {
            return;
        } // the same version was selected
        const localUntrackedResources = this.getLocalUntrackedResources();
        if (this.changedResources.resourceStates.length - localUntrackedResources.length) {
            vscode.window.showErrorMessage(`There is one or more changed resources. Discard your local changes before checking out another version.`, { modal: true });
        }
        else {
            try {
                const newSession = await SessionRepository_1.getSession(this.session);
                // check if any of the local untracked files conflict with files in the new session
                const conflictingUntrackedResources = localUntrackedResources
                    .filter(resource => newSession.files.has(path.basename(resource.resourceUri.fsPath)));
                if (conflictingUntrackedResources.length > 0) {
                    const conflictingUntrackedResourceNames = conflictingUntrackedResources
                        .map(resource => this.toOpenFileNotificationLink(resource.resourceUri))
                        .join(", ");
                    vscode.window.showErrorMessage(`Merge conflict: delete/rename your local version of following session file(s): ${conflictingUntrackedResourceNames} before checking out the session content.`, { modal: false });
                }
                else {
                    await this.setSession(newSession, true);
                }
            }
            catch (ex) {
                vscode.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
            }
        }
    }
    getLocalModifiedResources() {
        return this.changedResources.resourceStates.filter(resource => { var _a; return !((_a = resource.decorations) === null || _a === void 0 ? void 0 : _a.faded); });
    }
    getLocalUntrackedResources() {
        return this.changedResources.resourceStates.filter(resource => { var _a; return (_a = resource.decorations) === null || _a === void 0 ? void 0 : _a.faded; });
    }
    toOpenFileNotificationLink(resourceUri) {
        return `[${path.basename(resourceUri.fsPath)}](${this.toOpenFileCommand(resourceUri)})`;
    }
    toOpenFileCommand(resourceUri) {
        return encodeURI('command:vscode.open?' + JSON.stringify([resourceUri.toString()]));
    }
    async setSession(newSession, overwrite) {
        if (newSession.versionDate > this.latestSessionVersionDate) {
            this.latestSessionVersionDate = newSession.versionDate;
        }
        this.session = newSession;
        if (overwrite) {
            this.resetFilesToCheckedOutVersion();
        } // overwrite local file content
        this._onRepositoryChange.fire(this.session);
        await this.updateChangedGroup();
        await this.saveWorkspaceSettings();
        await this.saveCurrentConfiguration();
    }
    getSession() {
        return this.session;
    }
    getWorkspaceFolder() {
        return this.workspaceFolder;
    }
    getSourceControl() {
        return this.sessionScm;
    }
    getRepository() {
        return this.sessionRepository;
    }
    /** save configuration for later VS Code sessions */
    async saveCurrentConfiguration() {
        return await SessionConfiguration_1.saveConfiguration(this.workspaceFolder.uri, this.session);
    }
    /** Saves setting of eligible session plugins to workspace configuration. */
    async saveWorkspaceSettings() {
        if (this.session.plugins.has(SessionSourceControl.SOLVER_PLUGIN)) {
            const solver = this.session.plugins.get(SessionSourceControl.SOLVER_PLUGIN);
            if ((solver === null || solver === void 0 ? void 0 : solver.url) !== "/plugins/solver.js") {
                return;
            }
            const solverUrl = solver.settings["url"];
            // todo: this configuration is upgraded right away; move to the new planner configuration pattern
            await vscode.workspace.getConfiguration(configuration_1.PDDL_PLANNER, this.workspaceFolder.uri).update(configuration_1.EXECUTABLE_OR_SERVICE, solverUrl + "/solve", vscode.ConfigurationTarget.WorkspaceFolder);
        }
    }
    /**
     * Refresh is used when the information on the server may have changed.
     * For example another user updates the session files online.
     */
    async refresh() {
        this.setRefreshing(true);
        try {
            const sessionCheckResult = await SessionRepository_1.checkSession(this.session.getHash());
            this.latestSessionVersionDate = sessionCheckResult[1];
        }
        catch (ex) {
            // typically the ex.statusCode == 404, when there is no further version
        }
        await this.updateChangedGroup();
        this.setRefreshing(false);
    }
    get onRepositoryChange() {
        return this._onRepositoryChange.event;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    onResourceChange(_uri) {
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
        this.timeout = setTimeout(() => this.tryUpdateChangedGroup(), 500);
    }
    async tryUpdateChangedGroup() {
        var _a;
        try {
            await this.updateChangedGroup();
        }
        catch (ex) {
            vscode.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
        }
    }
    /** Refreshes the list of changed resources under this version control. */
    async updateChangedGroup() {
        // for simplicity we ignore which document was changed in this event and scan all of them
        const changedResources = [];
        const uris = this.provideSourceControlledResources();
        const otherFolderFiles = await this.getLocalFileNames();
        for (const uri of uris) {
            let state;
            if (await workspaceFs_1.exists(uri)) {
                const document = await vscode.workspace.openTextDocument(uri);
                if (this.isDirty(document)) {
                    state = ChangedResourceState.Dirty;
                }
                otherFolderFiles.splice(otherFolderFiles.indexOf(path.basename(uri.fsPath)), 1);
            }
            else {
                state = ChangedResourceState.Deleted;
            }
            if (state) {
                const resourceState = this.toSourceControlResourceState(uri, state);
                changedResources.push(resourceState);
            }
        }
        for (const otherFile of otherFolderFiles) {
            const resourceUri = vscode.Uri.joinPath(this.getWorkspaceFolder().uri, otherFile);
            const fileStats = await vscode.workspace.fs.stat(resourceUri);
            if (fileStats.type === vscode.FileType.Directory) {
                continue;
            }
            // add this file as a new file to the session changed resources
            const resourceState = this.toSourceControlResourceState(resourceUri, ChangedResourceState.New);
            changedResources.push(resourceState);
        }
        this.changedResources.resourceStates = changedResources;
        this.sessionScm.count = this.changedResources.resourceStates.length;
        // force refreshing of the status-bar commands
        this.refreshStatusBar();
    }
    /**
     * Enumerates the resources under source control.
     */
    provideSourceControlledResources() {
        return [...this.session.files.keys()]
            .map(fileName => this.sessionRepository.createLocalResourceUri(fileName));
    }
    async getLocalFileNames() {
        const children = await vscode.workspace.fs.readDirectory(this.getWorkspaceFolder().uri);
        return children
            // keep only files, not directories
            .filter(fileEnt => fileEnt[1] === vscode.FileType.File)
            .map(fileEnt => fileEnt[0])
            // keep only pddl files, exclude the config file
            .filter(fileName => fileName !== SessionConfiguration_1.CONFIGURATION_FILE)
            // keep only pddl files, exclude VS Code workspace file
            .filter(fileName => !fileName.endsWith('.code-workspace'));
    }
    /** Determines whether the resource is different, regardless of line endings. */
    isDirty(doc) {
        const originalText = this.session.files.get(path.basename(doc.uri.fsPath));
        return (originalText === null || originalText === void 0 ? void 0 : originalText.replace('\r', '')) !== doc.getText().replace('\r', '');
    }
    setRefreshing(isRefreshing) {
        this.isRefreshing = isRefreshing;
        this.refreshStatusBar();
    }
    toSourceControlResourceState(docUri, state) {
        const repositoryUri = this.sessionRepository.provideOriginalResource(docUri, new vscode.CancellationTokenSource().token);
        const fileName = path.basename(docUri.fsPath);
        let command;
        let tooltip;
        switch (state) {
            case ChangedResourceState.Dirty:
                command = {
                    title: "Show changes",
                    command: "vscode.diff",
                    arguments: [repositoryUri, docUri, `Session#${this.session.hash}:${fileName} ↔ Local changes`],
                    tooltip: "Diff your changes"
                };
                tooltip = '';
                break;
            case ChangedResourceState.New:
                command = {
                    title: "Show document",
                    command: 'vscode.open',
                    arguments: [docUri]
                };
                tooltip = 'File will be added to the session.';
                break;
            case ChangedResourceState.Deleted:
                command = {
                    title: "Show content of the removed document",
                    command: 'vscode.open',
                    arguments: [repositoryUri]
                };
                tooltip = 'File was locally deleted and will be removed from the session.';
                break;
            default:
                throw new Error("Unexpected resource state: " + state);
        }
        const resourceState = {
            resourceUri: docUri,
            command: command,
            decorations: {
                strikeThrough: state === ChangedResourceState.Deleted,
                faded: state === ChangedResourceState.New,
                tooltip: tooltip
            }
        };
        return resourceState;
    }
    /**
     * Creates a copy of the current session on the server and requests it to be cloned to the targetLocalFolder.
     * @param targetLocalFolder local folder uri, where the session should be cloned
     * @param downloadIt if _true_, session files are downloaded after the session is duplicated
     * @returns new session hash
     */
    async duplicateAsWritable(targetLocalFolder, downloadIt) {
        var _a;
        try {
            const currentSession = await this.getCurrentSession();
            const newSessionWriteHash = await SessionRepository_1.duplicateSession(currentSession);
            if (downloadIt) {
                // trigger the command to download the new session
                vscode.commands.executeCommand(exports.SESSION_COMMAND_LOAD, newSessionWriteHash, targetLocalFolder);
            }
            return newSessionWriteHash;
        }
        catch (ex) {
            throw new Error("Failed creating duplicate session in Planning.Domains. " + ((_a = ex.message) !== null && _a !== void 0 ? _a : ex));
        }
    }
    dispose() {
        this._onRepositoryChange.dispose();
        this.sessionScm.dispose();
    }
}
exports.SessionSourceControl = SessionSourceControl;
SessionSourceControl.SOLVER_PLUGIN = "solver";
class VersionQuickPickItem {
    constructor(version, picked) {
        this.version = version;
        this.picked = picked;
    }
    get label() {
        return `Version from ${utils_1.toFuzzyRelativeTime(this.version)}`;
    }
    get description() {
        return this.picked ? ' (currently checked-out)' : '';
    }
    get alwaysShow() {
        return this.picked;
    }
}
//# sourceMappingURL=SessionSourceControl.js.map