/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolInfoProvider = void 0;
const vscode_1 = require("vscode");
const SymbolUtils_1 = require("./SymbolUtils");
const utils_1 = require("../utils");
const pddl_workspace_1 = require("pddl-workspace");
class SymbolInfoProvider {
    constructor(pddlWorkspace) {
        this.pddlWorkspace = pddlWorkspace;
        this.symbolUtils = new SymbolUtils_1.SymbolUtils(pddlWorkspace);
    }
    async provideHover(document, position, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        await this.symbolUtils.assertFileParsed(document);
        const info = this.symbolUtils.getSymbolInfo(document, position);
        return info === null || info === void 0 ? void 0 : info.hover;
    }
    async provideReferences(document, position, context, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        await this.symbolUtils.assertFileParsed(document);
        const info = this.symbolUtils.getSymbolInfo(document, position);
        if (!info) {
            return [];
        }
        return this.symbolUtils.findSymbolReferences(document, info, context.includeDeclaration);
    }
    async provideDefinition(document, position, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        await this.symbolUtils.assertFileParsed(document);
        const info = this.symbolUtils.getSymbolInfo(document, position);
        return info === null || info === void 0 ? void 0 : info.location;
    }
    async provideDocumentSymbols(document, token) {
        if (token.isCancellationRequested) {
            return null;
        }
        await this.symbolUtils.assertFileParsed(document);
        const fileInfo = this.pddlWorkspace.getFileInfo(document);
        if (fileInfo === null || fileInfo === void 0 ? void 0 : fileInfo.isDomain()) {
            const domainInfo = fileInfo;
            const containerName = '';
            const actionSymbols = domainInfo.getActions()
                .map(action => {
                var _a;
                return new vscode_1.SymbolInformation((_a = action.name) !== null && _a !== void 0 ? _a : "unnamed action", vscode_1.SymbolKind.Module, containerName, SymbolUtils_1.SymbolUtils.toLocation(document, action.getLocation()));
            });
            if (!domainInfo.getProcesses()) {
                throw new Error(`Domain not parsed yet: ` + domainInfo.fileUri);
            }
            const processSymbols = domainInfo.getProcesses()
                .map(process => {
                var _a;
                return new vscode_1.SymbolInformation((_a = process.name) !== null && _a !== void 0 ? _a : "unnamed process", vscode_1.SymbolKind.Struct, containerName, SymbolUtils_1.SymbolUtils.toLocation(document, process.getLocation()));
            });
            if (!domainInfo.getEvents()) {
                throw new Error(`Domain not parsed yet: ` + domainInfo.fileUri);
            }
            const eventSymbols = domainInfo.getEvents()
                .map(event => {
                var _a;
                return new vscode_1.SymbolInformation((_a = event.name) !== null && _a !== void 0 ? _a : "unnamed event", vscode_1.SymbolKind.Event, containerName, SymbolUtils_1.SymbolUtils.toLocation(document, event.getLocation()));
            });
            const predicateSymbols = domainInfo.getPredicates().map(variable => new vscode_1.SymbolInformation(variable.declaredName, vscode_1.SymbolKind.Boolean, containerName, SymbolUtils_1.SymbolUtils.toLocation(document, variable.getLocation())));
            const functionSymbols = domainInfo.getFunctions().map(variable => new vscode_1.SymbolInformation(variable.declaredName, vscode_1.SymbolKind.Function, containerName, SymbolUtils_1.SymbolUtils.toLocation(document, variable.getLocation())));
            const symbols = actionSymbols.concat(processSymbols, eventSymbols, predicateSymbols, functionSymbols);
            return symbols;
        }
        else if (fileInfo === null || fileInfo === void 0 ? void 0 : fileInfo.isProblem()) {
            const problemInfo = fileInfo;
            try {
                return this.provideProblemSymbols(document, problemInfo, token);
            }
            catch (err) {
                utils_1.showError(err);
            }
        }
        return [];
    }
    async provideProblemSymbols(document, problemInfo, token) {
        if (token.isCancellationRequested) {
            return null;
        }
        const defineNode = problemInfo.syntaxTree.getDefineNode();
        if (defineNode) {
            const fullRange = utils_1.nodeToRange(document, defineNode);
            const firstLine = firstRow(fullRange);
            const defineSymbol = new vscode_1.DocumentSymbol('problem', problemInfo.name, vscode_1.SymbolKind.Namespace, fullRange, firstLine);
            const childrenNodes = defineNode.getChildrenOfType(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /\(\s*:/);
            const childrenSymbols = childrenNodes
                .map(node => this.createProblemSymbol(document, node));
            return [defineSymbol].concat(childrenSymbols);
        }
        return [];
    }
    createProblemSymbol(document, node) {
        const fullRange = utils_1.nodeToRange(document, node);
        const selectableRange = new vscode_1.Range(document.positionAt(node.getToken().getStart() + 1), document.positionAt(node.getToken().getEnd()));
        return new vscode_1.DocumentSymbol(node.getToken().tokenText.substr(1), '', vscode_1.SymbolKind.Package, fullRange, selectableRange);
    }
}
exports.SymbolInfoProvider = SymbolInfoProvider;
function firstRow(fullRange) {
    return fullRange.with({ end: fullRange.start.translate({ lineDelta: +1 }).with({ character: 0 }) });
}
//# sourceMappingURL=SymbolInfoProvider.js.map