/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParameterInfo = exports.ActionInfo = exports.TypeInfo = exports.VariableInfo = exports.SymbolInfo = exports.DocSymbol = exports.SymbolUtils = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
class SymbolUtils {
    constructor(workspace) {
        this.workspace = workspace;
        this.leadingSymbolPattern = /([\w_][\w_-]*)$/gi;
        this.followingSymbolPattern = /^([\w_-]+)/gi;
    }
    getSymbolInfo(document, position) {
        const fileInfo = this.workspace.getFileInfo(document);
        if (!fileInfo) {
            throw new Error(`Pddl file not found in the PDDL workspace model: ` + document.uri.toString());
        }
        const domainInfo = this.workspace.pddlWorkspace.asDomain(fileInfo);
        if (!domainInfo) {
            return undefined;
        }
        const symbolAtPosition = this.getSymbolAtPosition(document, position);
        if (!symbolAtPosition) {
            return undefined;
        }
        const symbol = symbolAtPosition;
        if (symbol.isPrefixedBy('(')) {
            const symbolName = symbol.name.toLowerCase();
            const predicateFound = domainInfo.getPredicates().find(p => p.matchesShortNameCaseInsensitive(symbol.name));
            if (predicateFound) {
                return new VariableInfo(this.createHover(symbol.range, 'Predicate', this.brackets(predicateFound.declaredName), predicateFound.getDocumentation()), new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(predicateFound.getLocation())), predicateFound);
            }
            const functionFound = domainInfo.getFunctions().find(f => f.matchesShortNameCaseInsensitive(symbol.name));
            if (functionFound) {
                return new VariableInfo(this.createHover(symbol.range, 'Function', this.brackets(functionFound.declaredName), functionFound.getDocumentation()), new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(functionFound.getLocation())), functionFound);
            }
            const derivedFound = domainInfo.getDerived().find(d => d.matchesShortNameCaseInsensitive(symbol.name));
            if (derivedFound) {
                return new VariableInfo(this.createHover(symbol.range, 'Derived predicate/function', this.brackets(derivedFound.declaredName), derivedFound.getDocumentation()), new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(derivedFound.getLocation())), derivedFound);
            }
            const actionFound = domainInfo.getActions().find(a => { var _a; return ((_a = a.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === symbolName; });
            if (actionFound) {
                return new ActionInfo(this.createActionHover(symbol.range, actionFound), new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(actionFound.getLocation())), actionFound);
            }
        }
        else if (symbol.isPrefixedBy('- ') || domainInfo.getTypes().includes(symbol.name)) {
            if (domainInfo.getTypes().includes(symbol.name)) {
                const parents = domainInfo.getTypeInheritance().getVerticesWithEdgesFrom(symbol.name);
                const inheritsFromText = parents && parents.length > 0 ? "Inherits from: " + parents.join(', ') : "";
                const typeLocation = domainInfo.getTypeLocation(symbol.name);
                if (!typeLocation) {
                    return undefined;
                }
                return new TypeInfo(this.createHover(symbol.range, 'Type', symbol.name, [inheritsFromText]), new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(typeLocation)), symbol.name);
            }
        }
        else if (symbol.isPrefixedBy('?')) {
            if (fileInfo.isDomain()) {
                const parameterNode = domainInfo.syntaxTree.getNodeAt(document.offsetAt(symbol.range.start));
                const scopeNode = parameterNode.findParametrisableScope(symbol.name);
                if (scopeNode) {
                    const indexOfParamDeclaration = scopeNode ?
                        scopeNode.getText().indexOf('?' + symbol.name) :
                        parameterNode.getStart();
                    return new ParameterInfo(this.createHover(symbol.range, 'Parameter', parameterNode.getToken().tokenText, []), new vscode_1.Location(document.uri, document.positionAt(indexOfParamDeclaration)), scopeNode, symbol.name);
                }
            }
        }
        // we return an answer only if we find something
        // otherwise no hover information is given
        return undefined;
    }
    getWordAtDocumentPosition(document, position) {
        // find the word at the position leveraging the TextDocument facility
        const wordRange = document.getWordRangeAtPosition(position, /\w[-\w]*/);
        if (!wordRange || wordRange.isEmpty || !wordRange.isSingleLine) {
            return undefined;
        }
        const word = document.getText(wordRange);
        const lineIdx = wordRange.start.line;
        const before = document.getText(new vscode_1.Range(new vscode_1.Position(lineIdx, 0), wordRange.start));
        const after = document.getText(new vscode_1.Range(wordRange.end, new vscode_1.Position(lineIdx, Number.MAX_SAFE_INTEGER)));
        const line = document.getText(new vscode_1.Range(lineIdx, 0, lineIdx, Number.MAX_SAFE_INTEGER));
        return { before: before, word: word, after: after, range: wordRange, line: line };
    }
    getWordAtTextPosition(text, position) {
        const line = text.split('\n')[position.line];
        const leadingText = line.substring(0, position.character);
        const followingText = line.substring(position.character - 1);
        this.leadingSymbolPattern.lastIndex = 0;
        let match = this.leadingSymbolPattern.exec(leadingText); //todo: this pattern does not match, if the word was selected
        if (!match) {
            return undefined;
        }
        const leadingSymbolPart = match[1];
        this.followingSymbolPattern.lastIndex = 0;
        match = this.followingSymbolPattern.exec(followingText);
        if (!match) {
            return undefined;
        }
        const followingSymbolPart = match[1];
        const symbolName = leadingSymbolPart + followingSymbolPart.substr(1);
        const range = new vscode_1.Range(position.line, position.character - leadingSymbolPart.length, position.line, position.character + followingSymbolPart.length - 1);
        return {
            before: line.substring(0, range.start.character),
            word: symbolName,
            after: line.substring(range.end.character + 1),
            line: line,
            range: range
        };
    }
    getSymbolAtPosition(document, position) {
        const wordContext = this.getWordAtDocumentPosition(document, position);
        // is the position not a word, or within comments?
        if (wordContext === undefined || wordContext.before.includes(';')) {
            return undefined;
        }
        return new DocSymbol(wordContext.word, wordContext.range, wordContext.line);
    }
    createHover(range, title, symbolName, documentation) {
        const markdownString = this.createSymbolMarkdownDocumentation(title, symbolName, documentation);
        return new vscode_1.Hover(markdownString, range);
    }
    createSymbolMarkdownDocumentation(title, symbolName, documentation) {
        const markdownString = new vscode_1.MarkdownString(title ? `**${title}**` : undefined);
        markdownString.appendCodeblock(symbolName, 'pddl');
        documentation.forEach(d => markdownString.appendText(END_LINE + END_LINE).appendMarkdown(d));
        return markdownString;
    }
    createActionHover(range, action) {
        var _a;
        let label = 'Action';
        if (action.isDurative()) {
            label = 'Durative ' + label;
        }
        let doc = new vscode_1.MarkdownString(`**${label}**`)
            .appendCodeblock((_a = action.name) !== null && _a !== void 0 ? _a : "", 'pddl');
        if (action.parameters.length) {
            doc = doc.appendMarkdown('Parameters:' + END_LINE + END_LINE);
            action.parameters.forEach(p => doc.appendMarkdown('* `' + p.toPddlString() + '`' + END_LINE));
        }
        action.getDocumentation().forEach(d => doc.appendText(END_LINE + END_LINE).appendMarkdown(d));
        return new vscode_1.Hover(doc, range);
    }
    findSymbolReferences(document, symbol, includeDeclaration) {
        const fileInfo = this.workspace.getFileInfo(document);
        if (!fileInfo) {
            console.log(`File ${document.uri.toString()} not know to the PDDL workspace.`);
            return undefined;
        }
        const domainInfoFound = this.workspace.pddlWorkspace.asDomain(fileInfo);
        if (!domainInfoFound) {
            return undefined;
        }
        const domainInfo = domainInfoFound;
        const problemFiles = this.workspace.pddlWorkspace.getProblemFiles(domainInfo);
        let locations = [];
        let includeReference = includeDeclaration;
        if (symbol instanceof VariableInfo) {
            // add variable references found in the domain file
            domainInfo.getVariableReferences(symbol.variable).forEach(range => {
                if (includeReference) {
                    locations.push(new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), utils_1.toRange(range)));
                }
                else {
                    // we skipped the declaration, but let's include any further references
                    includeReference = true;
                }
            });
            // add variable references found in all problem files
            problemFiles.forEach(p => p.getVariableReferences(symbol.variable)
                .forEach(range => locations.push(new vscode_1.Location(utils_1.toUri(p.fileUri), utils_1.toRange(range)))));
        }
        else if (symbol instanceof TypeInfo) {
            // add type references found in the domain file
            if (includeDeclaration) {
                locations.push(symbol.location);
            }
            const typeName = symbol.type;
            domainInfo.getTypeReferences(typeName).forEach(range => {
                const vsRange = utils_1.toRange(range);
                if (!vsRange.isEqual(symbol.location.range)) {
                    locations.push(new vscode_1.Location(utils_1.toUri(domainInfo.fileUri), vsRange));
                }
            });
            // add type references found in all problem files
            problemFiles.forEach(p => p.getTypeReferences(typeName)
                .forEach(range => locations.push(new vscode_1.Location(utils_1.toUri(p.fileUri), utils_1.toRange(range)))));
        }
        else if (symbol instanceof ParameterInfo) {
            const parameterInfo = symbol;
            parameterInfo.scopeNode.getChildrenRecursively(node => node.isType(pddl_workspace_1.parser.PddlTokenType.Parameter) && node.getToken().tokenText === '?' + symbol.name, node => locations.push(new vscode_1.Location(document.uri, utils_1.nodeToRange(document, node))));
            if (!includeDeclaration) {
                locations = locations.slice(1);
            }
        }
        return locations;
    }
    async assertFileParsed(document) {
        if (!this.workspace.getFileInfo(document)) {
            await this.workspace.upsertAndParseFile(document);
        }
    }
    static toLocation(document, pddlRange) {
        return new vscode_1.Location(document.uri, utils_1.toRange(pddlRange));
    }
    brackets(symbolName) {
        return `(${symbolName})`;
    }
}
exports.SymbolUtils = SymbolUtils;
class DocSymbol {
    constructor(name, range, line) {
        this.name = name;
        this.range = range;
        this.line = line;
    }
    isPrefixedBy(prefix) {
        return this.line.substring(0, this.range.start.character).endsWith(prefix);
    }
}
exports.DocSymbol = DocSymbol;
class SymbolInfo {
    /**
     * Creates symbol information.
     * @param hover hover info
     * @param location location of the symbol's declaration
     */
    constructor(hover, location) {
        this.hover = hover;
        this.location = location;
    }
}
exports.SymbolInfo = SymbolInfo;
class VariableInfo extends SymbolInfo {
    constructor(hover, location, variable) {
        super(hover, location);
        this.variable = variable;
    }
}
exports.VariableInfo = VariableInfo;
class TypeInfo extends SymbolInfo {
    constructor(hover, location, type) {
        super(hover, location);
        this.type = type;
    }
}
exports.TypeInfo = TypeInfo;
class ActionInfo extends SymbolInfo {
    constructor(hover, location, action) {
        super(hover, location);
        this.action = action;
    }
}
exports.ActionInfo = ActionInfo;
class ParameterInfo extends SymbolInfo {
    constructor(hover, location, scopeNode, name) {
        super(hover, location);
        this.scopeNode = scopeNode;
        this.name = name;
    }
}
exports.ParameterInfo = ParameterInfo;
const END_LINE = `
`;
//# sourceMappingURL=SymbolUtils.js.map