/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.VariableType = exports.UndeclaredVariable = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
class UndeclaredVariable {
    constructor(fileInfo) {
        this.syntaxTree = new pddl_workspace_1.parser.PddlSyntaxTreeBuilder(fileInfo.getText()).getTree();
    }
    getVariable(diagnostic, document) {
        const match = UndeclaredVariable.undeclaredVariableDiagnosticPattern.exec(diagnostic.message);
        if (!match) {
            return undefined;
        }
        const variableName = match[1];
        const lineWithUndeclaredVariable = document.lineAt(diagnostic.range.start.line);
        const variableNameMatch = lineWithUndeclaredVariable.text.match(new RegExp("\\(\\s*" + variableName + "[ |\\)]", "i"));
        if (variableNameMatch === null || variableNameMatch.index === undefined) {
            return undefined;
        }
        const undeclaredVariableOffset = document.offsetAt(lineWithUndeclaredVariable.range.start) +
            variableNameMatch.index +
            variableNameMatch[0].toLowerCase().indexOf(variableName);
        const variableUsage = this.syntaxTree.getNodeAt(undeclaredVariableOffset + 1).expand();
        if (variableUsage.isDocument()) {
            console.log("Undeclared predicate/function was not found: " + variableName);
            return undefined;
        }
        const parameterNames = variableUsage.getNestedChildren()
            .filter(node => node.isType(pddl_workspace_1.parser.PddlTokenType.Parameter))
            .map(node => node.getText().replace('?', ''));
        const parameters = parameterNames.map(param => this.findParameterDefinition(variableUsage, param));
        if (parameters.some(p => !p)) {
            console.log("Undeclared predicate/function has some unexpected parameters: " + variableName);
            return undefined;
        }
        const validParameters = parameters.map(p => p);
        return [new pddl_workspace_2.Variable(variableName, validParameters), variableUsage];
    }
    findParameterDefinition(variableUsage, parameterName) {
        const scope = variableUsage.findParametrisableScope(parameterName);
        const parameterDefinitionNode = scope && scope.getParameterDefinition();
        return parameterDefinitionNode &&
            pddl_workspace_1.parser.parseParameters(parameterDefinitionNode.getText())
                .find(p => p.name.toLowerCase() === parameterName.toLowerCase());
    }
    createEdit(document, variable, node) {
        let type = VariableType.Undecided;
        while (type === VariableType.Undecided && !node.isDocument()) {
            node = node.getParent();
            if (node.isType(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator)) {
                switch (node.getToken().tokenText) {
                    case "(+":
                    case "(-":
                    case "(/":
                    case "(*":
                    case "(<":
                    case "(<=":
                    case "(>":
                    case "(>=":
                    case "(=":
                    case "(assign":
                    case "(increase":
                    case "(decrease":
                    case "(scale-up":
                    case "(scale-down":
                    case "(sumall":
                        type = VariableType.Function;
                        break;
                    case "(and":
                    case "(not":
                    case "(or":
                    case "(at start":
                    case "(over all":
                    case "(at end":
                    case "(forall":
                        type = VariableType.Predicate;
                        break;
                }
            }
        }
        let newSectionName;
        switch (type) {
            case VariableType.Function:
                newSectionName = pddl_workspace_1.parser.PddlStructure.FUNCTIONS;
                break;
            case VariableType.Predicate:
                newSectionName = pddl_workspace_1.parser.PddlStructure.PREDICATES;
                break;
            default:
                console.log(`Could not determine whether ${variable.getFullName()} inside ${node.getToken().tokenText} is a predicate or a function.`);
                return [undefined, type];
        }
        const defineNode = this.syntaxTree.getDefineNode();
        const sectionNode = defineNode.getFirstOpenBracket(newSectionName);
        const edit = new vscode_1.WorkspaceEdit();
        const indent1 = UndeclaredVariable.createIndent(document, 1);
        const indent2 = UndeclaredVariable.createIndent(document, 2);
        const eol = UndeclaredVariable.createEolString(document);
        if (sectionNode) {
            edit.insert(document.uri, document.positionAt(sectionNode.getEnd() - 1), indent1 + `(${variable.getFullName()})` + eol);
        }
        else {
            const previousSectionNode = pddl_workspace_1.parser.PddlStructure.findPrecedingSection(newSectionName, defineNode, pddl_workspace_1.parser.PddlStructure.PDDL_DOMAIN_SECTIONS);
            edit.insert(document.uri, document.positionAt(previousSectionNode.getEnd()), eol + indent1 + `(${newSectionName}${eol + indent2}(${variable.getFullName()})${eol + indent1})`);
        }
        return [edit, type];
    }
    static createEolString(document) {
        return document.eol === vscode_1.EndOfLine.CRLF ? '\r\n' : '\n';
    }
    static createIndent(document, indentLevel) {
        const config = vscode_1.workspace.getConfiguration('editor', document.uri);
        let indent;
        if (config.get('insertSpaces')) {
            const tabSize = config.get('tabSize', 4);
            indent = ' '.repeat(tabSize * indentLevel);
        }
        else {
            indent = '\t'.repeat(indentLevel);
        }
        return indent;
    }
}
exports.UndeclaredVariable = UndeclaredVariable;
UndeclaredVariable.undeclaredVariableDiagnosticPattern = /^Undeclared symbol\s*:\s*([\w-]+)\s*/i;
var VariableType;
(function (VariableType) {
    VariableType[VariableType["Predicate"] = 0] = "Predicate";
    VariableType[VariableType["Function"] = 1] = "Function";
    VariableType[VariableType["Undecided"] = 2] = "Undecided";
})(VariableType = exports.VariableType || (exports.VariableType = {}));
//# sourceMappingURL=UndeclaredVariable.js.map