/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssociationProvider = exports.COMMAND_ASSOCIATE_DOMAIN = exports.COMMAND_ASSOCIATE_PROBLEM = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const pddl_workspace_3 = require("pddl-workspace");
const AssociationCodeActionProvider_1 = require("./AssociationCodeActionProvider");
const utils_1 = require("../utils");
const workspaceUtils_1 = require("./workspaceUtils");
const pddl_workspace_4 = require("pddl-workspace");
exports.COMMAND_ASSOCIATE_PROBLEM = 'pddl.workspace.associateProblem';
exports.COMMAND_ASSOCIATE_DOMAIN = 'pddl.workspace.associateDomain';
/**
 * Provides associations between workspace files that cannot be associated by natural links.
 */
class AssociationProvider {
    constructor(context, codePddlWorkspace) {
        this.codePddlWorkspace = codePddlWorkspace;
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.COMMAND_ASSOCIATE_PROBLEM, (planUri) => this.associateProblem(planUri).catch(utils_1.showError)));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(exports.COMMAND_ASSOCIATE_DOMAIN, async (problemUri) => {
            const problemDocument = await vscode_1.workspace.openTextDocument(problemUri);
            const problemFileInfo = codePddlWorkspace.getFileInfo(problemDocument);
            this.associateDomain(problemUri, codePddlWorkspace.pddlWorkspace.getDomainFilesFor(problemFileInfo)).catch(utils_1.showError);
        }));
        context.subscriptions.push(vscode_1.languages.registerCodeActionsProvider(pddl_workspace_1.PDDL, new AssociationCodeActionProvider_1.AssociationCodeActionProvider(), { providedCodeActionKinds: [vscode_1.CodeActionKind.QuickFix] }));
        context.subscriptions.push(vscode_1.languages.registerCodeActionsProvider(pddl_workspace_1.PLAN, new AssociationCodeActionProvider_1.AssociationCodeActionProvider(), { providedCodeActionKinds: [vscode_1.CodeActionKind.QuickFix] }));
        context.subscriptions.push(vscode_1.languages.registerCodeActionsProvider(pddl_workspace_1.HAPPENINGS, new AssociationCodeActionProvider_1.AssociationCodeActionProvider(), { providedCodeActionKinds: [vscode_1.CodeActionKind.QuickFix] }));
    }
    async associateProblem(planUri) {
        const problemUri = await workspaceUtils_1.selectFile({
            language: pddl_workspace_4.PddlLanguage.PDDL,
            promptMessage: 'Select the matching problem file...',
            findPattern: AssociationProvider.PDDL_PATTERN,
            fileOpenLabel: 'Select',
            fileOpenFilters: { 'PDDL Problem Files': ['pddl'] },
            workspaceFolder: vscode_1.workspace.getWorkspaceFolder(planUri)
        });
        if (problemUri) {
            const problemDocument = await vscode_1.workspace.openTextDocument(problemUri);
            //window.showTextDocument(problemDocument);
            await this.associatePlanToProblem(planUri, problemDocument);
        }
    }
    async associatePlanToProblem(planUri, problemDocument) {
        const parsedFileInfo = await this.codePddlWorkspace.upsertAndParseFile(problemDocument);
        if (!(parsedFileInfo instanceof pddl_workspace_2.ProblemInfo)) {
            throw new Error("Selected file is not a problem file.");
        }
        const problemInfo = parsedFileInfo;
        this.codePddlWorkspace.pddlWorkspace.associatePlanToProblem(utils_1.toURI(planUri), problemInfo);
        console.log(`Associated ${problemDocument.uri} to ${planUri}.`);
        // re-validate the plan file
        const planInfo = this.codePddlWorkspace.getFileInfoByUri(planUri);
        if (planInfo !== undefined) {
            this.codePddlWorkspace.pddlWorkspace.invalidateDiagnostics(planInfo);
        }
        else {
            console.log("Plan not found in the workspace model: " + planUri);
        }
    }
    async associateDomain(problemUri, suggestedFiles) {
        const domainUri = await workspaceUtils_1.selectFile({
            language: pddl_workspace_4.PddlLanguage.PDDL,
            promptMessage: 'Select the matching domain file...',
            findPattern: AssociationProvider.PDDL_PATTERN,
            fileOpenLabel: 'Select',
            fileOpenFilters: { 'PDDL Domain Files': ['pddl'] },
            workspaceFolder: vscode_1.workspace.getWorkspaceFolder(problemUri)
        }, suggestedFiles);
        if (domainUri) {
            const domainDocument = await vscode_1.workspace.openTextDocument(domainUri);
            await this.associateDomainToProblem(problemUri, domainDocument);
            return domainDocument;
        }
        else {
            return undefined;
        }
    }
    async associateDomainToProblem(problemUri, domainDocument) {
        const parsedFileInfo = await this.codePddlWorkspace.upsertAndParseFile(domainDocument);
        if (!(parsedFileInfo instanceof pddl_workspace_3.DomainInfo)) {
            throw new Error("Selected file is not a domain file.");
        }
        const domainInfo = parsedFileInfo;
        const problemInfo = this.codePddlWorkspace.getFileInfoByUri(problemUri);
        if (problemInfo === undefined) {
            throw new Error(`Problem file ${problemUri} not found in the workspace model.`);
        }
        this.codePddlWorkspace.pddlWorkspace.associateProblemToDomain(problemInfo, domainInfo);
        console.log(`Associated ${domainDocument.uri} to ${problemUri}.`);
        // re-validate the problem file
        const pddlWorkspace = this.codePddlWorkspace.pddlWorkspace;
        pddlWorkspace.invalidateDiagnostics(problemInfo);
        pddlWorkspace.getPlanFiles(problemInfo).forEach(planInfo => pddlWorkspace.invalidateDiagnostics(planInfo));
        pddlWorkspace.getHappeningsFiles(problemInfo).forEach(happeningsInfo => pddlWorkspace.invalidateDiagnostics(happeningsInfo));
    }
}
exports.AssociationProvider = AssociationProvider;
AssociationProvider.PDDL_PATTERN = '**/*.pddl';
//# sourceMappingURL=AssociationProvider.js.map