/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodePddlWorkspace = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const workspaceUtils_1 = require("./workspaceUtils");
const CodeDocumentPositionResolver_1 = require("./CodeDocumentPositionResolver");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
class CodePddlWorkspace {
    /**
     * VS Code wrapper for the PDDL workspace model
     * @param pddlWorkspace underlying PDDL workspace model
     * @param context Code extension context. When not supplied, the instance is for testing only.
     * @param pddlConfiguration pddl extension configuration. When not supplied , the instance is for testing only.
     */
    constructor(pddlWorkspace, context, pddlConfiguration) {
        this.pddlWorkspace = pddlWorkspace;
        this.context = context;
        if (context) { // unit tests do not clean-up
            subscribeToWorkspace(this, context, pddlConfiguration);
        }
    }
    registerCommands() {
        const revealActionCommand = vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand('pddl.revealAction', (domainFileUri, actionName) => {
            revealAction(this.pddlWorkspace.getFileInfo(utils_1.toURI(domainFileUri)), actionName).catch(utils_1.showError);
        });
        if (this.context) {
            this.context.subscriptions.push(revealActionCommand);
        }
        return this;
    }
    static getInstance(pddlWorkspace, context, pddlConfiguration) {
        return new CodePddlWorkspace(pddlWorkspace, context, pddlConfiguration).registerCommands();
    }
    static getInstanceForTestingOnly(pddlWorkspace) {
        return new CodePddlWorkspace(pddlWorkspace);
    }
    async upsertFile(document, force = false) {
        const language = workspaceUtils_1.toLanguage(document);
        if (language === undefined) {
            return undefined;
        }
        return await this.pddlWorkspace.upsertFile(utils_1.toURI(document.uri), language, document.version, document.getText(), this.createPositionResolver(document), force);
    }
    async upsertAndParseFile(document) {
        const language = workspaceUtils_1.toLanguage(document);
        if (language === undefined) {
            return undefined;
        }
        return this.pddlWorkspace.upsertAndParseFile(utils_1.toURI(document.uri), language, document.version, document.getText(), this.createPositionResolver(document));
    }
    getFileInfoByUri(uri) {
        return this.pddlWorkspace.getFileInfo(utils_1.toURI(uri));
    }
    getFileInfo(document) {
        return this.getFileInfoByUri(document.uri);
    }
    async removeFile(textDoc) {
        const fileExists = await pddl_workspace_1.utils.afs.exists(textDoc.fileName);
        return this.pddlWorkspace.removeFile(utils_1.toURI(textDoc.uri), { removeAllReferences: !fileExists });
    }
    getDomainFilesFor(problemFileInfo) {
        const domainFiles = this.pddlWorkspace.getDomainFilesFor(problemFileInfo);
        return domainFiles
            .filter(domainInfo => this.isRealFile(domainInfo));
    }
    isRealFile(domainInfo) {
        return domainInfo.fileUri.scheme !== CodePddlWorkspace.GIT_SCHEME;
    }
    static isRealDocument(document) {
        return document.uri.scheme !== this.GIT_SCHEME;
    }
    setEpsilon(epsilon) {
        this.pddlWorkspace.epsilon = epsilon;
    }
    createPositionResolver(document) {
        return new CodeDocumentPositionResolver_1.CodeDocumentPositionResolver(document);
    }
}
exports.CodePddlWorkspace = CodePddlWorkspace;
CodePddlWorkspace.GIT_SCHEME = "git";
function subscribeToWorkspace(pddlWorkspace, context, pddlConfiguration) {
    // add all open documents
    vscode_1.workspace.textDocuments
        .filter(textDoc => workspaceUtils_1.isAnyPddl(textDoc))
        .filter(textDoc => CodePddlWorkspace.isRealDocument(textDoc))
        .forEach(textDoc => {
        pddlWorkspace.upsertFile(textDoc);
    });
    // subscribe to document opening event
    context.subscriptions.push(vscode_1.workspace.onDidOpenTextDocument(textDoc => {
        if (workspaceUtils_1.isAnyPddl(textDoc) && CodePddlWorkspace.isRealDocument(textDoc)) {
            pddlWorkspace.upsertFile(textDoc);
        }
    }));
    // subscribe to document changing event
    context.subscriptions.push(vscode_1.workspace.onDidChangeTextDocument(docEvent => {
        if (workspaceUtils_1.isAnyPddl(docEvent.document) && CodePddlWorkspace.isRealDocument(docEvent.document)) {
            pddlWorkspace.upsertFile(docEvent.document);
        }
        else {
            // for all problem files that pre-parse using data from this updated document, re-validate
            pddlWorkspace.pddlWorkspace.getAllFilesIf(f => f.isProblem())
                .filter(problemInfo => !!problemInfo.getPreParsingPreProcessor())
                .filter(problemInfo => { var _a; return (_a = problemInfo.getPreParsingPreProcessor()) === null || _a === void 0 ? void 0 : _a.getInputFiles().some(inputFile => docEvent.document.fileName.endsWith(inputFile)); })
                .forEach(async (problemInfo) => {
                const problemFile = await vscode_1.workspace.openTextDocument(utils_1.toUri(problemInfo.fileUri));
                pddlWorkspace.upsertFile(problemFile, true);
            });
        }
    }));
    // subscribe to document deletion event
    context.subscriptions.push(vscode_1.workspace.onDidDeleteFiles(deletionEvent => {
        const deletionSuccesses = deletionEvent.files
            .map(deletedUri => pddlWorkspace.pddlWorkspace.removeFile(utils_1.toURI(deletedUri), { removeAllReferences: false }));
        if (deletionSuccesses.some(s => !s)) {
            console.error(`Some files were not removed from the PDDL repository model`);
        }
    }));
    if (pddlConfiguration) {
        vscode_1.workspace.onDidChangeConfiguration(() => {
            pddlWorkspace.setEpsilon(pddlConfiguration.getEpsilonTimeStep());
        });
    }
}
async function revealAction(domainInfo, actionName) {
    const document = await vscode_1.workspace.openTextDocument(utils_1.toUri(domainInfo.fileUri));
    const actionFound = domainInfo.getActions().find(a => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === actionName.toLowerCase(); });
    const actionRange = actionFound && utils_1.toRange(actionFound.getLocation());
    const openEditor = vscode_1.window.visibleTextEditors.find(e => e.document.uri.toString() === document.uri.toString());
    if (openEditor && actionRange) {
        openEditor.revealRange(actionRange, vscode_1.TextEditorRevealType.AtTop);
    }
    else {
        vscode_1.window.showTextDocument(document.uri, { preserveFocus: true, selection: actionRange, preview: true });
    }
}
//# sourceMappingURL=CodePddlWorkspace.js.map