/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.selectFile = exports.selectHappenings = exports.getDomainAndProblemForHappenings = exports.NoDomainAssociated = exports.NoProblemAssociated = exports.getDomainAndProblemForPlan = exports.getDomainFileForProblem = exports.toLanguage = exports.isHappenings = exports.isPlan = exports.isPddl = exports.isAnyPddl = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const pddl_workspace_3 = require("pddl-workspace");
const path_1 = require("path");
const utils_1 = require("../utils");
function isAnyPddl(doc) {
    return isPddl(doc) || isPlan(doc) || isHappenings(doc);
}
exports.isAnyPddl = isAnyPddl;
function isPddl(doc) {
    return doc.languageId === pddl_workspace_1.PDDL && doc.uri.scheme !== "git";
}
exports.isPddl = isPddl;
function isPlan(doc) {
    return doc.languageId === pddl_workspace_1.PLAN && doc.uri.scheme !== "git";
}
exports.isPlan = isPlan;
function isHappenings(doc) {
    return doc.languageId === pddl_workspace_1.HAPPENINGS && doc.uri.scheme !== "git";
}
exports.isHappenings = isHappenings;
function toLanguage(doc) {
    return pddl_workspace_1.toLanguageFromId(doc.languageId);
}
exports.toLanguage = toLanguage;
function getDomainFileForProblem(problemFile, pddlWorkspace) {
    // find domain files in the same folder that match the problem's domain name
    const domainFiles = pddlWorkspace.getDomainFilesFor(problemFile);
    if (domainFiles.length > 1) {
        const candidates = domainFiles.map(d => pddl_workspace_3.PddlWorkspace.getFileName(d.fileUri)).join(', ');
        const message = `There are multiple candidate domains with name ${problemFile.domainName}: ${candidates}. Click 💡 to select it...`;
        throw new NoDomainAssociated(problemFile, message);
    }
    else if (domainFiles.length === 0) {
        const message = `There are no domains open in the same folder with name (domain '${problemFile.domainName}') open in the editor. Click 💡 to select it...`;
        throw new NoDomainAssociated(problemFile, message);
    }
    else {
        return domainFiles[0];
    }
}
exports.getDomainFileForProblem = getDomainFileForProblem;
function getDomainAndProblemForPlan(planInfo, pddlWorkspace) {
    const problemFileInfo = pddlWorkspace.getProblemFileForPlan(planInfo);
    if (!problemFileInfo) {
        throw new NoProblemAssociated(planInfo);
    }
    const domainFileInfo = pddlWorkspace.getDomainFileFor(problemFileInfo);
    if (!domainFileInfo) {
        throw new NoDomainAssociated(problemFileInfo);
    }
    return { problem: problemFileInfo, domain: domainFileInfo };
}
exports.getDomainAndProblemForPlan = getDomainAndProblemForPlan;
class NoProblemAssociated extends Error {
    constructor(plan) {
        super(NoProblemAssociated.getMessage(plan));
        this.plan = plan;
    }
    static getMessage(plan) {
        if (plan.problemName !== pddl_workspace_1.parser.UNSPECIFIED_PROBLEM) {
            return `No problem file with name '(problem ${plan.problemName})' and located in the same folder as the plan is open in the editor. Click 💡 to select it...`;
        }
        else {
            return `No problem file is associated with the plan file. Click 💡 to select it...`;
        }
    }
}
exports.NoProblemAssociated = NoProblemAssociated;
NoProblemAssociated.DIAGNOSTIC_CODE = "NoAssociatedProblem";
class NoDomainAssociated extends Error {
    constructor(problem, message) {
        super(message !== null && message !== void 0 ? message : NoDomainAssociated.getMessage(problem));
        this.problem = problem;
    }
    static getMessage(problem) {
        if (problem.domainName !== pddl_workspace_1.parser.UNSPECIFIED_DOMAIN) {
            return `No domain file with name '(define ${problem.domainName})' and located in the same folder is open in the editor. Click 💡 to select it...`;
        }
        else {
            return `No domain file is associated with this problem file. Click 💡 to select it...`;
        }
    }
}
exports.NoDomainAssociated = NoDomainAssociated;
NoDomainAssociated.DIAGNOSTIC_CODE = "NoAssociatedDomain";
function getDomainAndProblemForHappenings(happeningsInfo, pddlWorkspace) {
    const problemFileInfo = pddlWorkspace.getProblemFileForHappenings(happeningsInfo);
    if (!problemFileInfo) {
        throw new Error(`No problem file with name '(problem ${happeningsInfo.problemName}') and located in the same folder as the plan is open in the editor.`);
    }
    const domainFileInfo = pddlWorkspace.getDomainFileFor(problemFileInfo);
    if (!domainFileInfo) {
        throw new Error(`No domain file corresponding to problem '${problemFileInfo.name}' and located in the same folder is open in the editor.`);
    }
    return { problem: problemFileInfo, domain: domainFileInfo };
}
exports.getDomainAndProblemForHappenings = getDomainAndProblemForHappenings;
async function selectHappenings() {
    // is a happenings file currently active?
    if (vscode_1.window.activeTextEditor && isHappenings(vscode_1.window.activeTextEditor.document)) {
        return vscode_1.window.activeTextEditor.document.uri.fsPath;
    }
    const workspaceFolder = vscode_1.window.activeTextEditor && vscode_1.window.activeTextEditor.document &&
        vscode_1.workspace.getWorkspaceFolder(vscode_1.window.activeTextEditor.document.uri);
    const happeningsUri = await selectFile({
        language: pddl_workspace_2.PddlLanguage.HAPPENINGS,
        promptMessage: 'Select happenings file to debug...',
        findPattern: '**/*.happenings',
        fileOpenLabel: 'Start debugging',
        fileOpenFilters: { 'PDDL Plan Happenings': ['happenings'] },
        workspaceFolder: workspaceFolder
    });
    if (happeningsUri) {
        vscode_1.window.showTextDocument(happeningsUri);
        return happeningsUri.fsPath;
    }
    else {
        return undefined;
    }
}
exports.selectHappenings = selectHappenings;
/**
 * Lets the user select a text document in the workspace.
 * @param options select file options
 * @param textDocuments text documents to pick from
 * @returns selected document, or null if the user canceled, or undefined, if the user wants to select another document from the disk
 */
async function selectTextDocument(options, textDocuments) {
    const openDocumentPicks = textDocuments.map(d => new TextDocumentQuickPickItem(d));
    const items = [...openDocumentPicks, anotherDocumentQuickPickItem];
    const selectedOpenDocument = await vscode_1.window.showQuickPick(items, { canPickMany: false, placeHolder: options.promptMessage });
    if (!selectedOpenDocument) {
        return null;
    }
    else if (selectedOpenDocument !== anotherDocumentQuickPickItem) {
        return selectedOpenDocument.getTextDocument();
    }
    else {
        return undefined; // the user selected that they want to continue to the wider list selection
    }
}
/**
 * Selects a document given the criteria in the _options_
 * @param options select file options
 * @param suggestedFiles files that the PddlWorkspace identified as candidates
 */
async function selectFile(options, suggestedFiles) {
    var _a;
    // 0. is one of the suggested text documents the right one?
    if (suggestedFiles && suggestedFiles.length) {
        const openFileInfoPicks = suggestedFiles.map(d => new SuggestedFileInfoQuickPickItem(d));
        const items = [...openFileInfoPicks, anotherDocumentFileInfoQuickPickItem];
        const selectedPick = await vscode_1.window.showQuickPick(items, { canPickMany: false, placeHolder: options.promptMessage });
        if (!selectedPick) {
            return undefined;
        }
        else if (selectedPick !== anotherDocumentFileInfoQuickPickItem) {
            return utils_1.toUri(selectedPick.getFileInfo().fileUri);
        }
    }
    // 1. is there a suitable file open in the editor?
    {
        const openDocs = vscode_1.workspace.textDocuments.filter(doc => toLanguage(doc) === options.language);
        const selectedTextDocument = await selectTextDocument(options, openDocs);
        if (selectedTextDocument) {
            return selectedTextDocument.uri;
        }
        else if (selectedTextDocument === null) {
            return undefined;
        }
    }
    // 2. else, select a file in the workspace
    {
        const workspaceFileUris = await vscode_1.workspace.findFiles(options.findPattern, '.git/**', 100);
        const workspaceFilePicks = workspaceFileUris.map(uri => new WorkspaceUriQuickPickItem(uri));
        const items = [...workspaceFilePicks, anotherLocalFileQuickPickItem];
        const workspaceFileUriPicked = await vscode_1.window.showQuickPick(items, { canPickMany: false, placeHolder: options.promptMessage });
        if (!workspaceFileUriPicked) {
            return undefined;
        }
        else if (workspaceFileUriPicked !== anotherLocalFileQuickPickItem) {
            return workspaceFileUriPicked.getUri();
        }
    }
    // 3. else select a file from the local disk
    {
        const defaultUri = (_a = options.workspaceFolder) === null || _a === void 0 ? void 0 : _a.uri;
        const selectedHappeningsUris = await vscode_1.window.showOpenDialog({
            defaultUri: defaultUri,
            canSelectFiles: true, canSelectFolders: false,
            canSelectMany: false,
            openLabel: options.fileOpenLabel,
            filters: options.fileOpenFilters
        });
        if (selectedHappeningsUris && selectedHappeningsUris.length > 0) {
            return selectedHappeningsUris[0];
        }
        else {
            // nothing was selected
            return undefined;
        }
    }
}
exports.selectFile = selectFile;
const anotherDocumentFileInfoQuickPickItem = {
    label: 'Select another file in the workspace...',
    alwaysShow: true,
    getFileInfo() {
        throw new Error('do not ask!');
    }
};
class SuggestedFileInfoQuickPickItem {
    constructor(fileInfo) {
        this.fileInfo = fileInfo;
        const filePath = fileInfo.fileUri.fsPath;
        this.label = path_1.basename(filePath);
        this.description = path_1.dirname(filePath);
    }
    getFileInfo() {
        return this.fileInfo;
    }
}
const anotherDocumentQuickPickItem = {
    label: 'Select another file in the workspace...',
    alwaysShow: true,
    getTextDocument() {
        throw new Error('do not ask!');
    }
};
class TextDocumentQuickPickItem {
    constructor(doc) {
        this.doc = doc;
        this.label = path_1.basename(doc.fileName);
        this.description = path_1.dirname(doc.fileName);
    }
    getTextDocument() {
        return this.doc;
    }
}
const anotherLocalFileQuickPickItem = {
    label: 'Select another file...',
    alwaysShow: true,
    getUri() {
        throw new Error('do not ask!');
    }
};
class WorkspaceUriQuickPickItem {
    constructor(uri) {
        this.uri = uri;
        this.label = path_1.basename(uri.fsPath);
        this.description = path_1.dirname(uri.fsPath);
    }
    getUri() {
        return this.uri;
    }
}
//# sourceMappingURL=workspaceUtils.js.map